#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

##############################################################################
=pod

=head1 db_layer.pm

db_layer.pm - subroutines to extract database calls from the scripts

=head1 DESCRIPTION

This library is used for all subroutines that extract database calls from the
scripts.

=head1 NOTES & BUGS

It will be possible to specify complex queries within this API.  Wherever a 
'WHERE' clause can be specified, the following information can be configured
to get the outcome that is desired:

$sql_hashref->{'TABLE'}->{'WHERE'} = \@WHERE_ARRAY
where WHERE_ARRAY is an arry of hashrefs, with the following keys:
  WHERE:
    'field'   
           * the name of the field to compare to
           * ( in the field  means that this is the start of a block 
	     for comparisons, surrounded by ( ).  The ) is found in the 'value'
             portion of some where clause element defined after this one.

    'value'
           * <tag><value>
           WHERE
           <tag>
                * % in it anywhere, then a LIKE comparison is done
                * ! and a % in it, then a NOT LIKE comparison is done
                * ! in it, then <> comparison is done
                * ? in it, then the value is simply ?, for use in multiple dbi calls
                * ) means that this is the end of a block for comparisons,
		  surrounded by ( ).  The ( is found in the 'field' definition
                  for some where clause element prior to this one.
                * >= do a >= comparison
                * <= do a <= comparison
                * > do a > comparison
                * < do a < comparison
                * none of the above, then an = comparison is done

           <value>
                * The actual value to compare to
    'and_or'
            * and if you want this portion of the query to be AND'd in the 
              where clause
            * or if you want this portion of the query to be OR'd in the 
              where clause
            * If you do NOT specify an operator, then AND is assumed.

To assign the values to the WHERE clause, you would use
$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", "!%-A", "and");

=cut
##############################################################################

##############################################################################
=pod

=head2 build_sql

build_sql - &build_sql($sql_hashref, $site, $dbh, $override)

=over 4

=item DESCRIPTION

Determine what type of SQL statement to create, call the appropriate routine to
generate it, and return the SQL statement (or undef, if the user does not have
permission to execute the SQL statement).

=item INPUT

$sql_hashref - Hashref describing the insert SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks 

=item OUTPUT

$query - Undef if the user does not have appropriate permissions, otherwise the
         query to execute.

=item NOTES

=back

=cut
###############################################################################
sub build_sql {
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($statement) = uc $sql_hashref->{'statement'};

    &debug("In build_sql for statement $statement", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($statement eq "SELECT") {
	return (&build_select($sql_hashref, $site, $dbh, $override));
    }
    elsif ($statement eq "UPDATE") {
	return (&build_update($sql_hashref, $site, $dbh, $override));
    }
    elsif ($statement eq "INSERT") {
	return (&build_insert($sql_hashref, $site, $dbh, $override));
    }
    elsif ($statement eq "DELETE") {
	return (&build_delete($sql_hashref, $site, $dbh, $override));
    }
    else {
	&debug("Cannot build SQL for statement: '$sql_hashref->{'statement'}'", 1, $site->{'debug_level'}, $site->{'debug_log'});
    }
    return;
}

##############################################################################
=pod

=head2 build_select

build_select -  &build_select($sql_hashref, $site, $dbh, $override);

=over 4

=item DESCRIPTION

Check user permissions for the desired SQL select statement.  If the user passes
permissions check, then Create a SQL select statement from the $sql_hashref.  
Return the SQL select statement, or undef if permissions are not met.

$sql_hashref is defined by (items with a * in front of them are optional):
    $sql_hashref->{'statement'} = "select";
    $sql_hashref->{'TABLE'}->{'NAME'} = name_of_table;  Name of table to do SQL on
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field; Fields to select
    $sql_hashref->{'TABLE'}->{'WHERE'} = Array where clause
         See NOTE for db_layer.pm to get exact syntax of WHERE clause
    $sql_hashref->{'TABLE'}->{'ORDER'} = field; Field to do ORDER BY on
    $sql_hashref->{'TABLE'}->{'DISTINCT'} = DISTINCT; If set, then do a SELECT distinct

=item INPUT

$sql_hashref - Hashref describing the select SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks 

=item OUTPUT

$select - SQL statement:<filter> if permissions permit
                If the results will need to be filtered after the select is performed,
                due to permission reasons, then "true_filter" will be appended to the
                end of the SQL select statement.  If the results do not need to be 
                filtered, then nothing is appended in the <filter> location.
          undef if the user does not have permissions to execute the statement.

=item NOTES

=back

=cut
###############################################################################
sub build_select
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($field, $perm_check, $select, $table_field_names, $comparison, $where_hash, $where, $while_indx);

    $select = "";

    if (!defined ($perm_check = &check_permissions($sql_hashref, $site, $dbh, $override))) {
	&debug("Insufficient permissions to build select statement", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    # start off an insert statement
    $select = "SELECT ";

    if (defined $sql_hashref->{'TABLE'}->{'DISTINCT'}) {
	$select .= "DISTINCT ";
    }

    foreach $field (keys %{$sql_hashref->{'TABLE'}->{'FIELDS'}})
    {
	if($sql_hashref->{'TABLE'}->{'FIELDS'}->{$field})
	{
	    $select .= "$field, ";  # fieldname
	}
    }

    $select =~ s/, $//gms;  # get rid of last , and space

    $select .= " FROM $sql_hashref->{'TABLE'}->{'NAME'} ";

    if (defined $sql_hashref->{'TABLE'}->{'WHERE'}) {
	$select .= "WHERE ";
	$while_indx = 0;
	while ($where_hash = $sql_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
		$comparison = &build_where_clause($where_hash->{'field'}, $where_hash->{'value'}, $where_hash->{'and_or'});
		$select .= $comparison;
	}
    }	


    $select =~ s/OR $//gms; # get rid of the last OR
    $select =~ s/AND $//gms; # get rid of the last AND

    if (defined $sql_hashref->{'TABLE'}->{'ORDER'}) {
	$select .= " ORDER BY $sql_hashref->{'TABLE'}->{'ORDER'}";

    }

    if ($perm_check eq "true_filter") {
	$select .= ":true_filter";
    }
    return $select;
}

##############################################################################
=pod

=head2 build_update

build_update -  &build_update($sql_hashref, $site, $dbh, $override);

=over 4

Check user permissions for the desired SQL update statement.  If the user passes
permissions check, then Create a SQL select statement from the $sql_hashref.  
Return the SQL select statement, or undef if permissions are not met.

mdate is automatically updated.

$sql_hashref is defined by (items with a * in front of them are optional):
    $sql_hashref->{'statement'} = "update";
    $sql_hashref->{'TABLE'}->{'NAME'} = name_of_table;  Name of table to do SQL on
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $value;   Set $field to $value
    $sql_hashref->{'TABLE'}->{'WHERE'} = Array where clause
         See NOTE for db_layer.pm to get exact syntax of WHERE clause

=item INPUT

$sql_hashref - Hashref describing the update SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks 

=item OUTPUT

$update - SQL statement if permissions permit
          undef if the user does not have permissions to execute the statement.

=item NOTES

=back

=cut
###############################################################################
sub build_update
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($field, $update, $q, $now, $comparison, $perm_check, $value, $while_indx);
    my ($where_hash, @where_array);
    &debug("build_update: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if (!defined ($perm_check = &check_permissions($sql_hashref, $site, $dbh, $override))) {
	&debug("Insufficient permissions to build update statement", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $update = $q = "";
    $now = time(); 

    # start off an update statement
    $q = "UPDATE $sql_hashref->{'TABLE'}->{'NAME'} SET ";
    my $keys = 0;


    foreach $field (keys %{$sql_hashref->{'TABLE'}->{'FIELDS'}})
    {
	$value = &string_todbi($sql_hashref->{'TABLE'}->{'FIELDS'}->{$field});
	$q .= "$field = '$value', " if ( $field !~ /mdate/ );
	$keys++;
    }
	    
    # the other half of the statement -- may be as simple as
    # 'objid = NNNNN'
	
    if($keys > 0)
    {
	$q .= " mdate = $now WHERE ";

	if (defined $sql_hashref->{'TABLE'}->{'WHERE'}) {
	    $while_indx = 0;
	    while ($where_hash = $sql_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
		&debug_hashref ($where_hash, 3, $site->{'debug_level'}, $site->{'debug_log'});
		$comparison = &build_where_clause($where_hash->{'field'}, $where_hash->{'value'}, $where_hash->{'and_or'});
		&debug ("build_update: Added $comparison", 3, $site->{'debug_level'}, $site->{'debug_log'});
		$q .= $comparison;
	    }
	    $q =~ s/OR $//gms; # get rid of the last OR
	    $q  =~ s/AND $//gms; # get rid of the last AND
	}	
    }

    $update = $q;
#    &debug ("build_update: Returning $update", 3, $site->{'debug_level'}, $site->{'debug_log'});
      
    return $update;
}

##############################################################################
=pod

=head2 build_insert

build_insert -  &build_insert($sql_hashref, $site, $dbh, $override);

=over 4

=item DESCRIPTION

This routine will take the definition of the insert statement, check user permissions,
and if the permission check passes, it will build an insert statement to be executed.  
It will generate an event, and get a valid objid back for the insert request.  Therefore,
it is no longer necessary to call an insert and select to levents from external scripts,
it is all done in here.  If you are trying to insert into extraprop, where you do not 
need to insert a new objid, then you would just fill in 
$sql_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} with it's objid, and then it will not
generate a new one for you.

cdate, mdate are automatically inserted.

Create a SQL insert statement from the $sql_hashref passed in.  $sql_hashref is
in the following format:
    $sql_hashref->{'TABLE'}->{'NAME'} = name_of_table;  Name of table to do SQL on
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $value; Set $field equal to $value

=item INPUT

$sql_hashref - Hashref describing the update SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks 

=item OUTPUT

$insert - SQL statement if permissions permit
          undef if the user does not have permissions to execute the statement.

=item NOTES

=back

=cut
###############################################################################
sub build_insert
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($insert, $value, $now, $field, $objid, $perm_check, $safe_value);

    if (!defined ($perm_check = &check_permissions($sql_hashref, $site, $dbh, $override))) {
	&debug("Insufficient permissions to build insert statement", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }
    $insert = "";
    $now = time();

    # start off an insert statement
    $insert = "INSERT INTO $sql_hashref->{'TABLE'}->{'NAME'} ( ";
    $value = "VALUES ( ";


    # make insert statement
    
    foreach $field (keys %{$sql_hashref->{'TABLE'}->{'FIELDS'}})
    {
	if($sql_hashref->{'TABLE'}->{'FIELDS'}->{$field})
	{
	    $insert .= "$field, ";  # fieldname
	    
	    # value
	    $safe_value = &string_todbi($sql_hashref->{'TABLE'}->{'FIELDS'}->{$field});
	    $value .= "'$safe_value', "; 
	}
    }
    
    $insert .= " cdate";
    $value .= " '$now'";

    # if we hanven't specified objid, then we need to create one, but NOT for clicks
    if (($insert !~ /objid/)  && ($sql_hashref->{'TABLE'}->{'NAME'} ne "clicks")) {
	$insert .= ", objid";
	$value .= ", '__objid__'";
    }

    $insert .= ") ";
    $value .= ") ";
    
    $insert .= $value;

    if ($value =~  /__objid__/) {
	$objid = &get_new_objid($site, $dbh, $sql_hashref->{'TABLE'}->{'NAME'}, $insert);
	if (defined $objid) {
	    $insert =~ s/__objid__/$objid/gms;
	    if (defined $sql_hashref->{'append_objid'}) {
		$insert .= "::$objid";  # append the $objid on the end for other routines to know what it is
	    }
	}
	else {
	    &debug ("Could not get new objid", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return;
	}
    }

#    &debug ("build_insert: Returning $insert", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $insert;
}

##############################################################################
=pod

=head2 build_delete

build_delete -  &build_delete($sql_hashref, $site, $dbh, $override);

=over 4

=item DESCRIPTION

This routine will take the definition of the delete statement, check user permissions,
and if the permission check passes, it will build a delete statement to be executed.  

Create a SQL delete statement from the $sql_hashref passed in.  $sql_hashref is
in the following format:
    $sql_hashref->{'TABLE'}->{'NAME'} = name_of_table;  Name of table to do SQL on
    $sql_hashref->{'TABLE'}->{'WHERE'} = Array where clause
         See NOTE for db_layer.pm to get exact syntax of WHERE clause

=item INPUT

$sql_hashref - Hashref describing the update SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks 

=item OUTPUT

$delete - SQL statement if permissions permit
          undef if the user does not have permissions to execute the statement.

=item NOTES

=back

=cut
###############################################################################
sub build_delete
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($field, $and_or, $comparison, $delete, $perm_check, $where_hash);
    my ($while_indx);

    if (!defined ($perm_check = &check_permissions($sql_hashref, $site, $dbh, $override))) {
	&debug("Insufficient permissions to build delete statement", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $delete = "";

    # make the delete statement
    $delete = "DELETE FROM $sql_hashref->{'TABLE'}->{'NAME'} ";

    if (defined $sql_hashref->{'TABLE'}->{'WHERE'}) {
	$delete .= "WHERE ";
	$while_indx = 0;
	while ($where_hash = $sql_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
	    $comparison = &build_where_clause($where_hash->{'field'}, $where_hash->{'value'}, $where_hash->{'and_or'});
	    $delete .= $comparison;
	}
    }

    $delete =~ s/OR $//gms; # get rid of the last OR
    $delete =~ s/AND $//gms; # get rid of the last AND
#    &debug("build_delete: Returning $delete.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $delete;
}

##############################################################################
=pod

=head2 build_select_from_update

build_select_from_update -  &build_select_from_update($sql_hashref, $site, $dbh);

=over 4

=item DESCRIPTION

Take a definition for an UPDATE statement, and build the equivalent SELECT statement.
This is used to select exactly what data will be effected by the update statement.

=item INPUT

$update_hashref - Hashref describing the update SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

=item OUTPUT

$q -- a string, the SQL select statement

=item NOTES

=back

=cut
###############################################################################
sub build_select_from_update
{
    my ($update_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $select_hashref, $field, @where_clause, $where_indx, $where_hashref);
    my ($while_indx);

    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = $update_hashref->{'TABLE'}->{'NAME'};
    foreach $field (keys %{$update_hashref->{'TABLE'}->{'FIELDS'}}) {
	$select_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;
    }

    $where_indx = 0;
    $while_indx = 0;
    while ($where_hashref = $update_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
    }

    $q = &build_select($select_hashref, $site, $dbh, 1);
#    &debug("build_select_from_update: Returning $q.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $q;
}

##############################################################################
=pod

=head2 build_select_from_delete

build_select_from_delete -  &build_select_from_delete($sql_hashref, $site, $dbh);    

=over 4

=item DESCRIPTION

Take a definition for an DELETE statement, and build the equivalent SELECT statement.
This is used to select exactly what data will be effected by the update statement.

=item INPUT

$delete_hashref - Hashref describing the delete SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

=item OUTPUT

$q -- a string, the SQL select statement

=item NOTES

=back

=cut
###############################################################################
sub build_select_from_delete
{
    my ($delete_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $select_hashref, $field, @where_clause, $where_hashref, $while_indx, $where_indx);

    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = $delete_hashref->{'TABLE'}->{'NAME'};
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";

    $where_indx = 0;
    $while_indx = 0;
    while ($where_hashref = $delete_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
    }

    $q = &build_select($select_hashref, $site, $dbh, 1);
#    &debug("build_select_from_delete: Returning $q.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $q;
}



##############################################################################
=pod

=head2 build_insert_from_update

build_insert_from_update -  &build_insert_from_update($sql_hashref, $site, $dbh);    

=over 4

=item DESCRIPTION

Take a definition for an UPDATE statement, and build the equivalent INSERT statement.  This is used for the cases where and UPDATE is attempted, and if it
fails, then an INSERT must be performed

=item INPUT

$sql_hashref - Hashref describing the update SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

=item OUTPUT

$q -- a string, the INSERT statemen

=item NOTES

=back

=cut
###############################################################################
sub build_insert_from_update
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $insert_hashref, $field, $value, $junk, @where_clause, $where_hashref);
    my ($while_indx);

    &debug("In build_insert_from_update", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $insert_hashref->{'statement'} = "INSERT";
    $insert_hashref->{'TABLE'}->{'NAME'} = $sql_hashref->{'TABLE'}->{'NAME'};
    foreach $field (keys %{$sql_hashref->{'TABLE'}->{'FIELDS'}}) {
	$insert_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field};
    }

    $while_indx = 0;
    while ($where_hashref = $sql_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
	$value =  $where_hashref->{'value'};
	$value =~ s/\%//gms;
	$value =~ s/\!//gms;
	$value =~ s/\?//gms;
	$insert_hashref->{'TABLE'}->{'FIELDS'}->{$where_hashref->{'field'}} = $value;
    }

    $q = &build_insert($insert_hashref, $site, $dbh, 1);
#    &debug("build_insert_from_update: Returning $q.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $q;
}

##############################################################################
=pod

=head2 build_where_clause

build_where_clause - &build_where_clause($field, $value, $next_and_or)

=over 4

=item DESCRIPTION

Taking one element from the $sql_hashref WHERE hash definition, and set up the
where clause element.  See the NOTE section of db_layer.pm to see the syntax of the
$value parameter.

=item INPUT

$field - Name of the field to do the comparison against

$value - The field value to use to create the where clause with

$and_or - The command to use to combine with the next where clause element (AND, OR).

=item OUTPUT

$where_clause - One where clause comparison to add to the SQL query

=item NOTES

=back

=cut
###############################################################################
sub build_where_clause {
    my ($field) = shift;
    my ($value) = shift;
    my ($and_or) = shift;

    my ($where_clause, $val, $add_closing);


    $where_clause = "";

    if ($value =~ /\)/) {
	$add_closing = "yes";
    }

    if (!defined $and_or) {
	$and_or = "AND";        # if not specified, assume AND
    }
    $and_or = uc($and_or);

    if ($value =~ /\!/) { 
	$val = $value;
	$val =~ s/\!//gms;
	$val =~ s/\)//gms;
	if ($value =~ /\%/) {
	    if ($value =~ /\?/) {
		$where_clause .= "$field NOT LIKE $val AND ";
	    }
	    else {
		$where_clause .= "$field NOT LIKE '$val' AND ";
	    }
	}
	else {
	    if ($value =~ /\?/) {
		$where_clause .= "$field <> $val AND ";
	    }
	    else {
		$where_clause .= "$field <> '$val' AND ";
	    }
	}
    }
    elsif ($value =~ /\%/) {
	$value = &string_todbi($value);
	$value =~ s/\)//gms;
	if ($value =~ /\?/ || $field =~ /sid/) {
	    $where_clause .= "$field LIKE $value $and_or ";
	}
	else {
	    $where_clause .= "$field LIKE '$value' $and_or ";
	}
    }
    elsif ($value =~ /\>=/) {
	$value = &string_todbi($value);
	$value =~ s/\>=//gms;
	if ($value =~ /\?/ || $field =~ /sid/) {
	    $where_clause .= "$field >= $value $and_or ";
	}
	else {
	    $where_clause .= "$field >= '$value' $and_or ";
	}
    }

    elsif ($value =~ /\<=/) {
	$value = &string_todbi($value);
	$value =~ s/\<=//gms;
	if ($value =~ /\?/) {
	    $where_clause .= "$field <= $value $and_or ";
	}
	else {
	    $where_clause .= "$field <= '$value' $and_or ";
	}
    }

    elsif ($value =~ /\</) {
	$value = &string_todbi($value);
	$value =~ s/\<//gms;
	if ($value =~ /\?/) {
	    $where_clause .= "$field < $value $and_or ";
	}
	else {
	    $where_clause .= "$field < '$value' $and_or ";
	}
    }

    elsif ($value =~ /\>/) {
	$value = &string_todbi($value);
	$value =~ s/\>//gms;
	if ($value =~ /\?/) {
	    $where_clause .= "$field > $value $and_or ";
	}
	else {
	    $where_clause .= "$field > '$value' $and_or ";
	}
    }

    else {
	$value = &string_todbi($value);
	$value =~ s/\)//gms;
	if ($value =~ /\?/ || $field =~ /sid/) {
	    $where_clause .= "$field = $value $and_or ";
	}
	else {
	    $where_clause .= "$field = '$value' $and_or ";
	}
    }

    if ($add_closing) {
	$where_clause =~ s/ $and_or $//gms;  # remove the last and or in the statement
	$where_clause .= ") $and_or ";
    }

    return ($where_clause);
}

sub rm_all_where_tags {
    my ($field) = shift;
    my ($value) = shift;
    my ($site) = shift;

    &debug("rm_all_where_tags:  In here with $field, $value", 3, $site->{'debug_level'}, $site->{'debug_log'});


    if ((defined $field) && ($field ne "")) {
	$field =~ s/\(//gms;
    }

    if ((defined $value) && ($value ne "")) {
	$value =~ s/\?//gms;
	$value =~ s/\)//gms;
	$value =~ s/\>=//gms;
	$value =~ s/\<=//gms;
	$value =~ s/\>//gms;
	$value =~ s/\<//gms;
	$value =~ s/\%//gms;
	$value =~ s/\!//gms;
    }

    &debug("rm_all_where_tags:  After conversions  $field, $value", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($field, $value);
}

##############################################################################
=pod

=head2 build_where_hash

build_where_hash - &build_where_hash("title", "!Hi There", "and");

=over 4

=item DESCRIPTION

Setup the hashref for one element of the hashref.  Set the ->{'field'}, ->{'value'} and ->{'and_or'}
portion of the hash preparing it for a call to build_sql.

=item INPUT

$field - Name of the field to do the comparison against

$value - The field value to use to create the where clause with

$and_or - The command to use to combine with the next where clause element (AND, OR).

=item OUTPUT

$where_hash - Hashref with the keys in the order as stated in the NOTES section of 
db_layer.pm

=item NOTES

=back

=cut
###############################################################################
sub build_where_hash {
    my ($field) = shift;
    my ($value) = shift;
    my ($and_or) = shift;

    my ($where_hash);

    $where_hash = {};
    $where_hash->{'field'} = $field;
    $where_hash->{'value'} = $value;
    $where_hash->{'and_or'} = $and_or;

    return ($where_hash);
}


##############################################################################
=pod

=head2 get_new_objid

get_new_objid - &get_new_objid($site, $dbh, "thing", $sql_query)

=over 4

=item DESCRIPTION
This routine could be called 'create_event' as it this inserts an event into the
levents table, and returns back an objid which is the next objid to be used in an
insert statement.

=item INPUT

$site - Site hashref

$dbh - Current active database handler

$tablename - Name of table insert requesting the objid

$sql_statement - SQL statement with __objid__ in place of the objid that is generated
                 by levents.

=item OUTPUT

$objid - The next objid in the series if no problems occured, or undef if error occured.

=item NOTES

This subroutine must create it's own SQL INSERT statement, as if it calls build_insert
we would get into a loop, with build_insert calling get_new_objid and so on.

=back

=cut
###############################################################################
sub get_new_objid {
    my ($site) = shift;
    my ($dbh) = shift;
    my ($tablename) = shift;
    my ($sql_statement) = shift;

    my ($dvalue, $q, $numRows, @results, $event, $series, $now, $objid);
    my (@objid_results);

    $now = time();
    $sql_statement =~ s/'//gms;
    $dvalue = "Create ObjID\t$sql_statement";
    if ( $site->{'site_datadriver'} =~ /mysql/ ) {
        $q = "INSERT INTO levents (dtable, dvalue, cdate, series) VALUES";
         $q .= " ('$tablename', '$dvalue', '$now', '$site->{'series'}')";
    } elsif ( $site->{'site_datadriver'} =~ /Oracle/ ) {
         $q .= "INSERT INTO levents (event, dtable, dvalue, cdate, series) VALUES (levents_event.nextval, '$tablename', '$dvalue', $now, '$site->{'series'}')";
   } elsif ( $site->{'site_datadriver'} =~ /Pg/ ) {
         $q .= "INSERT INTO levents (event, dtable, dvalue, cdate, series) VALUES (NEXTVAL('event'), '$tablename', '$dvalue', $now, '$site->{'series'}')";
   } 


    $numRows = &query($q, $dbh, \@results);
    if ($numRows > 0) {
         if ( $site->{'site_datadriver'} =~ /mysql/ ) {
           $q = "SELECT event, series FROM levents WHERE dtable = '$tablename'";
           $q .= " AND dvalue = '$dvalue' AND cdate = '$now'";
         } elsif ( $site->{'site_datadriver'} =~ /Oracle/ ) {
             $q = "SELECT event, series FROM levents WHERE ";
             $q .= " dvalue = '$dvalue' AND cdate = '$now'";
         } elsif ( $site->{'site_datadriver'} =~ /Pg/ ) {
             $q = "SELECT CURRVAL('event'),series FROM levents WHERE ";
             $q .= " dvalue = '$dvalue' AND cdate = '$now'";
          my $sts=$dbh->prepare($q);
          $sts->execute;
          my ($event,$series)=$sts->fetchrow;  
          $objid = $event. "-" . $series;
          return ($objid);
         } 
	$numRows = &query($q, $dbh, \@objid_results);
	if ($numRows > 0) {
	    $objid = $objid_results[0]->{'event'} . "-" . $objid_results[0]->{'series'};
	    return ($objid);
	}
    }
    return;
}

##############################################################################
=pod

=head2 query

query - &query($q, $dbh, \@results);

=over 4

=item DESCRIPTION

To submit a query to the $dbh, returning results if there are any

=item INPUT

$q - String which is the SQL query to execute

$dbh - Current active database handle

$p_results - A pointer to an array to return the results back in.

=item OUTPUT

$p_results -  An array of hashes, each index consisting on the simple has with the keys being
              the fields requested in the query

$rows - The number of rows returned

=item NOTES

You would not want to issue this command for searches that will bring back a lot of
data, as it returns ALL results.

=back

=cut
###############################################################################
sub query {
    my ($q) = shift;
    my ($dbh) = shift;
    my ($p_results) = shift;
    my ($maxrows) = shift; # OPTIONAL

    my ($sth, $rows, $results, $check, $i, $filter);

    ($q, $filter) = split /\:\:/, $q;
    $rows = 0;
    if (length ($q) > 0) {
	$sth=$dbh->prepare($q);
	$check = $sth->execute();
	
	# get the row if it is a select statement,
	# otherwise we do not need the results
	if (($q =~ /SELECT/) || ($q =~ /select/)){
	    if ($maxrows) {
		# get ALL rows returned
		while (($$p_results[$rows] = $sth->fetchrow_hashref('NAME_lc')) && ($rows < $maxrows)) {
		    $rows++;
		}
	    }
	    else {
		# get ALL rows returned
		while ($$p_results[$rows] = $sth->fetchrow_hashref('NAME_lc')) {
		    $rows++;
		}
	    }
	    $sth->finish;
	    return $rows;
	}
	$sth->finish;
	return $check;
    }
    return (0);  # no query to send
}


##############################################################################
=pod

=head2 query_getAll

query_getAll - &query_getAll($q, $site, $dbh, \@results);

=over 4

=item DESCRIPTION

To submit a query to the $dbh, returning ALL results that the user has permissions
to view.  If a non-select statement is passed into this routine, it will automatically
call query_execute to execute the result.  

=item INPUT

$q - String which is the SQL query to execute

$site - Site hashref

$dbh - Current active database handle

$p_results - A pointer to an array to return the results back in.

=item OUTPUT

$rows - The number of rows returned

$p_results -  Hashref with all of the results.  The main key into the hashref is 
              the objid of the row, and the value is the hashref of the row retreived.
              Only information that the user has $minumum_permissions to will be displayed.

=item NOTES

You would not want to issue this command for searches that will bring back a lot of
data, as it returns ALL results.

=back

=cut
###############################################################################
sub query_getAll {
    my ($q) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($p_results) = shift;

    my ($sth, $rows, $results, $check, $i, $filter, $thisrow);
    my ($add_row);

    if ($q !~ /SELECT/) {
	return (&query_execute($q, $site, $dbh));
    }
    ($q, $filter) = split /\:\:/, $q;
    $rows = 0;
    if (length ($q) <= 0) {
	&debug("Query of no length passed into query_getNext", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    if ( $site->{'site_datadriver'} =~ /Oracle/ ) {
	$q =~ s/\'//g if ( $q =~ /sessions/ ); 
    }

    $sth=$dbh->prepare($q);
    $check = $sth->execute();
    
    # get ALL rows returned
    while ($thisrow = $sth->fetchrow_hashref('NAME_lc')) {
	if ($filter and $filter eq "true_filter") {  # we need to filter the results
	    if ($rows == 0) {
		$add_row = &check_row_fields_permissions($thisrow, $site, "r");
		if (!defined $add_row) {
		    &debug("Cannot select these fields, insufficient permissions", 2, $site->{'debug_level'}, $site->{'debug_log'});
		    $sth->finish;
		    return;
		}
	    }
	    $add_row = &check_objclass($thisrow, $site, "r");
	    if (defined $add_row) {
		$$p_results[$rows] = $thisrow;
		$rows++;
	    }
	}
	else {  # we don't need to filter, it is permitted
	    $$p_results[$rows] = $thisrow;
	    $rows++;
	}
    }
    $sth->finish;
    &debug("query_getAll: $q\n\t-->$rows rows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
    return $rows;
}



##############################################################################
=pod

=head2 


=over 4

=item DESCRIPTION

To submit a SELECT statement and filter the results through the permissions
layer to see how many rows are affected.

=item INPUT

$q - String which is the SQL query to execute

$site - Site hashref

$dbh - Current active database handle

$permission - The minimum permission that must be maintained for the object

=item OUTPUT

$rows - The number of rows returned

=item NOTES

You would not want to issue this command for searches that will bring back a lot of
data, as it returns ALL results.

=back

=cut
###############################################################################
sub query_findNumRows {
    my ($q) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($sth, $rows, $results, $check, $i, $filter, $thisrow);
    my ($add_row);

    if ($q !~ /SELECT/) {
	return;
    }

    ($q, $filter) = split /\:\:/, $q;
    $rows = 0;
    if (length ($q) <= 0) {
	&debug("Query of no length passed into query_getNext", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $sth=$dbh->prepare($q);
    $check = $sth->execute();
    
    # get ALL rows returned
    while ($thisrow = $sth->fetchrow_hashref('NAME_lc')) {
	if ($filter eq "true_filter") {  # we need to filter the results
	    $add_row = &check_objclass($thisrow, $site, "r");
	    if (defined $add_row) {
		$rows++;
	    }
	}
	else {  # we don't need to filter, it is permitted
	    $rows++;
	}
    }
    $sth->finish;
    &debug("query_findNumRows: $q\n\t-->$rows rows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
    return $rows;
}

##############################################################################
=pod

=head2 query_getNext

query_getNext - &query_getNext($q, $site, $dbh, $sth);

=over 4

=item DESCRIPTION

If the query has not been executed, then execute the query.  If the query has 
already been executed once, get the next row from the valid $sth.  Keep attempting
to get a new row that the user has permissions to view. Return the first row, and 
the $sth that the user has permissions to view.  If the user should not be executing
this query, then return undef.  If there are no more rows to query, return 0 rows
and $sth undef.

=item INPUT

$q - String which is the SQL query to execute

$site - Site hashref

$dbh - Current active database handle

$sth - Current active statement handle, if the query has already been executed, otherwise
       undef

=item OUTPUT

$numRows - undef if the user does not have permissions for this query
        1 if we are returning a row,
        0 if there are no more rows left.

$result -  Hashref with all of the results.  The main key into the hashref is 
              the objid of the row, and the value is the hashref of the row retreived.

$sth - Statement handler, at it's current poriton of execution.  To be passed in on
       the next call to get the next row.  

=item NOTES

=back

=cut
###############################################################################
sub query_getNext {
    my ($q) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($sth) = shift;


    my ($rows, $results, $check, $i, $filter, $thisrow);
    my ($add_row, $continue);

    if ($q !~ /SELECT/) {
	return (&query_execute($q, $site, $dbh));
    }
    ($q, $filter) = split /\:\:/, $q;
    $rows = 0;
    if (length ($q) <= 0) {
	&debug("Query of no length passed into query_getNext", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $continue = 1;
    while ($continue) { 
	if (!defined $sth) {
	    $sth=$dbh->prepare($q);
	    $check = $sth->execute();
	    $thisrow = $sth->fetchrow_hashref('NAME_lc');
	}
	else {
	    $thisrow = $sth->fetchrow_hashref('NAME_lc');
	}

	# no more rows to retreive
	if (!defined $thisrow) {
	    &debug("query_getNext: $q\n\t-->DONE, no rows returned", 2, $site->{'debug_level'}, $site->{'debug_log'});
	    return (0, $thisrow, $sth);
	}

	# get ALL rows returned
	if ($filter eq "true_filter") {  # we need to filter the results
	    if ($rows == 0) {
		$add_row = &check_row_fields_permissions($thisrow, $site, "r");
		if (!defined $add_row) {
		    &debug("Cannot select these fields, insufficient permissions", 2, $site->{'debug_level'}, $site->{'debug_log'});
		    return;
		}
	    }
	    $add_row = &check_objclass($thisrow, $site, "r");
	    if (defined $add_row) {
		&debug("query_getNext: $q\n\t-->1 row returned", 2, $site->{'debug_level'}, $site->{'debug_log'});
		return (1, $thisrow, $sth);
	    }
	}
	else {  # we don't need to filter, it is permitted
	    &debug("query_getNext: $q\n\t-->1 row returned", 2, $site->{'debug_level'}, $site->{'debug_log'});
	    return (1, $thisrow, $sth);	    
	}
    }
}

##############################################################################
=pod

=head2 query_getRows

query_getRows - &query_getRows($q, $site, $dbh, $start, $numToGet, \@results);

=over 4

=item DESCRIPTION

To execute a query, and return at max $getNumRows, starting from $start.  Only
return the rows that the user has permission to view.  Return the next $start position.
Return $start of undef when all of the rows have been retreived.

=item INPUT

$q - String which is the SQL query to execute

$site - Site hashref

$dbh - Current active database handle

$startRow - Row to start getting data from
 
$getNumRows - Number of rows to retreive, starting at $startRow.  If it is "",
              then we get all rows starting from $startRow.

$p_results -  Hashref with all of the results.  The main key into the hashref is 
              the objid of the row, and the value is the hashref of the row retreived.
              Only information that the user has $minumum_permissions to will be displayed.

=item OUTPUT

$gotRows - Number of row retreived

$curRow - The next row to read.  This can be set to $start the next time that the routine
          is called.

$p_results -  Populated with results.

=item NOTES

=back

This would be good for 'search', as we need to execute the call every time in search,
and we know that start row and how many rows to return.  This reduces the amount
of memory load, but due to the nature of executing the query every time, this is a
slower routine than the rest.  However, it fits well with the way that search is 
currently implemented in the system.

=cut
###############################################################################
sub query_getRows {
    my ($q) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($startRow) = shift;
    my ($getNumRows) = shift;
    my ($p_results) = shift;

    my ($sth, $rows, $results, $check, $i, $filter, $thisrow);
    my ($add_row, $endrow, $gotRows, $curRow);

    if ($q !~ /SELECT/) {
	return (&query_execute($q, $site, $dbh));
    }
    ($q, $filter) = split /\:\:/, $q;

    if (length ($q) <= 0) {
	&debug("Query of no length passed into query_getRows", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $sth=$dbh->prepare($q);
    $check = $sth->execute();
    $gotRows = 0;
    $rows = 0;
    
    # While there are rows left, and we haven't got $getNumRows yet, or getNumRows is not defined
    while (($thisrow = $sth->fetchrow_hashref('NAME_lc')) && (($getNumRows eq "") || ($gotRows < $getNumRows))) {
	if ($curRow >= $startRow) {
	    if ($filter eq "true_filter") {  # we need to filter the results
		if ($rows == 0) {
		    $add_row = &check_row_fields_permissions($thisrow, $site, "r");
		    &debug("Cannot select these fields, insufficient permissions", 2, $site->{'debug_level'}, $site->{'debug_log'});
		    $sth->finish;
		    return;
		}
		$add_row = &check_objclass($thisrow, $site, "r");
		if (defined $add_row) {
		    $$p_results[$gotRows] = $thisrow;
		    $gotRows++;
		    $rows++;
		}
	    }
	    else {  # we don't need to filter, it is permitted
		$$p_results[$gotRows] = $thisrow;
		$gotRows++;
		$rows++;
	    }
	}
	$curRow++;
    }
    $sth->finish;
    if (!defined $thisrow) {  # we have got everything!
	&debug("query_getRows: $q\n\t-->$gotRows returned, no more after this", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return ($gotRows);  # make curRow undef so it knows when to stop
    }

    &debug("query_getRows: $q\n\t-->$gotRows returned, on row $curRow next time", 2, $site->{'debug_level'}, $site->{'debug_log'});
    return ($gotRows, $curRow);
}

##############################################################################
=pod

=head2 query_execute

query_execute - &query_execute($q, $site, $dbh);

=over 4

=item DESCRIPTION

To execute a query.  Return undef if the query did not execute properly, 
return "true" if it succeeded.  It is assumed that by the time that this routine
is called, that the permissions have already been checked (in other words, if the
user didn't have the permissions, the query would not have been created). 

=item INPUT

$q - String which is the SQL query to execute

$site - Site hashref

$dbh - Current active database handle

=item OUTPUT

"true" if the query was executed successfully
undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub query_execute {
    my ($q) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($rows, $results, $check, $i, $objid, $thisrow);
    my ($add_row, $continue, $sth);

    ($q, $objid) = split /\:\:/, $q;
    $rows = 0;
    if (length ($q) <= 0) {
	&debug("query_execute: No query passed in", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    $sth=$dbh->prepare($q);
    $check = $sth->execute();
    $sth->finish;
    &debug("query_execute: $q\n\t-->$check rows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});

    if ((!defined $check) || ($check == 0)) {
	&debug("query_execute: It affected 0 rows, so there was an error!", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }
    return "true";
}

##############################################################################
=pod

=head2 get_field_names

get_field_names - &get_field_names("things", $dbh, $site);

=over 4

=item DESCRIPTION

To retrieve a list of fieldnames from the $table.

=item INPUT

$tablename - Name of the table to retrieve the fieldnames for

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$field_names - string of fields that exist in the things table, comma delimited.

=item NOTES

=back

=cut
###############################################################################
sub get_field_names {
    my ($tablename) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($select_hashref, $q, $sth, $record, @fields, $field_names, $numRows);
    my ($insert_hashref, $delete_hashref, @results, $nextRows);
    $select_hashref = {};
    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = $tablename;
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $q = &build_select($select_hashref, $site, $dbh, 1);
    ($numRows, $nextRows) = &query_getRows($q, $site, $dbh, 0, 1, \@results);
    if ($numRows == 0) {  # This was the first query to the table, there is nothing there
                          # so we will not get any fieldnames.  Temporarily insert a row to
                          # use for select, then delete it.
	&debug("get_field_names:  Need to insert temporary row to $tablename", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$insert_hashref = {};
	$insert_hashref->{'statement'} = "insert";
	$insert_hashref->{'TABLE'}->{'NAME'} = $tablename;
	$insert_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "TEMP";
	$q = &build_sql($insert_hashref, $site, $dbh, 1);
	if (!&query_execute($q, $site, $dbh)) {
	    &debug("get_field_names:  Could not insert temporary record from $tablename", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}

	# Get the select stuff handled
	$select_hashref = {};
	$select_hashref->{'statement'} = "SELECT";
	$select_hashref->{'TABLE'}->{'NAME'} = $tablename;
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	$q = &build_select($select_hashref, $site, $dbh, 1);
	($numRows, $nextRows) = &query_getRows($q, $site, $dbh, 0, 1, \@results);
	if ($numRows == 0) {
	    &debug("get_field_names: Cannot get names of fields for table $tablename.  Tried everything, but not working.  $dbh is the dbh.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
	else {
	    $record = $results[0];
	}

	# Delete the row
	$delete_hashref = {};
	$delete_hashref->{'statement'} = "delete";
	$delete_hashref->{'TABLE'}->{'NAME'} = $tablename;
	$delete_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", "TEMP", "and");
	$q = &build_sql($insert_hashref, $site, $dbh, 1);
	if (!&query_execute($q, $site, $dbh)) {
	    &debug("get_field_names:  Could not delete temporary record from $tablename", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }	
    else {
	$record = $results[0];
    }
    if (defined $record) {
	@fields = keys %{$record};
	$field_names = join ",", @fields;
	return $field_names;
    }
    return;
}

##############################################################################
=pod

=head2 load_object

load_object - &load_object($objid, "things,prices", "title,unitpr", \@where, $site, $dbh);

=over 4

=item DESCRIPTION

Queries the $tables tables and extraprop for all data for that objid, and assembles a hash like
$h->{'lang'}->{'fieldname'}.  There is one lang entry for each language that is defined.
For products, inventory and pricing tables are also queried, and stored in $h-{'inventory'}->{'wh'}
foreach warehouse defined, and $h->{'prices'}->{'objid'} for each price defined.

=item INPUT

$objid - Object ID of the object to load in

$tables - list of tables to look for the data in.  If left blank, it searches
          things.  The attribute tables extraprop is always searched.

$fields -  List of fields to retreive, comma sepereated.  If blank, then uses *

$where_clause - Pointer to an array, defining the where clause to append.  See th
                Note in db_layer.pm to see the format for the where clause.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$override - If set, then do not filter by permissions, set everything to the highest
                permissions, ->{'c'}
            If NOT set, then filter by permissions and
             seperate hashref into ->{'c'} for create attributes, ->{'m'} for modify
             attributes and ->{'r'} for read only attributes.
           

=item OUTPUT

$object_hashref - Hashref of all of the information for the object.

=item NOTES

=back

=cut
###############################################################################
sub load_object {
    my ($objid) = shift;
    my ($tables) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($object_hashref, $filtered_hashref, @all_tables, $table);

    &debug("In load_object with $where_clause, $objid, $dbh", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $object_hashref = {};
    $tables =~ s/ //gms;

    if ((!defined $tables) || ($tables eq "")) {
	$tables = "things,extraprop";
    }

    # always look in extraprop
    if ($tables !~ /extraprop/) {
	$tables .= ",extraprop";
    }

    $fields =~ s/ //gms;

    # always have language
    if (($fields ne "") && ($fields !~ /language/)) {
	$fields .= ",language";
    }


    @all_tables = split /\,/, $tables;
    foreach $table (@all_tables) {
	$object_hashref = &get_object($objid, $object_hashref, $table, $fields, $where_clause, $site, $dbh);
    }

    if ($override) {
	&debug("Not filtering results, override is set", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$filtered_hashref->{'c'} = $object_hashref;
	return ($filtered_hashref);
    }

    $filtered_hashref = &filter_row_result($object_hashref, $site);
    return ($filtered_hashref);
}


##############################################################################
=pod

=head2 load_default_object

load_default_object - &load_default_object($objid,"things","title,brief",\@where, $site, $dbh);

=over 4

=item DESCRIPTION

To load the object specified by $objid, and set all of the fields requested to
their default language value.  This means, take the value of the field that is
defined for the languages closest to the front of the list $lang_order, where
$lang_order is found by calling getLanguageOrder.  Return the hashref describing
the details of the object, with all of the language sensitive data requested
filled in at the root of the hashref, with the value from the most important 
language

=item INPUT

$objid - The object to load

$tables - The tables to look for the object in

$fields - The fields to look for, or "" or * to get all fields

$where - Pointer to an array describing a where clause, for details look in the
         NOTES section of db_layer.pm

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_hash - Hashref describing object with all of the $fields found that
               match the $where clause from $tables.  All of the language
               sensitive data is set to be the value from the most important
               language in the root.

=item NOTES

=back

=cut
###############################################################################
sub load_default_object {
    my ($objid) = shift;
    my ($tables) = shift;
    my ($fields) = shift;
    my ($where) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($lang_order, $object_hash);

    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
    $object_hash = &load_object($objid, $tables, $fields, $where, $site, $dbh, 1);
    $object_hash  = &set_fields_to_default_lang_val($object_hash->{'c'}, $fields, $lang_order, $site);
    return ($object_hash);
}

##############################################################################
=pod

=head2 get_object

get_object - &get_object("1-AA", $object_hashref, "contacts", "uname, passwd", $where_clause, $site, $dbh);

=over 4

=item DESCRIPTION

Determines from the table, whether the table is a primary or secondary table, and
calls the appropriate routine to retreive the information.

=item INPUT

$objid - Object ID of the object of the things record to retreive

$object_hashref - Hashref to store the record in, in proper format.  Could
                  already be populated in some areas.

$table - The table which to get the data from.

$where_clause - Pointer to an array, defining the where clause to append.  See th
                Note in db_layer.pm to see the format for the where clause.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_hashref - Hashref returned in proper format.  All language specific fields
                  are stored in 'lang' hashrefs, where 'lang' is the language that
                  they are defined for.

=item NOTES

If you want to support an additional secondary table, then you must write a 
format routine, named format_<table>_record, accepting the parameters
    $object_hashref, $table, $site, $numSecondaryRows, \@secondary_res, $fields

You must also add it to the $secondary_tables list.

=back

=cut
###############################################################################
sub get_object {
    my ($objid) = shift;
    my ($object_hashref) = shift;
    my ($table) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($site) = shift;
    my ($dbh) = shift;


    my ($primary_tables, $secondary_tables);

    # List of primary tables which are supported
    $primary_tables = "things,contacts";

    # List of secondary tables which are supported
    $secondary_tables = "extraprop,inventory,prices";

    if ($primary_tables =~ /$table/) {
	&debug("Getting data for $table from primary tables", 3, $site->{'debug_level'}, $site->{'debug_log'});	    
	$object_hashref = &get_primary_data($objid, $object_hashref, $table, $fields, $where_clause, $site, $dbh);
    }
    elsif ($secondary_tables =~ /$table/) {
	&debug("Getting data for $table from secondary tables", 3, $site->{'debug_level'}, $site->{'debug_log'});	    
	$object_hashref = &get_secondary_data($objid, $object_hashref, $table, $fields, $where_clause, $site, $dbh);
    }
    else {
	&debug("get_object does not support $table yet.  Please modify for your needs.", 1, $site->{'debug_level'}, $site->{'debug_log'});
    }

    return $object_hashref;
}

##############################################################################
=pod

=head2 get_primary_data

get_primary_data - &get_primary_data("1-AA", $object_hashref, "contacts", "uname, passwd", $where_clause, $site, $dbh);

=over 4

=item DESCRIPTION

To retrieve the data from the primary table and return a hashref in proper format, with
all language specific fields stored in their own hash referenced by the language.

The primary table is the table in which objid is the primary key and we are gaurenteed
that only 1 row will be returned.  Ex: things, contacts, orders, addresses.

=item INPUT

$objid - Object ID of the object of the things record to retreive

$object_hashref - Hashref to store the record in, in proper format.  Could
                  already be populated in some areas.

$table - The table which to get the data from.

$where_clause - Pointer to an array, defining the where clause to append.  See th
                Note in db_layer.pm to see the format for the where clause.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_hashref - Hashref returned in proper format.  All language specific fields
                  are stored in 'lang' hashrefs, where 'lang' is the language that
                  they are defined for.

=item NOTES

=back

=cut
###############################################################################
sub get_primary_data  {
    my ($objid) = shift;
    my ($object_hashref) = shift;
    my ($table) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($select_hashref, $q, $numPrimaryRows);
    my ($table_fields, @primary_res, $where, @all_fields, $field);
    my ($where_indx, $where_hashref, $junk);

    $table_fields = &get_field_names($table, $site, $dbh);

    $select_hashref = {};

    # set up the table
    if ((!defined $table) || ($table eq "")) {
	$select_hashref->{'TABLE'}->{'NAME'} = "things";
    }
    else {
	$select_hashref->{'TABLE'}->{'NAME'} = $table;
    }

    # set up the fields
    if ((!defined $fields) || ($fields eq "")) {
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    }
    else {
	$fields =~ s/ //gms;
	@all_fields = split /\,/, $fields;
	foreach $field (@all_fields) {
	    if ($table_fields =~ /$field/) {
		$select_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;	    
	    }
	}
    }

    # set up the where clause
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

    foreach $where_hashref (@$where_clause) {
	($field, $junk) = &rm_all_where_tags($where_hashref->{'field'}, "", $site);
	# Don't use the status in the extraprop query!  Hardcode for now.
	next if (($select_hashref->{'TABLE'}->{'NAME'} eq "extraprop") && ($where_hashref->{'field'} eq "status"));
	if ($table_fields =~ /$field/) {
#	if ($table_fields =~ /$where_hashref->{'field'}/) {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
	}
    }

    $q = &build_select($select_hashref, $site, $dbh, $site->{'override_permissions'});
    $numPrimaryRows = &query_getAll($q, $site, $dbh, \@primary_res);

    if ($numPrimaryRows > 0) {
	# format the result
	$object_hashref = &format_primary_record($object_hashref, $table, $site, $primary_res[0]);
	return $object_hashref;
    }

    return undef;
}

##############################################################################
=pod

=head2 get_secondary_data

get_secondary_data - &get_secondary_data("1-AA", $object_hashref, "extraprop", "author,title", $where, $site, $dbh); 

=over 4

=item DESCRIPTION

To retrieve the secondary data from the tables, and order them in the specified
order.  The secondary table is the attribute tables, where multiple rows may be
returned by objid.  There is not a unique key alone, it is a complex multiple 
secondary key which makes the row unique.  Examples are extraprop, prices,
inventory, linkage, tcart...

=item INPUT

$objid - Object ID of the object to retrieve the prices for

$object_hashref - Hashref to store the prices records in, in proper format.  Could
                  already be populated in some areas.  Stored under the 'prices'
                  key.  Then are stored in the following syntax:

$table -  The table which is to be queried

$fields - list of fields to retreive, if blank, it retreives everything

$where_clause - Pointer to an array, defining the where clause to append.  See th
                Note in db_layer.pm to see the format for the where clause.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_hashref - Hashref returned in proper format.  

=item NOTES

=back

=cut
###############################################################################
sub get_secondary_data {
    my ($objid) = shift;
    my ($object_hashref) = shift;
    my ($table) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($select_hashref, $q, $numSecondaryRows, $table_fields, $format_subroutine_name);
    my (@secondary_res, $format_subroutine, @all_fields, $field, $where);
    my ($where_indx, $num_where, @existing_where, $where_hashref, $junk);

    $table_fields = &get_field_names($table, $site, $dbh);
    
    $select_hashref = {};

    $select_hashref->{'TABLE'}->{'NAME'} = $table;

    # set up the fields, if not defined use *, and use * if it is extraprop in 
    # order to reduce queries
    # set up the fields
    if ((!defined $fields) || ($fields eq "") || ($table eq "extraprop")) {
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    }
    else {
	$fields =~ s/ //gms;
	@all_fields = split /\,/, $fields;
	foreach $field (@all_fields) {
	    if ($table_fields =~ /$field/) {
		$select_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;	    
	    }
	}
	# Just in case nothing from above worked
	if (!defined $select_hashref->{'TABLE'}->{'FIELDS'}) {
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	}
    }

    # set up the where clause
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

    foreach $where_hashref (@$where_clause) {
	# Don't use the status in the extraprop query!  Hardcode for now.
	($field, $junk) = &rm_all_where_tags($where_hashref->{'field'}, "", $site);
	next if (($select_hashref->{'TABLE'}->{'NAME'} eq "extraprop") && ($field eq "status"));
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
    }

    $q = &build_select($select_hashref, $site, $dbh, $site->{'override_permissions'});
    $numSecondaryRows = &query_getAll($q, $site, $dbh, \@secondary_res);
    if ($numSecondaryRows > 0) {
	$format_subroutine_name = "format_" . $table . "_record";
	$format_subroutine = \&$format_subroutine_name;
	
	$object_hashref = &$format_subroutine($object_hashref, $site, $numSecondaryRows, \@secondary_res, $fields);

    }

    return $object_hashref;
}

##############################################################################
=pod

=head2 load_objects

load_objects - &load_objects($tables, $fields, $where, "", "", $site, $dbh, $site->{'override_permissions'});

=over 4

=item DESCRIPTION

Load in all $fields for all objects from all of the $tables that match the $where
clause, taking into consideration permissions.  It also sets the results for
each row to the default language results, meaning, copying the value of all
language sensitive data of the 'most important' language down to the main level
of the object hashref.

Returns

=item INPUT

$tables - comma seperated list of tables to search for information in.

$fields - comma seperated list of fields to try to find in the list of $tables

$where_clause - pointer to an array, where clause built using &build_where_hash

$startRows - "", start at row 0 for all tables
             "#:#:#", where each # is the start for for the table mapped to the
                      location of the table when the $tables list is sorted in
                      alphabetical order.

             Therefore, if you had "things,contacts,extraprop" in tables nad
             "3:7:9" in $startRows, this means that contacts starts at row
              3, extraprop starts at row 7 and things start at row 9 (as
              the alphabetical listing of the tables is "contacts,extraprop,things".

$max_num_rows - Total maximum number of rows to retreive.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$override - If set, override permission check.

$language_build_routine - OPTIONAL, if specified, this means to search
                          the language table (extraprop in this version), and use this
                          routine to build the queries to execute.  This is used in 
                          admin object search, where we need to consider extraprop as both
                          a primary and secondary table, primary for language definitions and
                          secondary for other overflow.
                          
                          If this is set, then we need to call get_language_primary_objects between
                          get_primary_objects and get_secondary_objects.  This will allow us to 
                          sort search results based on language.


$lang_startat - OPTIONAL - if specified, this is the starting row for looking for language data (data
                           from extraprop treated as a primary record).

$match_all_langs - OPTIONAL, if specified, indicates to only match on data in the appropriate render language
                            if the data is defined in more than one language.  This is necessary for a plain
                            search, where the object should only be shown in the render language.  
                            
NOTE:  All OPTIONAL parameters must be specified if any are to be specified.  They all work together, otherwise will
       get strange functionality! They are used in search.

=item OUTPUT

$numPrimary - The number of primary records found.

$next_start_row -    "#:#:#", where each # is the start for for the table mapped to the
                      location of the table when the $tables list is sorted in
                      alphabetical order.

             Therefore, if you had "things,contacts,extraprop" in tables nad
             "3:7:9" in $startRows, this means that contacts starts at row
              3, extraprop starts at row 7 and things start at row 9 (as
              the alphabetical listing of the tables is "contacts,extraprop,things".

@return_records - An array of records, each index being the result of one of
                  the searches.

=item NOTES

This is going to be needed for search in some means, but need to find a way
to figure out the total number of rows, and the page links on the bottom of
the search results.

=back

=cut
###############################################################################
sub load_objects {
    my ($tables) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($startRows) = shift;
    my ($max_num_rows) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;
    my ($language_build_routine) = shift;         # OPTIONAL
    my ($lang_startat) = shift;   # OPTIONAL
    my ($match_all_langs) = shift;  #OPTIONAL

    my ($object_hashref, $filtered_hashref, @all_tables, $table, @objects, @primary_table_list, @secondary_table_list);
    my ($primary_tables, $secondary_tables, $numPrimary, $num, $numSecondary, $primary_objects, $secondary_objects);
    my ($primary_search_order, $next_start, $all_records);
    my (@sorted_results, @results, $result, $objid, @all_results, @ordered_tables);
    my ($indx, $second_table, @return_records, $next_start_row, $lang_order, $search_indx);
    my ($numLanguage,  $next_lang_startat, @start, $table_max_num_rows);
    my ($max_rec_number, $search_language, $tmp);

    $search_language = 0;
    if ((defined $language_build_routine) && ($language_build_routine ne "")) {
	$search_language = 1;
    }

    $tmp = $search_language && $match_all_langs;

    &debug("In load_objects with $tables, $fields, $where_clause, $startRows, $max_num_rows, $override, search: $search_language, match: $match_all_langs", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # List of primary tables which are supported
    $primary_tables = "addresses,things,contacts";

    $primary_search_order->{'things'} = "title";
    $primary_search_order->{'contacts'} = "uname";
    $primary_search_order->{'addresses'} = "name";

    # List of secondary tables which are supported
    $secondary_tables = "extraprop,inventory,prices";

    $object_hashref = {};
    $tables =~ s/ //gms;
    
    if ((!defined $tables) || ($tables eq "")) {
	$tables = "things,extraprop";
    }
    
    # always look in extraprop
    if ($tables !~ /extraprop/) {
	$tables .= ",extraprop";
    }
    
    $fields =~ s/ //gms;
    
    if (($fields ne "") && ($fields ne "*")) {
	if ($fields !~ /language/) {
	    # always have language
	    $fields .= ",language";
	}
	
	if ($fields !~ /objid/) {
	    # always have objid
	    $fields .= ",objid";
	}
    }
    
    @all_tables = split /\,/, $tables;
    foreach $table (@all_tables) {
	if ($primary_tables =~ /$table/) {
	    push @primary_table_list, $table;
	}
	elsif ($secondary_tables =~ /$table/)  {
	    push @secondary_table_list, $table;
	}
	else {
	    &debug("load_objects: $table is NOT supported.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }
    
    $primary_objects = {};
    
    if ($max_num_rows ne "") {
	$table_max_num_rows = $max_num_rows * 2;  # get double just in case everything has to come from one table, we won't be confused in 
    }
    else {
	$table_max_num_rows = "";
    }
    
    # ordering amoung multiple tables
    ($numPrimary, $primary_objects) = &get_primary_objects(\@primary_table_list, $fields, $where_clause, $primary_search_order, $startRows, $table_max_num_rows, $site, $dbh, $override, $search_language, $match_all_langs);
    
    &debug("load_objects: After get_primary_objects, should we do language objects?", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($search_language) {
	&debug("\tload_objects: YES", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$primary_search_order->{'extraprop'} = "body";
	($primary_objects, $numLanguage, $next_lang_startat, $lang_startat) = &get_language_primary_objects($language_build_routine, $primary_objects, $primary_search_order, $lang_startat, $table_max_num_rows, $site, $dbh, $override);
    }
    $numPrimary += $numLanguage;

    # Now that we have all of the tables data, we need to merge all search results together into one huge hashref
    $indx = 0;
    foreach $table (@primary_table_list) {
	while ($result =  $primary_objects->{$table}->{'res'}[$indx++]) {
	    &debug_hashref($result, 3, $site->{'debug_level'}, $site->{'debug_log'});
	    &debug("Adding this result", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $all_records->{$result->{'objid'}} = $result;
	}
	$indx = 0;
    }

    # Put the language sensitive stuff in there
    $indx = 0;
    while ($result =  $primary_objects->{'extraprop'}->{'res'}[$indx++]) {
	$all_records->{$result->{'objid'}} = $result;
    }

    $indx = 0;
    foreach $objid (keys %{$all_records}) {
	$result = $all_records->{$objid};
	push @results, $result;
    }

    @ordered_tables = sort {uc($a) cmp uc($b) } @primary_table_list;
    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
    # now go through and determine all of the start rows & get all of the extra data for it
    my $num_sec_rows = 0;
    my $num_sec_objs = 0;
    foreach $result (@results) {
	# get the extra secondary data for this object
	foreach $second_table (@secondary_table_list) {
	    ($result, $num_sec_rows) = &get_secondary_objects($result->{'objid'}, $result, $second_table, $site, $dbh);
	    if ((defined $num_sec_rows) && ($num_sec_rows > 0)) {
		$num_sec_objs++;
	    }
	    
	}
	$result = &set_fields_to_default_lang_val($result, $site->{'languageSensitive'}, $lang_order, $site);
	$result->{'sort_by'} = $result->{$primary_search_order->{$result->{'table'}}};
	push @return_records, $result;
    }

    @sorted_results = sort {uc($a->{'sort_by'}) cmp uc($b->{'sort_by'})} @return_records;
    if ($max_num_rows ne "") {
	if ($#sorted_results > $max_num_rows) {
	    @sorted_results = @sorted_results[0..$max_num_rows-1];
	}
    }

    # Set the next index
    foreach $result (@sorted_results) {
	foreach $table (@ordered_tables) {
	    if (defined $primary_objects->{$table}->{'search_keys'}->{$result->{'sort_by'}}) {
		$primary_objects->{$table}->{'start_row'} = $primary_objects->{$table}->{'start_row'} + 1;
		if ((!defined $max_rec_number->{$table}) || ($max_rec_number->{$table} < $result->{'record_number'})) {
		    $max_rec_number->{$table} = $result->{'record_number'};# flag the greatest record number for each result
		    }
	    }
	}
	# See if it is a language data, to add to lang_startrow
	if (defined $primary_objects->{'extraprop'}->{'search_keys'}->{$result->{'sort_by'}}) {
	    $lang_startat++;
	}
    }

    my ($rec_num, $tbl);
    &debug_hashref($primary_objects->{'mult_tables'}, 3, $site->{'debug_level'}, $site->{'debug_log'});
    # Gets the number of records that were selected this time
    	foreach $rec_num (keys %{$primary_objects->{'mult_tables'}}) {
	    $table = $primary_objects->{'mult_tables'}->{$rec_num};
	    if ($rec_num <= $max_rec_number->{$table}) {
		$primary_objects->{$table}->{'start_row'}++;
	    }
	}
#    }
#    else {
#	foreach $tbl (keys %{$primary_objects}) {
#	    $primary_objects->{$tbl}->{'start_row'}++;
#	}
#    }    


    foreach $table (@ordered_tables) {
	# we reached the end, so remove it from the list
	if ((!defined $primary_objects->{$table}->{'start_row'}) || 
	     (!defined $primary_objects->{$table}->{'has_more'})) {  # if there is no more, then don't give a start row for this
#	     (!defined $primary_objects->{$table}->{'next_row'})) {  # if there is no more, then don't give a start row for this
#	if (!defined $primary_objects->{$table}->{'start_row'}) {
	    $next_start_row .= ":";
	}
	else {
	    # If we didn't search the language, then add in the extraprop stuff
#	    if ($search_language == 0) {
#		if ((defined $num_sec_objs) && ($primary_objects->{$table}->{'start_row'} < $max_num_rows)) {
#		    $primary_objects->{$table}->{'start_row'} += $num_sec_objs;
#		}
#	    }
#	    $next_start_row .= $primary_objects->{$table}->{'start_row'} . ":";
	    $next_start_row .= $max_rec_number->{$table} . ":";

	}
    }


    # There were no more to get
    if ($max_num_rows ne "") {
	if ($numPrimary < $max_num_rows) {
	    $next_start_row = undef $next_start_row;
	}
    }

    # see if there are more to query
    $tmp = $next_start_row;
    $tmp =~ s/\://gms;
    if ((length $tmp) == 0) {
	$next_start_row = undef $next_start_row;
    }

    # get rid of last :
    chop $next_start_row;

#    return ($numPrimary, $next_start_row, $lang_startat, @sorted_results);
    return ($#sorted_results+1, $next_start_row, $lang_startat, @sorted_results);
}

##############################################################################
=pod

=head2 get_secondary_objects

get_secondary_objects - &get_secondary_objects($result->{'objid'}, $result, $second_table, $site, $dbh);

=over 4

=item DESCRIPTION

Since AllCommerce stores parts of objects in numerous tables, it is possible
to classify tables as primary and secondary. Primary tables are tables where the
object can only appear once (ie, objid is a primary key).  Secondary tables
are tables where you can define various attributes for the object.

This routine gets all of the associated data from the secondary table $table for an object
and stores them in the hashref that could already have infromation populated in it.
The way that this works best is to pass in the information that was already populated
from the primary table search.  It joins the data into one hashref.

=item INPUT

$objid - Objid of the object to search for secondary data from.

$object_results - Results so far, information containing all of the data from the
                  primary tables.

$table - Name of the secondary table to retreive the information for.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_results - Results with secondary data inserted and formated in appropriate
                  order.

$numRows - Number of rows found

=item NOTES

Each secondary table must have it's own format subroutine, named "format_<table>_record",
where <table> is the name of the table.  This tells us how to format the records.  It is
necessary to provide a unique path to all of the data so that we do not overwrite anything.
It is also necessary to take into consideration language sensitive data.  See 
format_price_record and format_extraprop_record as examples and details on the parameters
that must be passed in to these routines.

=back

=cut
###############################################################################
sub get_secondary_objects {
    my ($objid) = shift;
    my ($object_results) = shift;
    my ($table) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $numRows, @res, $select_hashref, $format_subroutine_name);
    my ($where_indx, $format_subroutine);
    $select_hashref = {};
	
    # set up the table
    if ((!defined $table) || ($table eq "")) {
	$select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    }
    else {
	$select_hashref->{'TABLE'}->{'NAME'} = $table;
    }
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    $q = &build_select($select_hashref, $site, $dbh, $site->{'override_permissions'});
    @res = undef @res;
    $numRows = &query_getAll($q, $site, $dbh, \@res);

    if ($numRows > 0) {
	$format_subroutine_name = "format_" . $table . "_record";
	$format_subroutine = \&$format_subroutine_name;
	$object_results = &$format_subroutine($object_results, $site, $numRows, \@res, "");
    }
    return($object_results, $numRows);
}

sub get_language_primary_objects {
    my ($query_routine) = shift;
    my ($primary_results) = shift;
    my ($order_by) = shift;
    my ($start_at_record) = shift;
    my ($max_num_rows) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($tables, $fields, $where, $need_more, @languages, $lang, $lang_search_key);
    my ($table, @all_tables, $sql_hashref, $q, $field, @all_fields);
    my ($indx, @res, $numRows, $nextRow, $num_language_rows);
    my ($object_hashref, $primary_tables, $primary_fields, $primary_where);
    my ($result, $already_found, $primary_table, @load_these_objects, $objid);
    my ($res_indx, $next_start_at_record, @tables, $lang_startat);
    my ($previous_start, @start_recs);

    &debug("get_language_primary_objects: In here with start: $start_at_record, max: $max_num_rows", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # Get the query that we are looking for
    ($tables, $fields, $where, $primary_tables, $primary_fields, $primary_where) = &$query_routine($site->{'this_search_key'}, $site->{'lang_query_params'}, $site);

    # If there are | in the start record, then this is not the first page, so we need to grab the last start record for this start record
	if ($start_at_record =~ /\|/) {
	    @start_recs = split /\|/, $start_at_record;
	    $previous_start = $start_at_record;
	    $start_at_record = $start_recs[$#start_recs];  # Take the last one
	}

    $lang_startat = $start_at_record;
    @all_tables = split /\,/, $tables;
    
    # Build the query for each table
    foreach $table (@all_tables) {
	$sql_hashref = {};
	$sql_hashref->{'statement'} = "select";
	$sql_hashref->{'TABLE'}->{'NAME'} = $table;
	$sql_hashref->{'TABLE'}->{'DISTINCT'} = "DISTINCT";
	if ($fields eq "") {
	    $fields = "*";
	}
	@all_fields = split /\,/, $fields;
	foreach $field (@all_fields) {
	    $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;
	}
	$indx = 0;
	foreach $where (@$where) {
	    $sql_hashref->{'TABLE'}->{'WHERE'}[$indx++] = $where;
	}

	if (defined $order_by->{$table}) {
	    $sql_hashref->{'TABLE'}->{'ORDER'} = $order_by->{$table};
	}
	$q = &build_sql($sql_hashref, $site, $dbh, $override);

	# Loop through grabbing data until we run out of records, or until we have $max_num_rows
	$next_start_at_record = $start_at_record;
	$num_language_rows = 0;
	my $continue = "yes";
	while ($continue eq "yes") {
#	while ((defined $next_start_at_record) && ($num_language_rows < $max_num_rows)) {
	    @res = undef @res;
	    ($numRows, $next_start_at_record) = &query_getRows($q, $site, $dbh, $next_start_at_record, $max_num_rows-$num_language_rows, \@res);
	    if ($numRows > 0) {
		$num_language_rows += $numRows;
		$need_more = 0;
		foreach $result (@res) {
		    $already_found = 0;
		    if (defined $primary_results->{'objids'}->{$result->{'objid'}}) {
			$num_language_rows--;  # remove it from the rows, as we already have this one, flag to keep looking
			$lang_startat++; # Increment so that we don't look at this one again
		    }

		    else {
			$object_hashref = {};
			$object_hashref = &load_default_object($result->{'objid'}, $primary_tables, $primary_fields, $primary_where, $site, $dbh);
			# See if it exists in the primary language in another table
			if ($result->{'language'} ne $site->{'om_render_lang'}) {
			    if (defined $object_hashref->{$site->{'om_render_lang'}}->{$order_by->{$object_hashref->{'table'}}}) {
				$num_language_rows--;  # remove it from the rows, as we already have this one, flag to keep looking
				$lang_startat++; # Increment so that we don't look at this one again
			    }
			    else {  # add it
				$primary_results->{'extraprop'}->{'start_row'} = $start_at_record;
				$primary_results->{'extraprop'}->{'search_keys'}->{$object_hashref->{$order_by->{$object_hashref->{'table'}}}} = 
				    $object_hashref->{$order_by->{$object_hashref->{'table'}}};
				$primary_results->{'extraprop'}->{'res'}[$res_indx++] = $object_hashref;
				$primary_results->{'objids'}->{$object_hashref->{'objid'}} = $object_hashref->{'objid'};
			    }
			}
			else {  # add it
			    $primary_results->{'extraprop'}->{'start_row'} = $start_at_record;
			    $primary_results->{'extraprop'}->{'search_keys'}->{$object_hashref->{$order_by->{$object_hashref->{'table'}}}} = 
				$object_hashref->{$order_by->{$object_hashref->{'table'}}};
			    $primary_results->{'extraprop'}->{'res'}[$res_indx++] = $object_hashref;
			    $primary_results->{'objids'}->{$object_hashref->{'objid'}} = $object_hashref->{'objid'};
			}
		    }
		}
	    }
	    if (($max_num_rows ne "") && ($num_language_rows >= $max_num_rows)) {
		$continue = "no";
	    }

	
	    if (!defined $next_start_at_record) {
		$continue = "no";
	    }

	}
    }
    # Append all previous lang stuff for back button
#    if (defined $previous_start) {
#	$lang_startat = $previous_start . "|" . $lang_startat;
#   }
    return ($primary_results, $num_language_rows, $next_start_at_record, $lang_startat);
}


##############################################################################
=pod

=head2 get_primary_objects

get_primary_objects - &get_primary_objects(\@primary_table_list, $fields, $where_clause, $primary_search_order, $startRows, $max_num_rows, $site, $dbh, $override);

=over 4

=item DESCRIPTION

Since AllCommerce stores parts of objects in numerous tables, it is possible
to classify tables as primary and secondary. Primary tables are tables where the
object can only appear once (ie, objid is a primary key).  Secondary tables
are tables where you can define various attributes for the object.

This routine gets the $fields from the primary tables $p_tables, that match the
$where_clause in $search_order starting at $start_at_record and returns
$max_num_rows at the maximum.

=item INPUT

$p_tables - pointer to an array of primary table names to check for the $fields in.

$fields - comma delimited list of fieldnames to retrieve from $p_tables

$where_clause - pointer to an array, where clause built using &build_where_hash

$search_order - Hashref with primary key being the table name.  If it is defined
                and a field is associated with the table name, this is the 
                field to use for ordering for the table.

$start_at_record - pointer to an array, each index is the row number to start
                   at for each table. $p_tables is sorted into alphabetical order 
                   and then this array is mapped to the locations in $p_tables to
                   indicate the next row to start on for EACH table defined.

$max_num_rows - Total maximum number of rows to retreive.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$override - If set, override permission check.

$check_render_lang      - 1 if the calling routine plans on looking in other tables,
                          so ignore results here if they are not in the proper language
                          0 if we want all of the results, regardless of language
=item OUTPUT

$all_rows - Number of rows returned, combining the results from ALL tables

$all_results - Hashref in the following form:
     
    $all_results->{$table}->{'res'}[0..n] = $row_hashref;
    where 
           $table is the name of the table that it found the information in
           'res' is the key to the array of results in sorted order
            0..n is the indexes into the array for each row returned
           $row_hashref is the information for the object retrieved, language seperated

    $all_results->{$table}->{'num_rows'} = The number of rows returned from this table

    $all_results->{$table}->{'next_row'} = The next starting row for this table

=item NOTES

=back

=cut
###############################################################################
sub get_primary_objects {
    my ($p_tables) = shift;
    my ($fields) = shift;
    my ($where_clause) = shift;
    my ($search_order) = shift;
    my ($start_at_record) = shift;
    my ($max_num_rows) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;
    my ($search_language) = shift;
    my ($match_all_lang) = shift;

    my ($table_fields, $select_hashref, $q, @all_fields,  $object_hashref, $table);
    my ($result_hashref, $numRows, @ordered_tables, $i, $nextRow, $newRow, $next_start_row, $all_results);
    my ($table_list, @start_at, @tmp_start, $start, @query_tables);
    my ($field, $where_hashref, $where_indx, @res, $res_indx, @all_primary_results);
    my (@sorted_results, $result, $indx, $tmp, $all_rows, $all_records, $where_field, $lang_order);
    my ($start_offset, $record_number, @start_recs, $previous_start);

    &debug("get_primary_objects: In here with $start_at_record, $max_num_rows, search: $search_language, match: $match_all_lang", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # sort the table names alphabetically
    @ordered_tables = sort { uc($a) cmp uc($b) } @$p_tables;

    # split the start at numbers, so we know which one is which by comparing alphabetically 
    # if it is nothing, then we start at 0 for every table
    if ($start_at_record eq "") {
	for ($i = 0; $i <= $#ordered_tables; $i++) {
	    $start_at[$i] = 0;
	    push @query_tables, $ordered_tables[$i];
	}
    }
    else {
	# If there are | in the start record, then this is not the first page, so we need to grab the last start record for this start record
	if ($start_at_record =~ /\|/) {
	    @start_recs = split /\|/, $start_at_record;
	    $previous_start = $start_at_record;
	    $start_at_record = $start_recs[$#start_recs];  # Take the last one
	}

	@tmp_start = split /\:/, $start_at_record;
	$i = 0;
        # see if there are any tables to be removed from the list
	$table_list = join "", @ordered_tables;
	foreach $start (@tmp_start) {
	    # only include it if there are still rows to query from
	    if ($start ne "") {
		push @query_tables, $ordered_tables[$i];
		push @start_at, $start;
	    }
	    $i++;
	}
    }


    $i = $all_rows = 0;
    foreach $table (@query_tables) {
	$table_fields = &get_field_names($table, $site, $dbh);
	#surround by , to make sure that we match the entire fieldname (don't match on uname when you have name as a field!)
	$table_fields = ",$table_fields,";
	
	$select_hashref = {};
	
	# set up the table
	if ((!defined $table) || ($table eq "")) {
	    $select_hashref->{'TABLE'}->{'NAME'} = "things";
	}
	else {
	    $select_hashref->{'TABLE'}->{'NAME'} = $table;
	}
	
	# set up the fields
	if ((!defined $fields) || ($fields eq "")) {
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	}
	else {
	    $fields =~ s/ //gms;
	    @all_fields = split /\,/, $fields;
	    foreach $field (@all_fields) {
		if ($table_fields =~ /,$field,/) {
		    $select_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;	    
		}
	    }
	}
	
	# set up the where clause
	$where_indx = 0;
	foreach $where_hashref (@$where_clause) {
	    $where_field = $where_hashref->{'field'};
	    $where_field =~ s/\(//gms;  # get rid of starting brackets if there are any
	    if ($table_fields =~ /,$where_field,/) {
		$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
	    }
	}

	# set up the order by
	if ($search_order->{$table} ne "") {
	    $select_hashref->{'TABLE'}->{'ORDER'} = $search_order->{$table};
	}
	
	$q = &build_select($select_hashref, $site, $dbh, $override);
	&debug("get_primary_objects:  Executing $q", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug("get_primary_objects:  numRows = $numRows, max_num_rows = $max_num_rows, nextRow = $nextRow", 3, $site->{'debug_level'}, $site->{'debug_log'});

	# Need to do this in a loop until we have the correct amount of rows, or we are done with this
        # table...
	$start_offset = $start_at[$i];
	$nextRow = 0;
	my $continue = "yes";
	$res_indx = 0;
	$record_number = $start_offset;  # The number of the record that we are on
	while ($continue eq "yes") {
	    @res = undef @res;
	    ($numRows, $nextRow) = &query_getRows($q, $site, $dbh, $start_offset, $max_num_rows, \@res);
	    &debug("get_primary_objects:  Found $numRows rows", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $start_offset += $numRows;
	    
	    $all_rows = $all_rows + $numRows;
	    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
	    if ($numRows > 0) {
		foreach $result_hashref (@res) {
		    $record_number++;
		    # format the result
		    if (!defined $all_results->{$table}->{'start_row'}) {
			$all_results->{$table}->{'start_row'} = $start_at[$i];  # initialize it
		    }

		    # Check if we are allowed to see this one.
		    if (!defined $site->{'override_permissions'}) {
			if (!&check_all_attribute_permissions($result_hashref, "r", $site)) {
			    $numRows--;
			    $all_rows--;  # not taking it, so remove one of the row results
			    next;
			}
		    }

		    if ($search_language) {
			# Dont' match all languages, so see if it is defined somewhere else
			if ($match_all_lang == 0) {
			    if (&defined_in_render_language($result_hashref->{'objid'}, "", $search_order->{$table}, $site->{'om_render_lang'}, $site, $dbh)) {

				$all_rows--;  # not taking it, so remove one of the row results
				# Record the record number of all objects which appear in multiple tables.  If we use a record number
				# greater than this record number, then we have to increase the startrow parameter for the $table by one,
				# to flag to skip it next time
				$all_results->{'mult_tables'}->{$record_number} = $table;
				$start_offset += 1;  # Add an offset so the next time we go though the loop, we skip this record
				next;
			    }
			}
		    }

		    $all_results->{$table}->{'num_rows'} = $numRows;
		    $all_results->{$table}->{'next_row'} = $nextRow;
		    my $min_num_rows = $max_num_rows / 2;
#		    if (((defined $numRows) && ($numRows > $min_num_rows)) && (defined $nextRow)) {
		    if ((defined $numRows) && ($numRows > $min_num_rows)) {
			$all_results->{$table}->{'has_more'} = "yes";
		    }

		    $object_hashref = {};
		    $object_hashref = &format_primary_record($object_hashref, $table, $site, $result_hashref);
		    if ($site->{'languageSensitive'} =~ /$search_order->{$table}/) {
			# use these to figure out what data came from what table
			$all_results->{$table}->{'search_keys'}->{$object_hashref->{$object_hashref->{'language'}}->{$search_order->{$table}}} = 
			    $object_hashref->{$object_hashref->{'language'}}->{$search_order->{$table}};
		    }
		    else {
			# use these to figure out what data came from what table
			$all_results->{$table}->{'search_keys'}->{$object_hashref->{$search_order->{$table}}} = $object_hashref->{$search_order->{$table}};
		    }
		    $object_hashref->{'record_number'} = $record_number;
		    $all_results->{$table}->{'res'}[$res_indx++] = $object_hashref;
		    $all_results->{'objids'}->{$object_hashref->{'objid'}} = $object_hashref->{'objid'};
		    
		}
	    }
#	    if (($max_num_rows ne "") && ($numRows >= $max_num_rows)) {
	    if (($max_num_rows ne "") && ($all_rows >= $max_num_rows)) {
		$continue = "no";
	    }

	    if (!defined $nextRow) {
		$continue = "no";
	    }

	}
	$i++;  # increment the counter now
    }
    &debug("get_primary_objects: returning $all_rows results", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($all_rows, $all_results);
}

sub defined_in_render_language {
    my ($objid) = shift;
    my ($object_hashref) = shift;
    my ($field) = shift;
    my ($language) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($select_hashref, $q, $numRows, @res);

    # query to find out if it exists
    if (!$object_hashref) {
	$select_hashref = {};
	$select_hashref->{'statement'} = "SELECT";
	$select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
	$select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $objid, "and");
	$select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("fieldname", $field, "and");
	$select_hashref->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("language", $language, "and");
	$q = &build_sql($select_hashref, $site, $dbh, 1);
	$numRows = &query_getAll($q, $site, $dbh, \@res);
	if ($numRows == 1) {
	    return "true";
	}
	else {
	    return;
	}
    }
}



##############################################################################
=pod

=head2 format_primary_record

format_primary_record - &format_primary_record($object_hashref, "contacts", $site, $primarys_hashref);

=over 4

=item DESCRIPTION

To format the primary record returned in a hashref, with each key being a fieldname
and the value being the value stored in the database.  If the field is multi-dimentional,
the value is an internal hashref, with the keys being the fieldnames and the values
being the database values referenced by the dimentions.

=item INPUT

$object_hashref - Hashref to store the extraprop records in, in proper format.  Could
                  already be populated in some areas.


$table - Name of the table which the data came from

$site - A hashref containing information required throughout about the current
        site and session

$primarys_hashref - The hashref from the primarys query, the data for the object that was
                  queried.

=item OUTPUT

$object_hashref - Hashref returned in proper format.  All language specific fields
                  are stored in 'lang' hashrefs, where 'lang' is the language that
                  they are defined for.

=item NOTES

=back

=cut
###############################################################################
sub format_primary_record {
    my ($object_hashref) = shift;
    my ($table) = shift;
    my ($site) = shift;
    my ($primarys_hashref) = shift;

    my ($field);

    # set the fields for the main primarys record, format $h->{'eng'}->{'fieldname'}
    &debug("In format_primary_record", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($primarys_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $field (keys %{$primarys_hashref}) {
	&debug("checking field $field, $primarys_hashref->{'objclass'}, $site->{'OBJCLASSES'}->{'attribute'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	if ($site->{'languageSensitive'} =~ /$field/) {
	    # This is just a hack for title of attributes class for this release.  
	    if ((defined $primarys_hashref->{'objclass'}) && ($primarys_hashref->{'objclass'} eq $site->{'OBJCLASSES'}->{'attribute'}) &&
		($field eq "title")) {
		$object_hashref->{$field} = $primarys_hashref->{$field};
	    }
	    else {
		$object_hashref->{$primarys_hashref->{'language'}}->{$field} = $primarys_hashref->{$field};
		if ($object_hashref->{'lang_keys'} !~ /$primarys_hashref->{'language'}/) {
		    $object_hashref->{'lang_keys'} .= "$primarys_hashref->{'language'},";
		}
	    }
	}
	else {
	    $object_hashref->{$field} = $primarys_hashref->{$field};
	}
    }

    $object_hashref->{$primarys_hashref->{'language'}}->{'table'} = $table;  # note the table which this came from
    $object_hashref->{'table'} = $table;  # note the table which this came from
    &debug_hashref($object_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $object_hashref;
}

##############################################################################
=pod

=head2 format_extraprop_record

format_extraprop_record - &format_extraprop_record($object_hashref, $site, $numExRows, \@ex_res);

=over 4

=item DESCRIPTION

To format the extraprop record returned in a hashref, with each key being a fieldname
and the value being the value stored in the database.  If the field is multi-dimentional,
the value is an internal hashref, with the keys being the fieldnames and the values
being the database values referenced by the dimentions.

=item INPUT

$object_hashref - Hashref to store the extraprop records in, in proper format.  Could
                  already be populated in some areas.


$site - A hashref containing information required throughout about the current
        site and session

$numExRows - Number of extraprop rows that need to be added to $object_hashref

$ex_res - Pointer to array of all of the extraprop records that matched the query.
          Each index is a hashref to a complete extraprop record.

$fields - Name of fields, comma delimited which should be included in the results.

=item OUTPUT

$object_hashref - Hashref returned in proper format.  All language specific fields
                  are stored in 'lang' hashrefs, where 'lang' is the language that
                  they are defined for.

=item NOTES

=back

=cut
###############################################################################
sub format_extraprop_record {
    my ($object_hashref) = shift;
    my ($site) = shift;
    my ($numExRows) = shift;
    my ($ex_res) = shift;
    my ($fields) = shift;

    my ($index);

    for ($index = 0; $index < $numExRows; $index++) {
	if ($$ex_res[$index]->{'body'} ne "") {
	    # only add this if it is one of the requested fields
	    if (($fields eq "") || ($fields =~ /$$ex_res[$index]->{'fieldname'}/)) {
		if ($site->{'languageSensitive'} =~ /$$ex_res[$index]->{'fieldname'}/) {
		    $object_hashref->{$$ex_res[$index]->{'language'}}->{$$ex_res[$index]->{'fieldname'}} = $$ex_res[$index]->{'body'};
		    if ($object_hashref->{'lang_keys'} !~ /$$ex_res[$index]->{'language'}/) {
			$object_hashref->{'lang_keys'} .= "$$ex_res[$index]->{'language'},";
		    }
		}
		else {
		    $object_hashref->{$$ex_res[$index]->{'fieldname'}} = $$ex_res[$index]->{'body'};
		}
	    }
	}
    }

    return $object_hashref;
}

##############################################################################
=pod

=head2 format_prices_record

format_prices_record - &format_prices_record($object_hashref, $site, $numPrices, \@prices);

=over 4

=item DESCRIPTION

To format the prices records returned in a hashref, under the 'prices' key.  Each
value is in the format:

        $object_hashref->{'prices'}->{#}->{$field} 
            where # is a unique number starting at 0, for each row of price records.

=item INPUT

$object_hashref - Hashref to store the extraprop records in, in proper format.  Could
                  already be populated in some areas.


$site - A hashref containing information required throughout about the current
        site and session

$numExRows - Number of extraprop rows that need to be added to $object_hashref

$ex_res - Pointer to array of all of the extraprop records that matched the query.
          Each index is a hashref to a complete extraprop record.

=item OUTPUT

$object_hashref - Hashref returned in proper format.  All language specific fields
                  are stored in 'lang' hashrefs, where 'lang' is the language that
                  they are defined for.

=item NOTES

=back

=cut
###############################################################################
sub format_prices_record {
    my ($object_hashref) = shift;
    my ($site) = shift;
    my ($numPriceRows) = shift;
    my ($price_res) = shift;

    my ($index, $price_record, $field);

    for ($index = 0; $index < $numPriceRows; $index++) {
	$price_record = $$price_res[$index];
	foreach $field (keys %{$price_record}) {
	    $object_hashref->{'prices'}->{$index}->{$field} = $price_record->{$field};
	}
    }
    return $object_hashref;
}

##############################################################################
=pod

=head2 format_inventory_record

format_inventory_record - &format_inventory_record($object_hashref, $site, $numInventorys, \@inventory);

=over 4

=item DESCRIPTION

To format the inventory records returned in a hashref, under the 'inventory' key.  Each
value is in the format:

    $object_hashref->{'inventory'}->{'wh'} = $hashref;
    $hashref->{$field} = $value;

=item INPUT

$object_hashref - Hashref to store the inventory records in, in proper format.  Could
                  already be populated in some areas.


$site - A hashref containing information required throughout about the current
        site and session

$numInventoryRows - Number of inventory rows that need to be added to $object_hashref

$inventory_res - Pointer to array of all of the extraprop records that matched the query.
          Each index is a hashref to a complete extraprop record.

=item OUTPUT

$object_hashref - Hashref returned in proper format.  

       $object_hashref->{'inventory'}->{'wh'} = $hashref;
       $hashref->{$field} = $value;

=item NOTES

=back

=cut
###############################################################################
sub format_inventory_record {
    my ($object_hashref) = shift;
    my ($site) = shift;
    my ($numInventoryRows) = shift;
    my ($inventory_res) = shift;

    my ($index, $inventory_record, $field);

    for ($index = 0; $index < $numInventoryRows; $index++) {
	$inventory_record = $$inventory_res[$index];
	foreach $field (keys %{$inventory_record}) {
	    $object_hashref->{'inventory'}->{$inventory_record->{'wh'}}->{$field} = $inventory_record->{$field};
	}
    }
    return $object_hashref;
}

##############################################################################
=pod

=head2 select_count

select_count -  &select_count($sql_hashref, $site, $dbh, $override);

=over 4

=item DESCRIPTION

Check user permissions for the desired SQL count statement.  If the user passes
permissions check, then Create a SQL count statement from the $sql_hashref.
Return the SQL count statement, or undef if permissions are not met.

$sql_hashref is defined by (items with a * in front of them are optional):
    $sql_hashref->{'statement'} = "select count(*)"
    $sql_hashref->{'TABLE'}->{'NAME'} = name_of_table;  Name of table to do SQL on
    * $sql_hashref->{'TABLE'}->{'WHERE'}->{$field} = value:and|or; WHERE clause
         See NOTE for db_layer.pm to get exact syntax of WHERE clause

=item INPUT

$sql_hashref - Hashref describing the count SQL statement to generate

$site - Site hashref

$dbh - Current active database handler

$override - If set, then override permission checks

=item OUTPUT

$count - SQL statement if permissions permit
         undef if the user does not have permissions to execute the statement.

=item NOTES

=back

=cut
###############################################################################
sub select_count
{
    my ($sql_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($override) = shift;

    my ($field, $perm_check, $count, $comparison);

    $count = "";

    if (!defined ($perm_check = &check_permissions($sql_hashref, $site, $dbh, $override))) {
        &debug("Insufficient permissions to build count statement", 2, $site->{'debug_level'}, $site->{'debug_log'});
        return;
    }

    # start off an insert statement
    $count = "SELECT COUNT(*) ";

    $count .= " FROM $sql_hashref->{'TABLE'}->{'NAME'} ";

    if (exists $sql_hashref->{'TABLE'}->{'WHERE'}) {
        $count .= "WHERE ";
        foreach $field (keys %{$sql_hashref->{'TABLE'}->{'WHERE'}}) {
            $comparison = &build_where_clause_element($field, $sql_hashref->{'TABLE'}->{'WHERE'}->{$field});
            $count .= $comparison;
        }
    }

    $count =~ s/OR $//gms; # get rid of the last OR
    $count =~ s/AND $//gms; # get rid of the last AND

    &debug("Returning $count", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $count;
}
##############################################################################
=pod

=head2 om_db

om_db - &om_db();

=over 4

=item DESCRIPTION

Create a hashref to be set as $om_db.  This used to live in om_db.pm,
however, all database specific information now is contained in this library.

=item INPUT

None

=item OUTPUT

$h - om_db hashref

=item NOTES

=back

=cut
###############################################################################
sub om_db {
   # returns a hashref to be set as $om_db
   my($h);
   $h={};

   my $ORACLE_SID;

   ##
   # Client agnostic database routines that are db specific.
   #
   $h = {

      db_site_config  => {

         mysql          => "DBI:mysql:\$h->{\'site_database\'}:\$h->{\'site_database_serv\'}:\$h->{\'site_database_port\'}",
         Pg             => "DBI:Pg:dbname=\$h->{\'site_database\'};port=\$h->{\'site_database_port\'};host=\$h->{\'site_database_serv\'}",
         Oracle         => "DBI:Oracle:host=\$h->{\'site_database_serv\'};sid=$ORACLE_SID",
         informix       => "DBI:Informix:\$h->{\'site_database\'}\@\$h->{\'site_database_serv\'}",
         sybase         => "DBI:Sybase:server=\$h->{\'site_database_serv\'},database=\$h->{\'site_database\'}",
         db2            => "DBI:DB2:\$h->{\'site_database\'}",
         msql           => "DBI:mSQL:\$h->{\'site_database\'}:\$h->{\'site_database_serv\'}:\$h->{\'site_database_port\'}",
      },

      db_site_connect  => {

         mysql          => "DBI:mysql:\$h->{\'site_database\'}:\$h->{\'site_database_serv\'}:\$h->{\'site_database_port\'}",
         Pg             => "DBI:Pg:dbname=\$h->{\'site_database\'};port=\$h->{\'site_database_port\'};host=\$h->{\'site_database_serv\'}",
         Oracle         => "DBI:Oracle:host=\$h->{\'site_database_serv\'};sid=$ORACLE_SID",
         informix       => "DBI:Informix:\$h->{\'site_database\'}\@\$h->{\'site_database_serv\'}",
         sybase         => "DBI:Sybase:server=\$h->{\'site_database_serv\'},database=\$h->{\'site_database\'}",
         db2            => "DBI:DB2:\$h->{\'site_database\'}",
         msql           => "DBI:mSQL:\$h->{\'site_database\'}:\$h->{\'site_database_serv\'}:\$h->{\'site_database_port\'}",
      },

   }; # done with  $h

   return $h;

} #/om_db

1;




