#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

use strict;

##############################################################################
=pod

=head1 language.pm

language.pm - Subroutines for dealing with multiple language support

=head1 DESCRIPTION

This module is written to aid in the support of multiple languages.

=head1 NOTES & BUGS

At a minimum, the calling routine must require the following:
    language.pm
    db_layer.pm
    permission_layer.pm
    formtools.pm (&gen_select)

=cut

##############################################################################

##############################################################################
=pod

=head2 getLanguageOrder

getLanguageOrder - &getLanguageOrder("eng", $site);

=over 4

=item DESCRIPTION

Returns the order of the language importance, as defined by site.pm and the
current language that is in use.

=item INPUT

$session_lang - The language that the form is currently rendered in

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$order - order of the lanugage importance, session language first and then
         the order that the other languages were defined in site.pm.

=item NOTES

=back

=cut
###############################################################################
sub getLanguageOrder {
    my ($session_lang) = shift;
    my ($site) = shift;

    my ($order);

    $order = "$session_lang,$site->{'om_locale_lang_def'},$site->{'om_locale_lang_act'}";
    return ($order);
}

##############################################################################
=pod

=head2 get_language_pulldown

get_language_pulldown - &get_language_pulldown($site);

=over 4

=item DESCRIPTION

To generate a pulldown to use for all languages that can be used for this particular
site.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$pulldown - A SELECT HTML object, with all languages that are supported by the site.

=item NOTES

=back

=cut
###############################################################################
sub get_language_pulldown {
    my ($site) = shift;

    my (@langs, $result, $longlang, $language_select, $lang, @langlist);

    @langs = split /\,/, $site->{'om_locale_lang_act'};
    push @langs, $site->{'om_locale_lang_def'};
    
    foreach $lang (@langs) {
	next if ( ! $lang);
	$result=&om_iso_639($lang);
	$longlang=(split/\t/, $result)[3]; 
	push @langlist, "$lang|$longlang";
    }
    $language_select = gen_select( "$site->{'om_locale_lang_def'}",2,@langlist );
    return ($language_select);
}


##############################################################################
=pod

=head2 get_messages

get_messages - &get_messages("eng", $scriptname, $site);

=over 4

=item DESCRIPTION

To read in the script_msg hashref for $language.

=item INPUT

$language - The language to read in the script_msg messages for

$scriptname - The name of the script, to find the file to read in

$site - The site hashref, use for relative path and message sub

=item OUTPUT

$script_msg - Pointer to the script_msg hash

=item NOTES

=back

=cut
###############################################################################
sub get_messages {
    my ($language) = shift;
    my ($scriptname) = shift;
    my ($site) = shift;

    my ($script_msg, $filename);
    $filename = $scriptname . ".messages";

    require "$site->{'relpath'}/bits/$language/system/$filename";

    $script_msg = &message($site);
    return $script_msg;
}

##############################################################################
=pod

=head2 get_language_sensitive_fields

get_language_sensitive_fields - &get_language_sensitive_fields($site, $dbh);

=over 4

=item DESCRIPTION

To query the database to find the list of language sensitive fields.  These
fields are a complete list of all fields which should be considered language
sensitive and can be defined once per language that the object is
defined in.

=item INPUT

$site - The site hashref, use for relative path and message sub

$dbh - Current active database handler

=item OUTPUT

$lang_sens - Language sensitive fields, seperated by ","

=item NOTES

This record is defined in extraprop with the fieldname ac_language_sens

=back

=cut
###############################################################################
sub get_language_sensitive_fields {
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $select_hashref, $numRes, @results);

    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("fieldname", "ac_language_sens", "and");
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRes = &query_getAll($q, $site, $dbh, \@results);
    if ($numRes > 0) {
	return ($results[0]->{'body'});
    }
    else {
	return;
    }
}

##############################################################################
=pod

=head2 update_language_sensitive_fields

update_language_sensitive_fields - &update_language_sensitive_fields($add, $remove, $site, $dbh);

=over 4

=item DESCRIPTION

To update the database record defining the list of language sensitive fields.
All fields in $add will be added to the list, comma seperated and all
fields in the $remove will be removed from the list.

=item INPUT

$add - Comma seperated list of fields to add to the language sensistive record

$remove - Comma seperated list of fields to remove from the language sensitive 
          record

$site - The site hashref, use for relative path and message sub

$dbh - Current active database handler

=item OUTPUT

The ac_language_sens field is updated.

=item NOTES

This record is defined in extraprop with the fieldname ac_language_sens

=back

=cut
###############################################################################
sub update_language_sensitive_fields {
    my ($add) = shift;
    my ($remove) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $update_hashref, $numRes, @results);
    my ($languageSensitive, @add_fields, @rm_fields);
    my ($field);
    
    &debug("update_language_sensitive_fields: In with $add, $remove.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if (defined $site->{'languageSensitive'}) {
	$languageSensitive = $site->{'languageSensitive'};
    }
    else {
	$languageSensitive = &get_langauge_sensitive_fields($site, $dbh);
    }

    &debug("update_language_sensitive_fields: Language sensitive fields are $languageSensitive.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($remove ne "") {
	@rm_fields = split /\,/, $remove;
	foreach $field (@rm_fields) {
	    $languageSensitive =~ s/$field//gms;
	}
    }

    if ($add ne "") {
	@add_fields = split /\,/, $add;
	foreach $field (@add_fields) {
	    $languageSensitive .= ",$field";
	}
    }

    # get rid of double commas
    $languageSensitive =~ s/\,\,/\,/gms;
    
    &debug("update_language_sensitive_fields: New language field is $languageSensitive.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $update_hashref->{'statement'} = "UPDATE";
    $update_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $update_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $languageSensitive;
    $update_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("fieldname", "ac_language_sens", "and");
    $q = &build_sql($update_hashref, $site, $dbh, 1);
    if (!&query_execute($q, $site, $dbh)) {
	&debug("update_language_sensitive_fields: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
    }

    return;
}


sub printHashref {
    my ($hashref) = shift;
    my ($format) = shift;

    my ($key, $type);


    if (!defined $format) {
	$format = "";
    }
    foreach $key (keys %{$hashref}) {
	$type = ref $hashref->{$key};
	if ($type =~ /HASH/) {
	    print "$format $key $type\n";
	    print "$format-------------------------\n";
	    $format .= "\t";
	    &printHashref($hashref->{$key}, $format);  # recursive call for the internal hashref
	    chop $format;  # get rid of one \t from last recursive call
	}
	else {
	    print "$format $key is $hashref->{$key}\n";
	}
    }

}


##############################################################################
=pod

=head2 set_fields_to_default_lang_val

set_fields_to_default_lang_val - &set_fields_to_default_lang_val($obj_hashref, "title,brief", "eng,fra", $site);

=over 4

=item DESCRIPTION

Goes through the languages order, and sets the hash with the fields that are 
defined for the language in order of importance.  Could end up with a hash with 
all fields initialized, but each field could be in a different language.

=item INPUT

$lang_hashref - Hashref for an object, generated by loadObject().  Information
                for the object in all languages

$fields_to_default - If blank, default all fields, otherwise, just default
                     the fields specified.

$lang_order - Order of languages in order of importanced.  Set from getLanguageOrder

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$default_hashref - Hashref with a complete record of information for an object.  
                   The fields are defined in order of language importance, on 
                   a field by field basis.  Some of the fields could be in
                   English, and others in another language.  This is used for
                   displaying definitions in a previous language through the
                   administrator.

=item NOTES

=back

=cut
###############################################################################
sub set_fields_to_default_lang_val {
    my ($lang_hashref) = shift;
    my ($fields_to_default) = shift;   
    my ($lang_order) = shift;
    my ($site) = shift;

    my ($default_hashref, $field, $i);
    my (@languages, $lang);

    &debug("In set_field_to_default_lang_val with $fields_to_default $lang_order", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # if it is all fields, then set all languageSensitive fields to the default language value.
    if (($fields_to_default eq "*") || ($fields_to_default eq "")) {
	$fields_to_default = $site->{'languageSensitive'};
    }
    @languages = split /\,/, $lang_order;
    for ($i = 0; $i <= $#languages; $i++) {
	next if ($lang_hashref->{'lang_keys'} !~ /$languages[$i]/);  # does not have this language defined for this object yet
	foreach $field (keys %{$lang_hashref->{$languages[$i]}}) {
	    if ($fields_to_default =~ /$field/) {
		&debug("set_fields_to_default_lang_val:  Looking at $field, $lang_hashref->{$field}, $lang_hashref->{$languages[$i]}->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
		if ((!defined $lang_hashref->{$field}) && ($lang_hashref->{$languages[$i]}->{$field} ne "")) {
		    $lang_hashref->{$field} = $lang_hashref->{$languages[$i]}->{$field};
		    &debug("set_field_to_default_lang_val: Just set $field to $lang_hashref->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
		}
	    }
	}
    }

    return $lang_hashref;
}

1;
