#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

##############################################################################
=pod

=head1 permission_layer.pm

permission_layer.pm - Permissions layer library

=head1 DESCRIPTION

This library is used for the permission layer routines.

=head1 NOTES & BUGS

=cut

##############################################################################

##############################################################################

##############################################################################
=pod

=head2 check_permissions

check_permissions -

=over 4

=item DESCRIPTION

A general purpose subroutine to check the permissions of any SQL statement, 
based on the permissions set passed in. 

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" 		if the user is allowed to execute the result
undef		if user is NOT allowed to execute the result

=item NOTES

=back

=cut
###############################################################################
sub check_permissions {
	my ($sql_hashref,$site,$override) = @_;
	my $dbh;

	if ($override) { return "true"; }

	my ($statement) = uc $sql_hashref->{'statement'};

    if ($statement eq "SELECT") {
	return (&check_read_permissions($sql_hashref, $site, $dbh));
    }
    elsif ($statement eq "UPDATE") {
	return (&check_modify_permissions($sql_hashref, $site, $dbh));
    }
    elsif ($statement eq "INSERT") {
	return (&check_create_permissions($sql_hashref, $site, $dbh));
    }
    elsif ($statement eq "DELETE") {
	return (&check_delete_permissions($sql_hashref, $site, $dbh));
    }
    else {
	&debug("Cannot check permissions for SQL statement: '$sql_hashref->{'statement'}'", 1, $site->{'debug_level'}, $site->{'debug_log'});
    }
    return;
}
##############################################################################
=pod

=head2 check_results_permissions

check_results_permissions

=over 4

=item DESCRIPTION

STUB to interface with the database layer.  Needs to be written

=item INPUT

$results - pointer to an array of rows of records, each a hashref of all
           of the fields returned.

$type - Type of permission to check for.  Check for 's' and then this type.

$site - Site hashref

=item OUTPUT

"true" if the user is allowed to execute the results
undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub check_results_permissions {
    my ($results_array) = shift;
    my ($type) = shift;
    my ($site) = shift;

    return "true";
}

##############################################################################
=pod

=head2 check_objclass

check_objclass

=over 4

=item DESCRIPTION

STUB to interface with the database layer.  Needs to be written

=item INPUT

=item OUTPUT

"true" if the user is allowed to execute the results
undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub check_objclass {
    my ($row_hashref) = shift;
    my ($site) = shift;
    my ($type) = shift;

    return "true";
}


##############################################################################
=pod

=head2 check_row_fields_permissions

check_row_fields_permissions

=over 4

=item DESCRIPTION

STUB to interface with the database layer.  Needs to be written

=item INPUT

=item OUTPUT

"true" if the user is allowed to execute the results
undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub check_row_fields_permissions {
    my ($row_hashref) = shift;
    my ($site) = shift;
    my ($type) = shift;

    return "true";
}


##############################################################################

##############################################################################
=pod

=head2 filter_read_results

filter_read_results -

=over 4

=item DESCRIPTION

Since with certain select statements we cannot determine if we have permissions
to view the data or not, it is necessary to execute them, and then filter the
results. The results will be gone through, item by item, and stored in a 
filtered hashref which sorts the data by what type of permissions that is 
required. This will be helpful for admin_object, which will be required to know
what fields that the user has read permissions for and what fields the user has
write permissions for, to know what type of HTML object to place on the display
(input or not).


=item INPUT

$sql_results - pointer to an array of rows of records, each a hashref of all the
fields returned. 

$site - Site hashref with ->{'permissions'} key


=item OUTPUT

$filtered_sql_results - Hashref of SQL results, seperated into the following keys:
	'r' - items which have read only permissions
	'c' - items which have create permissions
	'm' - items which have modify permissions

=item NOTES

One record may be in various locations, depending on what permissions they
have for each attribute of the record.

For the time being, this is a lengthy full search and copy of the results.
This needs to be improved upon. External scripts should try to refrain from calling
select '*', or queries without specifying objclass in the where clause. The
amount of rows returned should be kept as small as possible

=back

=cut
###############################################################################
sub filter_read_results {
    my ($p_sql_results) = shift;
    my ($site) = shift;

    my ($row, $row_permission, $filtered_sql_results);

    if (!defined $p_sql_results) {
	&debug("Cannot filter_read_results, no data passed in.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    foreach $row (@$p_sql_results) {
	$row_permission = &filter_row_result($row, $site);
	if (defined $row_permission) {
	    $filtered_sql_results = &add_row($row->{'objid'}, 
					     $row_permission, 
					     $filtered_sql_results);
	}
    }

    return ($filtered_sql_results);
}

##############################################################################

##############################################################################
=pod

=head2 add_row

add_row -

=over 4

=item DESCRIPTION

Takes one hashref and add it's contents to the same locatoin in the other 
hashref.

=item INPUT

$objid - key to copy into the destination.

$source - Hashref to copy data from.

$destination - Hashref to copy data to.

=item OUTPUT

$destination - The hashref with the contents of the $source now in it.

=item NOTES

=back

=cut
###############################################################################
sub add_row {
    my ($objid) = shift;
    my ($source) = shift;
    my ($destination) = shift;

    my ($perm, $key);

    foreach $perm (keys %{$source}) {
	foreach $key (keys %{$source->{$perm}}) {
	    if (!defined $destination->{$perm}->{$objid}->{$key}) {
		$destination->{$perm}->{$objid}->{$key} = $source->{$perm}->{$key};
	    }
	}		
    }
    return ($destination);
}
##############################################################################

##############################################################################
=pod

=head2 filter_row_results

filter_row_results -

=over 4

=item DESCRIPTION

Takes one hashref defining a record, and filters it into 'r' 'c' 'm'
portions. 

 =item INPUT

$sql_result - hashref of the record

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

$filtered - Hashref of SQL results, sperated in to the following keys:
    'c' - items which have create permissions
    'm' - items which have modify permissions
    'r' - items which have read only permissions


=item NOTES


=back

=cut
###############################################################################

sub filter_row_result {
    my ($sql_result) = shift;
    my ($site) = shift;
    my ($filtered) = shift;  # undef initially, but here for recursive calls
    my ($master_key) = shift; #undef initially, but here for recursive calls

    my ($field, $perm_str, $type);

    foreach $field (keys %{$sql_result}) {
	$type = ref $sql_result->{$field};
	if ($type !~ /HASH/) {
	    $perm_str = &get_attribute_permissions($field, $site);
	    if ((!defined $perm_str) || ($perm_str =~ /x/)) {
		&debug("Removing $field since no permissions for it.", 3, $site->{'debug_level'}, $site->{'debug_log'});
		next;
	    }
	    elsif ($perm_str =~ /c/) {
		if (defined $master_key) {
		    $filtered->{'c'}->{$master_key}->{$field} = $sql_result->{$field};
		}
		else {
		    $filtered->{'c'}->{$field} = $sql_result->{$field};
		}
	    }
	    
	    elsif ($perm_str =~ /m/) {
		if (defined $master_key) {
		    $filtered->{'m'}->{$master_key}->{$field} = $sql_result->{$field};
		}
		else {
		    $filtered->{'m'}->{$field} = $sql_result->{$field};
		}
	    }
	    elsif ($perm_str =~ /r/) {
		if (defined $master_key) {
		    $filtered->{'r'}->{$master_key}->{$field} = $sql_result->{$field};
		}
		else {
		    $filtered->{'r'}->{$field} = $sql_result->{$field};
		}
	    }
	    else {
		&debug("Couldn't add $field anywhere.  Error!", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    }
	}
	else {  # recursive call to handle the inner hashrefs
	    $filtered = &filter_row_result($sql_result->{$field}, $site, $filtered, $field);
	}	    
    }
    return ($filtered);
}
##############################################################################

##############################################################################
=pod

=head2 get_hashref_contents

get_hashref_content -

=over 4

=item DESCRIPTION

To extract the hashref and all of its contents, expanding all hashrefs to
be found within the main hashref. Store the results in a giant string, which is
the contents of all hashrefs within hashrefs and so on. 

=item INPUT

$hashref - Hashref to print out.

$perm_key - Main key from previous occurance of call to get_hashref_contents.

$perms_sofar - If we have called a previous occurence, this is the permissions
so far. 


=item OUTPUT

$perms_sofar - Permissions hashref contents so far.

=item NOTES

=back

=cut
###############################################################################

sub get_hashref_contents {
    my ($hashref) = shift;
    my ($perm_key) = shift;
    my ($perms_sofar) = shift;

    my ($key, $type);

    if (!defined $perms_sofar) {
	$perms_sofar = "";
    }
    foreach $key (keys %{$hashref}) {
	$type = ref $hashref->{$key};
	if ($type =~ /HASH/) {
	    $perms_sofar = &printHashrefToFile($hashref->{$key}, $key, $perms_sofar);
	}
	else {
	    $perms_sofar .= "\$permission_hashref->{'$perm_key'}->{'$key'} = \"$hashref->{$key}\";\n";
	}
    }

    return $perms_sofar;
}
##############################################################################

##############################################################################
=pod

=head2 get_attribute_permissions

get_attribute_permissions -

=over 4

=item DESCRIPTION

Searches through the permissions hashref and finds ALL permissions
associated with a single attribute. The permissions string will consis of
the following letters in any order, depending on how it's permissions were
set up:

	r - has read access somewhere
	c - has create access
	m - has modify access
	d - has delete access
	x - should be excluded from everything

 =item INPUT

$attribute - Either a fieldname, or a class objid

$site - Site hashref with the ->{'permissions'} key

=item OUTPUT

$permission_str - A string of permissions found for this attribute

undef if it's not defined at all, meaning that it's like an 'x'

 =item NOTES

=back

=cut
###############################################################################

sub get_attribute_permissions {
    my ($attribute) = shift;
    my ($site) = shift;

    my ($permission_type, $attribute_name, $permission_str);
    my ($all_classes_perm);
 
    return if (!defined $attribute);
    return if ($attribute eq "");

    # if override, give all permissions
    if ($site->{'override_permissions'}) {
	return "cdmr";
    }
    $permission_str = "";
    foreach $permission_type (keys %{$site->{'permissions'}}) {
	next if ($permission_type eq "generate_triggers");
	$all_classes_perm = $site->{'permissions'}->{$permission_type}->{'*'};
	if (defined $all_classes_perm) {
	    $all_classes_perm = $site->{'permissions'}->{$permission_type}->{'*'};
	    if (($all_classes_perm =~ /\*/) || ($all_classes_perm =~ /$attribute/)) {
		$permission_str .= $permission_type;
	    }
	}
	    
	foreach $attribute_name (keys %{$site->{'permissions'}->{$permission_type}}) {
	    if ($attribute_name =~ /$attribute/) {
		$permission_str .= $permission_type;
	    }
	    if ($site->{'permissions'}->{$permission_type}->{$attribute_name} =~ /$attribute/) {
#	    if (($site->{'permissions'}->{$permission_type}->{$attribute_name} =~ /$attribute/) ||
#		($site->{'permissions'}->{$permission_type}->{$attribute_name} eq "*")) {
		$permission_str .= $permission_type;
	    }
	}
    }

    return ($permission_str);
}
##############################################################################

##############################################################################
=pod

=head2 check_all_attribute_permissions

check_all_attribute_permissions -

=over 4

=item DESCRIPTION

Checks the hashref which represents currecnt permissions for everything in
the SQL statements, and determine if the user has appropriate permissions
for executing the SQL. 

=item INPUT

$sql_permissions - Hashref of all attributes that were in the SQL
statement with the value being a string representing the permissions
available for the attribute.

$permission - Permission substring to look for in all of the attribute
values. 

 =item OUTPUT

"true"  if the user is allowed to execute the result; undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub check_all_attribute_permissions {
    my ($object_to_check) = shift;
    my ($permission) = shift;
    my ($site) = shift;
    
    my ($attribute, $attr_permission, $objclass_perm, $objtype_perm);

    if ($site->{'override_permissions'}) {
	return "true";
    }

    # Check the objclass
    if ((defined $object_to_check->{'objclass'}) && 
	($object_to_check->{'objclass'} ne "")) {
	$objclass_perm = &get_attribute_permissions($object_to_check->{'objclass'}, $site);
	if ($objclass_perm =~ /x/) {
	    &debug("check_all_attribute_permissions: Objclass $object_to_check->{'objclass'} has 'x' in permissions, excluded", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    return;
	}
    }

    # Check the objtype
    if ((defined $object_to_check->{'objtype'}) &&
	($object_to_check->{'objtype'} ne "")) {
	$objtype_perm = &get_attribute_permissions($object_to_check->{'objtype'}, $site);
	if ($objtype_perm =~ /x/) {
	    &debug("check_all_attribute_permissions: Objclass $object_to_check->{'objtype'} has 'x' in permissions, excluded", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    return;
	}
    }

    foreach $attribute (keys %{$object_to_check}) {

	$attr_permission = &get_attribute_permissions($attribute, $site);
	if ((defined $objtype_perm) && ($objtype_perm ne "")) {
	    $attr_permission .= $objtype_perm;
	}

	if ((defined $objclass_perm) && ($objclass_perm ne "")) {
	    $attr_permission .= $objclass_perm;
	}

	&debug("check_all_attribute_permissions:  $attr_permission is the perm", 3, $site->{'debug_level'}, $site->{'debug_log'});


	if ((!defined $attr_permission) || ($attr_permission =~ /x/)) {
	    return;
	}  
	if ($attr_permission !~ /$permission/) {
	    return;
	}          
    }

    &debug("check_all_attribute_permissions:  returning TRUE", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return "true";

}
##############################################################################

##############################################################################
=pod

=head2 get_all_attribute_permissions

get_all_attribute_permissions -

=over 4

=item DESCRIPTION

This function extracts all of the attributes from the SQl query, and gets
the attribute string for each attribute. It then returns the strings in a
hashref referenced by the attribute

 =item INPUT

$sql_hashref - Hashref defining the SQL query to execute

$site - Site hashref with the ->{'permissions'} key

 =item OUTPUT

$attribute_permissions - Hashref of attributes with the values being the
permission string for that attribute, as returned by
get_attribute_permissions.

 =item NOTES

=back

=cut
###############################################################################
sub get_all_attribute_permissions {
    my ($sql_hashref) = shift;
    my ($site) = shift;
    
    my ($attribute_permissions,  @where_array, $attribute_hash, $attribute);
    
    if (defined $sql_hashref->{'TABLE'}->{'FIELDS'}) {
	foreach $attribute (keys %{$ sql_hashref->{'TABLE'}->{'FIELDS'}}) {
	    $attribute_permissions->{$attribute} = 
		&get_attribute_permissions($attribute, $site);
	}
    }
    if (defined $sql_hashref->{'TABLE'}->{'WHERE'}) {
	@where_array = $sql_hashref->{'TABLE'}->{'WHERE'};
	foreach $attribute_hash (@where_array) {
	    if ($attribute_hash->{'field'} eq "objclass") {
		$attribute_permissions->{$attribute} = 
		    &get_attribute_permissions($attribute_hash->{'value'}, $site);
	    }
	    else {
		$attribute_permissions->{$attribute} = 
		    &get_attribute_permissions($attribute_hash->{'field'}, $site);
	    }
	}
    }

    return $attribute_permissions;
}
##############################################################################

##############################################################################
=pod

=head2 get_user_permissions

get_user_permissions -

=over 4

=item DESCRIPTION

This is the only function which retreives the permissions information from the database.
All other functions call on data which is a result of this function. 

=item INPUT

$user_objid - Customer id from the sessions table for the logged in user.

$site - Site hashref

$dbh - Current active database handler

$uperm_rows - pointer to an array used for output. Each index is one 'uperm' field of information

=item OUTPUT

$numPermRows = Number of permission records that are being returned to the calling routine.

$generate_trigger - 1 if the user has permissions to execute generate triggers; 0 otherwise.

=item NOTES

=back

=cut
###############################################################################
        
sub get_user_permissions {
	my ($user_objid) = shift;
	my ($site) = shift;
	my ($dbh) = shift;
	my ($uperm_rows) = shift;
	my ($uperm_objid, $uperm_gotten_so_far, $result);

	my $saved_user_objid = $user_objid;

	my $q = "SELECT body, fieldname FROM extraprop WHERE objid='$user_objid' AND fieldname='gen_trigs'";
	my (@sql_results);
	my $num = &query($q, $dbh, \@sql_results);
	my $generate_trigger = %{$sql_results[0]}->{'body'};

	$q = "SELECT body, fieldname FROM extraprop WHERE objid='$user_objid' AND fieldname='uperms'";
	$num = &query_getAll($q, $site, $dbh, \@sql_results);
	my %h_sql = %{$sql_results[0]};
	my @permission_sets = split(/\:/,$h_sql{'body'});
	my $numPermRows;
	foreach my $perm_sets (@permission_sets) {
		$user_objid = $perm_sets;
		$q = "SELECT objid, body, fieldname FROM extraprop WHERE objid='$user_objid'";
        	&query_getAll($q, $site, $dbh, \@sql_results);
		$uperm_objid = %{$sql_results[0]}->{'objid'};
		$uperm_gotten_so_far->{$uperm_objid} = $uperm_objid;
		$$uperm_rows[$numPermRows++] = %{$sql_results[0]}->{'body'};
	}

	@sql_results = undef @sql_results;
	# Get the individual uperm field, if there is one
	$q = "SELECT objid, body, fieldname FROM extraprop WHERE objid='$saved_user_objid' AND fieldname='uperm'";
	$num = &query_getAll($q, $site, $dbh, \@sql_results);
	if ($num > 0) {
	    foreach $result (@sql_results) {
		next if (defined $uperm_gotten_so_far->{$result->{'objid'}});
		$$uperm_rows[$numPermRows++] = $result->{'body'};
	    }
	}
	return ($numPermRows,$generate_trigger);
}
##############################################################################

##############################################################################
=pod

=head2 load_permission_row

load_permission_row -

=over 4

=item DESCRIPTION

This function takes a row of permissions (usually taken from the array returned by &get_user_permissions) and returns a hashref of permissions for each.
The row in tab delimited, as entered in the database.

=item INPUT

$row - one row from the body definition of the 'uperm' record.

$perm_hashref - hashref with all permissions already in memory.

$site - Site hashref with ->{'permissions'} key


=item OUTPUT

$perm_hashref - Hashref with all permissions up until this point loaded.

=item NOTES

=back

=cut
###############################################################################

sub load_permission_row {
	my ($row,$perm_hashref,$site) = @_;
	my ($attribute);
	my ($perm,$classes,$attributes);
	my @permission_set = split /\n/, $row;
	for $row (@permission_set) {
		($perm,$classes,$attributes) = split /\t/, $row;

		my @all_classes = split /\,/, $classes;
		my @all_attributes = split /\,/, $attributes;
		foreach my $class (@all_classes) {
			if (defined $perm_hashref->{$perm}->{$class}) {
				if ($perm_hashref->{$perm}->{$class} =~ /\*/) {  next $class; }	
				else {
					foreach $attribute (@all_attributes) {
						if ($perm_hashref->{$perm}->{$class} =~ /$attributes/) { next $attribute; }
						else { $perm_hashref->{$perm}->{$class} .= ",$attribute"; }
					}
				}
			}
			else { $perm_hashref->{$perm}->{$class} = $attributes; }
		}
	}
	return $perm_hashref;
}
##############################################################################

##############################################################################
=pod

=head2 load_user_permissions

load_user_permissions -

=over 4

=item DESCRIPTION

Get's the users permissions from a static file containing the &get_cache
function. The function uses a require statement to load the &get_cache
function. 


=item INPUT

$user_objid - Customer id from the sessions table for the logged in user.

$site - Site hashref

$dbh - Current active database handler

=item OUTPUT

$perm_hashref - Hashref with all permissions up until this point loaded. 

=item NOTES

=back

=cut
###############################################################################

sub load_user_permissions {
	my ($user_objid,$site,$dbh) = @_;
	my (@uperm_rows, $row,$perm_hashref);
	$row=0;
	&debug("load_user_permissions: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});
	my ($numPermissions, $gen_trig) =  &get_user_permissions($user_objid,$site,$dbh,\@uperm_rows);

	foreach my $permissions (@uperm_rows) {
	    &debug("load_user_permissions: Loading a permission row", 3, $site->{'debug_level'}, $site->{'debug_log'});
		$perm_hashref = &load_permission_row($permissions,$perm_hashref,$site);
	}
	$perm_hashref->{'generate_triggers'} = $gen_trig;
	&debug("load_user_permissions: Going to print the row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug_hashref($perm_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
	&print_permissions($site,$perm_hashref);
	
	return $perm_hashref;
}
##############################################################################

##############################################################################
=pod

=head2 print_permissions

print_permissions -

=over 4

=item DESCRIPTION

Takes the current permissions and prints them to a file. This file is then
called by other functions in order to reduce the cpu from coninuous query of
the database. 


=item INPUT

$site - Site hashref with ->{'permissions'} key

$perm_hashref - hashref with all permissions already in memory.

=item OUTPUT

/<permissions_path>/<username>.<sid>.pm

=item NOTES

The output file now contains the function &get_cache which is called from
&get_user_cache.

=back

=cut
###############################################################################

sub print_permissions {
	use Data::Dumper;
	$Data::Dumper::Indent=2;
	my ($site,$hashref) = @_;

	my $p_hash = Dumper $hashref;

	open PPO, ">$site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm";
	&debug("print_permissions: Going to print permissions to $site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm" , 3, $site->{'debug_level'}, $site->{'debug_log'});

	print PPO <<EOM;
	#!/usr/bin/perl

	# This file was generated automatically by the permissions scripts from 
	# the contents of the users permissions. Don't edit or remove this file.

	sub get_cache {
	my \$VAR1;

	$p_hash

	return \$VAR1;
	}
	1;
EOM
	close PPO;
}
##############################################################################

##############################################################################
=pod

=head2 rm_user_cache

rm_user_cache -

=over 4

=item DESCRIPTION

Removes the library file contaning the necessary function to load the
users permissions statically. 

=item INPUT

$site - Site hashref with ->{'permissions'} key


=item OUTPUT

=item NOTES

This file is removed when the user logs out. 

=back

=cut
###############################################################################

sub rm_user_cache {
	my $site = shift;

	if (-e "$site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm") {
		&debug("User cache removed: '$site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm'", 1, $site->{'debug_level'}, $site->{'debug_log'});
		unlink "$site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm";
	}
}
##############################################################################

##############################################################################
=pod

=head2 generate_triggers

generate_triggers -

=over 4

=item DESCRIPTION

This function is called to determine wether the user has permissions to
generate pages on the fly. 

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" if the user is allowed to execute the result; undef otherwise.

=item NOTES

=back

=cut
###############################################################################


sub check_generate_triggers {
	my ($sql_hashref, $site) = @_;

	if ($site->{'permissions'}->{'generate_triggers'} == 1) {
		return "true";
	}
	else { &debug("User does not have sufficient permissions to generate pages: '$site->{'permissions'}->{'generate_triggers'}'", 3, $site->{'debug_level'}, $site->{'debug_log'}); }
}
##############################################################################

##############################################################################
=pod

=head2 load_required_fields

load_required_fields -

=over 4

=item DESCRIPTION

Loads in the minimum required fields to create an object of a specific object
class. 

=item INPUT

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

$required_fields - Hashref of the required fields indexed by objclass.

=item NOTES

=back

=cut
###############################################################################

sub load_required_fields {
	my $site = shift;

	my $required_fields;
	unless (-e $site->{'required_fields_path'}) {
		 &debug("Required fields path not defined: '$site->{'required_fields_path'}'", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
	require "$site->{'required_fields_path'}/require_fields.pm";
	return ($required_fields);
}
##############################################################################

##############################################################################
=pod

=head2 load_user_cache

load_user_cache -

=over 4

=item DESCRIPTION

This function loads the permissions hashref from the filesystem for the
current user and session id. 

=item INPUT

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

$perm_hashref - Hashref with all permissions up until this point loaded.

=item NOTES

The function uses a require statement to use the file. Then uses the
&get_cache function within the required file to load the hashref. The file is
a result from using &print_permissions. 

=back

=cut
###############################################################################

sub load_user_cache {
   my $site = shift;

   require "$site->{'permission_path'}/$site->{'uname'}.$site->{'sid'}.pm";
   my $perm_hashref = &get_cache;
   return $perm_hashref;
}
##############################################################################

##############################################################################
=pod

=head2 check_read_permissions

check_read_permissions -

=over 4

=item DESCRIPTION

The function concludes wether the user has permissions to view the data.

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" 		if the user is allowed to execute the result
"true_filter"	if the user is allowed to execute the result but it must be
filtered afterwards
undef		if user is not allowed to execute the result

=item NOTES

=back

=cut
###############################################################################

sub check_read_permissions {
    my ($sql_hashref,$site, $dbh) = @_;
    
	if (lc($sql_hashref->{'statement'}) ne "select") {
		&debug("check_read_permissions: must pass in select statement, not $sql_hashref->{'statement'}", 1, $site->{'debug_level'}, $site->{'debug_log'});
		return;
    }

	my ($objclass_indx,$objclass_perm,$sql_permissions);
    $objclass_indx = &get_where_clause_index("objclass", $sql_hashref, $site);
    if ($objclass_indx >= 0) {
		$objclass_perm = &get_attribute_permissions($sql_hashref->{'TABLE'}->{'WHERE'}[$objclass_indx]->{'value'}, $site);
		if ($objclass_perm =~ /x/) {
	  	  if ($sql_hashref->{'TABLE'}->{'WHERE'}[$objclass_indx]->{'value'} =~ /[!%?]/) {
				&debug("check_read_permissions: Allowing the select, but must be filtered because objclass is either %, ? or !", 3, $site->{'debug_level'}, $site->{'debug_log'});
				return "true_filter";
	    	}
	   &debug("check_read_permissions: Objclass $sql_hashref->{'TABLE'}->{'WHERE'}[$objclass_indx]->{'value'} has 'x' in permissions, excluded", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    return;
		}

		if ($objclass_perm !~ /r/) {
	  	  &debug("check_read_permissions: Do not have read permissions for objclass $sql_hashref->{'TABLE'}->{'WHERE'}[$objclass_indx]->{'value'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	  	  return;
		}
    }

    if (defined $sql_hashref->{'TABLE'}->{'FIELDS'}->{'*'}) {
	&debug("check_read_permissions: Allowing the select, but must be filtered because selecting *", 2, $site->{'debug_level'}, $site->{'debug_log'});
	return "true_filter";
    }
    else {
	$sql_permissions = &get_all_attribute_permissions($sql_hashref, $site);
	if (&check_all_attribute_permissions($sql_permissions, "r")) {
	    &debug("check_read_permissions: All good to go, don't need to filter", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    return "true";
	} 
    }

    &debug("check_read_permissions: Shouldn't ever get here, but returning undef since don't know why we're here", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return;
}
##############################################################################

##############################################################################
=pod

=head2 get_where_clause_index

get_where_clause -

=over 4

=item DESCRIPTION

Finds the index value in a sql_hashref which contains a given where
clause. 

=item INPUT

$field - This is the field we are interested in. 

$sql_hashref - Hashref defining the SQL query to execute.

$site - Site hashref with ->{'permissions'} key


=item OUTPUT

$indx - integer which reflects the location in the sql where array of
$field. 

=item NOTES

=back

=cut
###############################################################################


sub get_where_clause_index {
    my ($field,$sql_hashref,$site) = @_;

    my ($where_hashref, $indx);
    
    $indx = 0;
    foreach $where_hashref (@{$sql_hashref->{'TABLE'}->{'WHERE'}}) {
		if ($field ne "") {
	   	if ($where_hashref->{'field'} eq $field) {
				return $indx;
	    	}
		}
		$indx++;
    }

    return -1;  # Not in there
}
##############################################################################

##############################################################################
=pod

=head2 check_create_permissions

check_create_permissions -

=over 4

=item DESCRIPTION

The function concludes wether the user has permissions to insert data into
the database.

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" 		if the user is allowed to execute the result
undef		if user is NOT allowed to execute the result

=item NOTES

=back

=cut
###############################################################################
sub check_create_permissions {
	my ($sql_hashref,$site, $dbh) = @_;

	my ($sql_permissions);

	if (lc($sql_hashref->{'statement'}) ne "delete") {
		&debug("check_create_permissions: must pass in select statement, not $sql_hashref->{'statement'}", 1, $site->{'debug_level'}, $site->{'debug_log'});
		return;
    }

	 my $objclass_indx = &get_where_clause_index("objclass", $sql_hashref, $site);
	 if  ($objclass_indx < 0) {
	 	&debug("check_create_permissions: There is no objclass included", 3, $site->{'debug_level'}, $site->{'debug_log'});
	 }
	 else {
	   $sql_permissions = &get_all_attribute_permissions($sql_hashref, $site);
		if (&check_all_attribute_permissions($sql_permissions, "c")) {
		       &debug("check_create_permissions: All good to go.", 3, $site->{'debug_level'}, $site->{'debug_log'}); return "true";
				 return "true";
		}
	}
	 	&debug("check_create_permissions: Shouldn't ever get here, but returning undef since don't know why we're here", 3, $site->{'debug_level'}, $site->{'debug_log'});
	   return;
}
##############################################################################

##############################################################################
=pod

=head2 check_delete_permissions

check_delete_permissions -

=over 4

=item DESCRIPTION

The function concludes wether the user has permissions to delete data from
the database.

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" 		if the user is allowed to execute the result
undef		if user is NOT allowed to execute the result

=item NOTES

=back

=cut
###############################################################################

sub check_delete_permissions {
	my ($sql_hashref,$site,$dbh) = @_;
	my ($field);

	if (lc($sql_hashref->{'statement'}) ne "delete") {
      &debug("check_delete_permissions: must pass in delete statement, not $sql_hashref->{'statement'}", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
		return;
	}

	my $override = 1;
	my $q = &build_select($sql_hashref,$site,$dbh,$override);
	my $results = &query_getAll($q,$site,$dbh);
	my $checked_fields = 0;

	my ($permission,$obji,$objid);
	foreach $objid (keys %{$results}) {
		$permission = &get_attribute_permission($results->{$objid}->{'objclass'},$site);
		if (!$permission || $permission =~ /^[^d]+$|x/) {
      	&debug("check_delete_permissions: No permission to delete here.", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
			return;
		}
		next if ($checked_fields);
		foreach $field (keys %{$results->{$objid}}) {
			#next if (my $blah) {}  #field is objid, language, cdate, mdate
			$permission = &get_attribute_permission($field,$site);
			if (!$permission || $permission =~ /^[^d]+$|x/) {
      		&debug("check_delete_permissions: No permission to delete here.", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
				return;
			}
		}
		$checked_fields=1;
	}
	return "true";
}
##############################################################################

##############################################################################
=pod

=head2 check_modify_permissions

check_modify_permissions -

=over 4

=item DESCRIPTION

The function concludes wether the user has permissions to update data in
the database.

=item INPUT

$sql_hashref - Hashref defining the SQL query to execute. 

$site - Site hashref with ->{'permissions'} key

=item OUTPUT

"true" 		if the user is allowed to execute the result
undef		if user is NOT allowed to execute the result

=item NOTES

=back

=cut
###############################################################################

sub check_modify_permissions {
	my ($sql_hashref,$site,$dbh) = @_;
	my ($field);

	if (lc($sql_hashref->{'statement'}) ne "update") {
      &debug("check_modify_permissions: must pass in delete statement, not $sql_hashref->{'statement'}", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
		return;
	}

	my $override = 1;
	my $q = &build_select($sql_hashref,$site,$dbh,$override);
	my $results = &query_getAll($q,$site,$dbh);
	my $checked_fields = 0;

	my ($permission,$obji,$objid);
	foreach $objid (keys %{$results}) {
		$permission = &get_attribute_permission($results->{$objid}->{'objclass'},$site);
		if (!$permission || $permission =~ /^[^m]+$|x/) {
      	&debug("check_modify_permissions: No permission to delete here.", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
			return;
		}
		next if ($checked_fields);
		foreach $field (keys %{$results->{$objid}}) {
			#next if #field is objid, language, cdate, mdate
			$permission = &get_attribute_permission($field,$site);
			if (!$permission || $permission =~ /^[^m]+$|x/) {
      		&debug("check_modify_permissions: No permission to delete here.", 1, $site->{'debug_level'}, $site->{'debug_log'}); 
				return;
			}
		}
		$checked_fields=1;
	}
	return "true";
}

1;
