#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# search.pm
#
####
use strict;

##############################################################################
=pod

=head1 search.pm

search.pm - library routines for search.cgi, and all other search routines
            within the administrator.

=head1 SYNOPSIS

=head1 DESCRIPTION

This script is responsible for gathering information from the database which
matches input specified by the user.  It can perform OR searches, AND searches
and EXACT match searches

=head1 NOTES & BUGS

Does not limit the amount of rows returned by the search as of yet.
Does not do anything with alttitle.

=cut
###############################################################################
####
#
# Version 7 TG 20000929
#     Replaced build_and_or_select with build_select
# Version 6 TG 20000918
#     Got rid of warnings
# Version 5 TG 20000913
#     Changed setSearchNavigation2 to setSearchNavigation
# Version 4 TG 20000830
#     Added exact match and documented
# Version 3 TG 20000829
#     added check for search_type_inp
# Version 1 TG 20000825
#     Created
###


##############################################################################
=pod

=head2 search

search - &search($dbh, "red blue", "and", $site->{'max_search_results'}, $site, $filters, \&subroutine_name);

=over 4

=item DESCRIPTION

To search the tables for the searchwords.  If $type is and, all search words must match.  If $type
is or, any search word can match, and if $type is exact, the words must match in the order specified.

An 'or' search is always done first, querying for everything that could come back from the tables.
Then, after these are all gathered, the results are filtered.  This method is preferred because 
variant products do not have their title's stored in the things table anywhere, and are dynamically
generated.  Therefore, for an 'and' search, the results portions could be in various locations, 
making the logic more complicated. 

=item INPUT

$dbh - current active database handle

$searchwords - string of words, separated by a space

$type - type of search and, or, exact

$maxrows - the maximum number or rows to strive to return.  DOES NOT WORK AT THE MOMENT,
	   need more thought.

$site - A hashref containing information required throughout about the current
site and session

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

$build_query_routine - pointer to a subroutine which is used to create the queries for each keyword
                       This is unique on a per script basis, depending on what the search is based on.  This way
                       search can be called by any script and used by any script, just by defining this subroutine
                       first!  The subroutine MUST return results in the exact same fashion that build_site_search_queries
                       does and accept the exact same parameters in the same order as build_site_search_queries does.

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

=item NOTES

This should be tested for effeciency on LARGE sites.

AND does not work if all of the words do not appear in one field.  So, if 2 of
the words appear in title, and the other word appears in brief, this does not
work.  This is not really good

This does not limit the number of results returned, this needs to be thought
about, as the data comes from all over the place, and we are supposed to 
order by weight, which means we would have to grab all of the data, and by
that time, we are already done the hard work, so why limit it.  Otherwise, 
we need to do an order by on the select, which may not be very efficient.

=back

=cut
###############################################################################
sub search {
    my ($searchwords) = shift;
    my ($type) = shift;  
    my ($maxrows) = shift;
    my ($filter_hashref) = shift;
    my ($build_query_routine) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@rows, $i, $search_key, @words, %MasterRec);
    my (@search_queries, $q, $select_hashref, $obj, $results, $numRows, @res);
    my ($h,@variant_products, $keyword, $numExtAdded, $numRowsAdded);
    my (@encoding_match, $master_matched, $numQueries);
    my ($numSearchResults) = 0;    
    
    $results = {};

    if ($searchwords =~ / /) {
	@words = split / /, $searchwords;
    }
    else {
	$words[0] = $searchwords;
    }

    foreach $search_key (@words) {
	$numQueries = 0;
	$numQueries = &$build_query_routine($search_key,$numQueries,\@search_queries);
	&printQueries($site, @search_queries);
	$results = &do_search($dbh, $site, $results, \@search_queries, $numQueries, $search_key, $maxrows-$numSearchResults, $type, $filter_hashref);

    }
    
    if ($#words > 0) {  # more than one word supplied
	if ($type eq "and") {
	    $results->{'things'} = &filterAndResults($results->{'things'}, $site, \@words, $filter_hashref);
	}
	
	if ($type eq "exact") {
	    $results->{'things'} = &filterExactResults($results->{'things'}, $site, $searchwords, $filter_hashref);
	}
    }

    return $results;
}


##############################################################################
=pod

=head2 do_search

do_search - &do_search($dbh, $site, $results, \@search_queries, $numQueries, $search_key, $numRows, $type, $filter_hashref);

=over 4

=item DESCRIPTION

Do the actual search.  Submit the query.   If any master records matched, grab all of the
variant products for that master as well, as they inherit searchable fields from the master.
Then, search the encodings and find all variant products which match the search_key.

=item INPUT

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$results - The current set of results, up to this point. A hashref of information, initially 
           seperated by tables names, and then by objid, and then the contents of the record,
  	   of records that met the search criteria  

$search_queries - Array or hashrefs, each consisting of the elements to create a select statement

$numQueries - The number of queries to submit

$search_key - the keyword to perform the search on.  This is one keyword for and and or, and is
              the exact string for exact matches.

$maxrows - the maximum number or rows to strive to return.  DOES NOT WORK AT THE MOMENT,
	   need more thought.


$type - type of search and, or, exact

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

=item NOTES

=back

=cut
##############################################################################
sub do_search {
    my ($dbh) = shift;    
    my ($site) = shift;
    my ($results) = shift;
    my ($search_queries) = shift;
    my ($numQueries) = shift;
    my ($search_key) = shift;
    my ($maxrows) = shift;
    my ($type) = shift;
    my ($filters) = shift;


    my ($i, $select_hashref, $master_matched, $numRows, $numExtAdded, @encoding_match);
    my (%MasterRec, @variant_products, $numSearchResults);


    $numRows = 0;

    for ($i = 0; $i < $numQueries; $i++) {
	$select_hashref = {};
	$select_hashref = $$search_queries[$i];
	($numRows, $results) = &submitQuery($select_hashref, $dbh, $site, $results, $maxrows-$numRows, $type, $filters);
	($master_matched) = &saveMasterProducts($results, \%MasterRec, $site);
    }

    # If a master record that matched, we need to find all of the variant products that match
    if ($master_matched == 1) {
	$numRows = &getVariantProducts($dbh, $site, \%MasterRec, \@variant_products);
	$results = &filterResults("things", $numRows, \@variant_products, $dbh,$site,  $results, $filters);
    }
	    
    &findMatchedExtraprop($dbh, $search_key, $results->{"extraprop"}, \@encoding_match, $site);
    $numExtAdded = $#encoding_match + 1;
    $results = &filterResults("things", $numExtAdded, \@encoding_match, $dbh, $site, $results, $filters);
	    
    return $results;

}


##############################################################################
=pod

=head2 submitQuery

submitQuery - &submitQuery();

=over 4

=item DESCRIPTION

Generate the select statement and issue it.  Perform first round of filtering

=item INPUT

$select_hashref - hashref with information to build a select query:
                  $select_hashref->{'TABLE'}->{'NAME'} = name of table to query
                  $select_hashref->{'TABLE'}->{'FIELD'}->{$field} = $field
                          where $field is the name of the field to select.  One
                          for each field to select from the table
                  $select_hashref->{'TABLE'}->{'WHERE'} = Where clause array.  See
                          NOTE in db_layer.pm to see the exact syntax
                          One for each field to select from the table

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$maxNumRows - maximum number of rows to return

$type - type of search (and, or, exact)

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$numRows - Number of rows found in the query - NOTE THIS SHOULD BE NUMBER OF ROWS ADDED

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

=item NOTES

THIS DOES NOT LIMIT THE NUMBER OF RESULTS COMING BACK YET

=back

=cut
##############################################################################
sub submitQuery {
    my $select_hashref = shift;
    my $dbh = shift;
    my $site = shift;
    my $results = shift;
    my $maxNumRows = shift;
    my $type = shift;
    my $filters = shift;


    my ($q, $numRows, @res);

    $select_hashref->{'statement'} = "SELECT";
    $q = &build_select($select_hashref, $site, $dbh, 1);
    $numRows = &query($q, $dbh, \@res, $maxNumRows);
    &debug("$q\n\t$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
    if ($numRows > 0) {
	$results = &filterResults($select_hashref->{'TABLE'}->{'NAME'}, $numRows, \@res, $dbh, $site, $results, $filters);
    }
    return ($numRows, $results);
}

##############################################################################
=pod

=head2 setSearchNavigation

setSearchNavigation - &setSearchNavigation($pr, $foundcount, $maxrows, $startrow, $lastrow, $scriptinfo);

=over 4

=item DESCRIPTION

To set the Next and Back buttons on a search page

=item INPUT

$pr - location substring:
         sid=$sid&mode=$mode&skuskel=$objid&status_inp=$status_inp

$foundcount - total number of rows returned from the query

$maxrows - maximum number of rows to display on one page

$startrow - row to start displaying

$lastrow - last row to display

$scriptinfo - path the script and the scriptname: ex: /cgi-bin/om/search.cgi

=item OUTPUT

$back - the back link

$next - the next link

$firstrecord - the starting row for this page

=item NOTES

=back

=cut
##############################################################################
sub setSearchNavigation {
    my ($pr, $foundcount, $maxrows, $startrow, $lastrow, $scriptinfo) = @_;
    
    my ($back, $next, $firstrecord, $this_start, $this_page, $startPages, $endPage, $numPages, $totalPages);
    my ($numDisplayPages, $dec, $lastChange, $startPage, $i, $page, $pages);

    
    $numDisplayPages = 3;     # Total number of pages to show on each side of the pages link from the
    # current page
    $totalPages = $numDisplayPages * 2 + 3;    # include the current page, first page and last page
    $this_start = $startrow;
    $this_page = $startrow/$maxrows;
    
    if ($foundcount == 0) {
        $firstrecord = 0;
    }
    else {
        $firstrecord = $startrow+1;
    }
    
    if ($startrow > 0) {
        $startrow = $startrow - $maxrows;
        if ($startrow < 0) {
            $startrow = 0;
        }
        $back="<A HREF=\"$scriptinfo\?$pr";
        $back.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Back</FONT></A>&nbsp;&nbsp;";
    }
    
    if ($lastrow < $foundcount) {
        $startrow = $lastrow;
        $next="\n\n<A HREF=\"$scriptinfo\?$pr";
        $next.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Next</FONT></A>";
    }
    
    $numPages = $foundcount / $maxrows;
    
    ($numPages, $dec) = split /\./, $numPages;
    
    if ($dec) {
        $numPages += 1;
    }
    
    $lastChange = $numPages-$numDisplayPages;
    
    if ($numPages > 1) {
        
        if ($numPages > $totalPages) {
            if ($this_page >= $lastChange) {
                $startPage = $numPages - (($numDisplayPages * 2) + 1);
                $endPage = $numPages;
            }
            elsif ($this_page >=  $numDisplayPages) {
                $startPage = $this_page - $numDisplayPages;
                $endPage = $this_page + $numDisplayPages;
            }
            else {
                $startPage = 0;
                $endPage = $numDisplayPages * 2 + 1;
            }
            
        }
        else {
            $startPage = 0;
            $endPage = $numPages;
        }
        
        for ($i = 0; $i < $numPages; $i++) {
            
            if ((($i >= $startPage) && ($i <= $endPage)) || ($i == ($numPages-1)) || ($i == 0)) {
                $page = $i + 1;
                $startrow = $i * $maxrows;
                $pages.="\n\n<A HREF=\"$scriptinfo\?$pr";
                $pages.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">$page</FONT></A>";
                # put spaces after 1st row and before last row
            }
            if ( ($i == 0) || ($i == ($numPages - 2)) ){
                $pages .= "&nbsp&nbsp&nbsp&nbsp";
            }
            
        }
    }
    return ($back, $next, $pages, $firstrecord);
}
################################### FILTER SUBROUTINES #####################################
##############################################################################
=pod

=head2 filterResults

filterResults - &filterResults("things", $numRows, \@sql_results, $dbh, $site, $results, $filters);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the words typed into the search
keywords input by the user through a filter.  Verifies that the object matches the
objtype filter in the $filters hashref, and matches the visibility filter in the
$filters hashref.  If the product is a master product, set a flag to get the variant
products for the matched master.  If it is a variant product, set the title and the
brief.

=item INPUT

$table - Table where the object came from "things" "extraprop"

$numRows - The total number of results returned so far

$sql_results - an array of hashrefs, each being on record returned by the search query.

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

=back

=cut
##############################################################################
sub filterResults {
    my ($table) = shift;
    my ($numRows) = shift;
    my ($sql_results) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    my ($results) = shift;
    my ($filters) = shift;  # objtypes that it needs to be
    
    my ($h, $i, $add_it, $res, $master_select);
    
    
    for ($i=0; $i < $numRows; $i++) {
        $add_it = 1;  # assume we'll add it
        
        $h = $$sql_results[$i];
        
        if ($table eq "things") {
	    if (defined $filters->{'objtype'}) {
		if (( $filters->{'objtype'} !~ /$h->{'objtype'}/) || ($h->{'objtype'} eq "")) {
		    $add_it = 0;
		}
	    }
            
	    if (defined $filters->{'visibility'}) {
		if (($filters->{'visibility'} !~ /$h->{'visibility'}/) || ($h->{'visibility'} eq "")) {
		    $add_it = 0;
		}
	    }
            
            # This is a master product, so all of the variant products need to be gathered
            if ( $h->{'ref_from'} eq $h->{'objid'}) {
		&debug("Adding $h->{'objid'} to check the variants!  $h->{'addvariants'}",3, $site->{'debug_level'}, $site->{'debug_log'});
                $h->{'addvariants'} = "yes";

            }
        }
        elsif ($table eq "extraprop") {
            if ($h->{'fieldname'} eq "encoding") {
                $h->{'search_encoding'} = "yes";
            }
        }
        
        if ($add_it == 1) {
            $h->{'table'} = $table;
            $results->{$table}->{$h->{'objid'}} = $h;
            &debug("Added $h->{'objid'} from $table",3,$site->{'debug_level'}, $site->{'debug_log'});
            
        }
    }
    
    return $results;
}

##############################################################################
=pod

=head2 filterExactResults

filterExactResults - &filterExactResults($results, $site, $search_string, \%filter_hashref);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the words typed into the search
keywords input by the user through a filter.  If all of the words show up in the
at least one of the $filter_hashref->{'things'} fields, then it is kept.  
If it doesn't then it is set to undef

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$site - A hashref containing information required throughout about the current
        site and session

$word_array - Pointer to an array, each index is one of the keywords inputted by the user

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

THIS WILL NOT WORK IF A PORTION OF THE INFORMATION IS IN THE TITLE AND THE OTHER IS IN
THE KEYWORDS.  THIS IS NOT A GOOD THING.

=back

=cut
##############################################################################
sub filterAndResults {
    my $results = shift;
    my $site = shift;
    my $word_array = shift;
    my $filter_hashref = shift;

    
    my (@filters, $field, $search_word, $h, $is_a_match, $objid);
    my ($this_search, $search_str, $it_matched, @words);


    @words = @$word_array;
    @filters = split /\,/, $filter_hashref->{'things'};

    $search_str = "if (";
    foreach $search_word (@words) {
	if (length ($search_word) > 0) {
	    $search_word = lc ($search_word);
	    $search_str .= "(lc(__field__) =~ /$search_word/) && ";
	}
    }

    $search_str =~ s/&& $//g;
    $search_str .= ") { return 1; } else { return 0; };";
    &debug("Search string is $search_str", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $objid (keys %{$results}) {
	$is_a_match = 0;
	$it_matched = 0;
	foreach $field (@filters) {
	    $this_search = $search_str;
	    $this_search =~ s/__field__/\$results->{'$objid'}->{'$field'}/gms;
	    $is_a_match = eval $this_search;
	    if ($is_a_match == 1) {  # we keep it, so don't remove it!
		$it_matched = 1;
	    }
	}
	if ($it_matched == 0) {
	    &debug ("removing $results->{$objid}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    undef $results->{$objid};
	}
    }
    return $results;
}

##############################################################################
=pod

=head2 filterExactResults

filterExactResults - &filterExactResults($results, $site, $search_string, \%filter_hashref);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the exact phrase typed into the search
keywords input by the user through a filter.  If the phrase shows up in any of the 
$filter_hashref->{'things'} fields, then it is kept.  If it doesn't then it is set
to undef

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$site - A hashref containing information required throughout about the current
        site and session

$search_string - the exact search string that must match.

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

=back

=cut
##############################################################################
sub filterExactResults {
    my $results = shift;
    my $site = shift;
    my $search_string = shift;
    my $filter_hashref = shift;

    
    my (@filters, $field, $search_word, $h, $is_a_match, $objid);
    my ($this_search, $search_str, $it_matched, @words);

    @filters = split /\,/, $filter_hashref->{'things'};

    $search_str = "if (";
    $search_word = lc ($search_string);
    $search_str .= "(lc(__field__) =~ /$search_string/)";

    $search_str .= ") { return 1; } else { return 0; };";
    &debug("Search string is $search_str", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $objid (keys %{$results}) {
	$is_a_match = 0;
	$it_matched = 0;
	foreach $field (@filters) {
	    $this_search = $search_str;
	    $this_search =~ s/__field__/\$results->{'$objid'}->{'$field'}/gms;
	    $is_a_match = eval $this_search;
	    if ($is_a_match == 1) {  # we keep it, so don't remove it!
		$it_matched = 1;
	    }
	}
	if ($it_matched == 0) {
	    &debug ("removing $results->{$objid}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    undef $results->{$objid};
	}
    }
    return $results;
}



################################### VARIANT PRODUCT SUBROUTINES #####################################
##############################################################################
=pod

=head2 saveMasterProducts

saveMasterProducts - &saveMasterProducts($results, \%MasterRec, $site); 

=over 4

=item DESCRIPTION

Save a hash of objids for all master products that have matched at least one search keyword.  Used
later to get the variant products for the master and verify if they too meet the search criteria.

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$MasterRec - pointer to the current hash of master objids.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$master_matched - 0 if no master products matched, 1 if master product matched

%MasterRec - hash of objids, which have been flagged as master records in filterResults

=item NOTES

=back

=cut
##############################################################################
sub saveMasterProducts {
    my ($results) = shift;
    my ($MasterRec) = shift;
    my ($site) = shift;

    my ($master_matched, $obj, $h);

    $master_matched = 0;  # assume no masters matched.

    # Save all master records that match
    foreach $obj (keys %{$results->{"things"}}) {
        $h = $results->{"things"}->{$obj};
        if (defined $h->{'addvariants'}) {
	    &debug("Saving Master product $h->{'objid'} as it matched",3, $site->{'debug_level'}, $site->{'debug_log'});
            $$MasterRec{$h->{'objid'}} = $h;
            $master_matched = 1;
        }
    }
    return ($master_matched, %{$MasterRec});
}

##############################################################################
=pod

=head2 findMatchedExtraprop

findMatchedExtraprop - &findMatchedExtraprop($dbh, %extraprop, \@results, $keyword, $site);

=over 4

=item DESCRIPTION

Find all encodings that match the keywords, and then search for the variant products

=item INPUT

$dbh - current active database handle

$extrarprop_results - The ->{'extraprop'} portion of results.  Only the body matchings for encodings

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search

$keyword - The current keyword to find the variant products for

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$results - updated with results

=item NOTES

THIS WILL NEED TO BE EXPANDED TO INCLUDE ALTTITLE

=back

=cut
##############################################################################
sub findMatchedExtraprop {
    my $dbh = shift;
    my $keyword = shift;
    my $extraprop_results = shift;
    my $results = shift;
    my $site = shift;
    
    my ($h, $objid, $numRows, $numRowsAdded, $indx, $numExtrapropRecs);
    
    &debug("In findMatchedExtraprop with $keyword", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $indx = $numExtrapropRecs = $numRowsAdded =  0;
    foreach $objid (keys %{$extraprop_results}) {
        $h = $extraprop_results->{$objid};
        if ($h->{'fieldname'} eq "encoding") {
            ($indx, $results) = &searchVariants($h, $keyword, $dbh, $site, $indx, $results);
        }
    }
 
}

##############################################################################
=pod

=head2 searchVariants

searchVariants - &searchVariants($h, $search_word, $dbh, $site, $nextIndx, \@results);

=over 4

=item DESCRIPTION

At the time of calling this routine, we have matched a variant
encoding to a search word (both of which are passed in).
Search for all master records that have the encoding as part
of their axisname.
Foreach master record:
             Get the location in the variant axisname that the
                encoding is used.
             Get the code that is used by the encoding body to
                represent the search word that was matched.
             Find all of the variant products for the master
                Foreach variant record:
             Compare the code in the location of the vcode,
                of the variant, if it matches, add the record

=item INPUT

$h - hash of an extraprop encodings record that matched search

$search_word - word to search the encodings body for

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$nextIndx -  The next location to put the results in the results array

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search

=item OUTPUT

$nextIndx -  The next location to put the results in the results array

$results - In the format that is expected by filterResults, array of hashrefs

=item NOTES

=back

=cut
##############################################################################
sub searchVariants {
    my ($h) = shift;   
    my ($search_word) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    my ($nextIndx) = shift;
    my ($results) = shift;
    
    my ($codes, $q2, $q2num, $q3, $q3num, $h3, @masters, $master, $x, $codematch, @vcodes, $objtype, $select_hashref);
    my ($sth3, $location);
    
    &debug("In searchVariant with $search_word, $dbh!!!",3,$site->{'debug_level'}, $site->{'debug_log'});
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("axisnames", "%$h->{'objid'}%", "and");
    $select_hashref->{'statement'} = "SELECT";
    $q2 = &build_select($select_hashref, $site, $dbh, 1);
    $q2num = &query($q2, $dbh, \@masters);
    &debug("$q2\n\t$q2num affected",2,$site->{'debug_level'}, $site->{'debug_log'});
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("ref_from", "?", "and");

    $select_hashref->{'statement'} = "SELECT";
    $q3 = &build_select($select_hashref, $site, $dbh, 1);
    $sth3=$dbh->prepare( $q3 );
    
    foreach $master (@masters) {
        &debug("Servicing master record $master->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
        $location = &getVariantLocation($master, $h->{'objid'}, $site);
        $codes = &getVariantCode($h, $search_word, $site);
        &debug("Location is $location and codes are $codes for $h->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
        $sth3->execute($master->{'objid'});
	$q3num = 0;
	while ($h3 = $sth3->fetchrow_hashref) {
	    $q3num++;
	    if ($h3->{'vcode'} ne "") {  # as long as it is a variant product
		@vcodes = split /:/, $h3->{'vcode'};
		$codematch = $vcodes[$location];
		&debug("Checking variant $h3->{'objid'}, $codematch vs $codes in location $location",3,$site->{'debug_level'}, $site->{'debug_log'});
		
		if ($codes !~ /$codematch/) {
		    next;
		}
		else {
		    $h3->{'table'} = "things";   # change it to the things table
		    &debug("\tAdded variant $h3->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
		    $$results[$nextIndx++] = $h3;
		}
	    }
        }
        &debug("$q3\n\t$q3num affected",2,$site->{'debug_level'}, $site->{'debug_log'});

    }
    return ($nextIndx, $results);
}


##############################################################################
=pod

=head2 getVariantProducts

getVariantProducts - &getVariantProducts($dbh, $site, \%MasterRec, \@results);

=over 4

=item DESCRIPTION

To  add all of the variant products for master products
that have matched the search criteria in one form or another.
This is because the variant products inherit all searchable
fields from the master product

=item INPUT

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session


%MasterRec - hash of master records that have matched the
              search criteria in one form or another

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search

=item OUTPUT

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search.  Updated with the variant products

=item NOTES

=back

=cut
##############################################################################
sub getVariantProducts {
    my ($dbh) = shift;
    my ($site) = shift;
    my ($MasterRec) = shift;
    my ($results) = shift;
    
    my ($q4, $sth4, $key, $q4num, $x, $h4, $objtype, $select_hashref);
    
    my ($i);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("ref_from", "?", "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("objid", "!?", "and");

    $select_hashref->{'statement'} = "SELECT";
    $q4 = &build_select($select_hashref, $site, $dbh, 1);

    $sth4=$dbh->prepare( $q4 );
    &debug("$q4",2,$site->{'debug_level'}, $site->{'debug_log'});
    &debug("Finding all variant products for master products that matched the search",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    $i = 0;
    foreach $key (keys %{$MasterRec}) {
        $sth4->execute($key, $key);
        while ($$results[$i] = $sth4->fetchrow_hashref) {
            &debug("Variant to add:  $$results[$i]->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
            $i++;
        }
    }
    &debug("Returning $i rows",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $i;
}


##############################################################################
=pod

=head2 getVariantCode

getVariantCode - &getVariantCode($h, $searchStr, $site);

=over 4

=item DESCRIPTION

Search the encoding body field and get the variant code that
matched the search string specified.

=item INPUT

$h - hash of an extraprop encodings record that matched search

$searchStr - word to search the encodings body for

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$code - the code that matched the search string

          For example,
             If the search was 'red', and the encoding held a line with
             d:red, 'd' will be returned.
             If there is more than one location that matched, then it
             seperates the matches with :
             For example, if a search was 'x', and an encoding held lines
             c:x-large, f:xx-large, x:xxx-large, 'c:f:x' would be returned

=item NOTES

=back

=cut
##############################################################################
sub getVariantCode {
    my ($h) = shift;
    my ($searchStr) = shift;
    my ($site) = shift;
    
    my ($i, $body, $cd, $val, $row, @rows, $code);
    
    &debug("In getCode with $h, and $searchStr, fieldname $h->{'fieldname'}, $h->{'body'}",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    ($body = $h->{'body'}) =~ s/\r//g;
    @rows = split /\n/, $body;
    foreach $row (@rows) {
        ($cd, $val) = split /:/,$row;
        if ($val =~ $searchStr) {
            if ($code ne "") {
                $code .= ":$cd";
            }
            else {
                $code = $cd;
            }
        }
    }
    &debug("Returning $code",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    return $code;
}


##############################################################################
=pod

=head2 getVariantLocation

getVariantLocation - &getVariantLocation($h, $objid, $site);

=over 4

=item DESCRIPTION

Search through the master products axisnames, and determine
in what location the encoding is located.

=item INPUT

$h - hash of master product

$objid - objid of the encoding to look for in axisnames

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$location - location of the encoding in the axisnames
           For example:
              $objid = '17-AA', $h->{'axisnames'} = '2-AA:17-AA:4-AA'
              1 will be returned, as the 1st position holds 17-AA
              (0 holds 2-AA, and 2 holds 4-AA)

=item NOTES

If the same encoding is used more than once, it will not
find the second one.  It doesn't make sense to include the
encoding in more than one location though.

=back

=cut
##############################################################################
sub getVariantLocation {
    my ($h) = shift;
    my ($objid) = shift;
    my ($site) = shift;
    
    my ($i, $body, $code, $val, $row, @rows, $location, $axisnames, @objids);
    
    &debug("In getLocation with $h, and $objid, axisnames  $h->{'axisnames'}",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    $axisnames = $h->{'axisnames'};
    
    @objids = split /:/, $axisnames;
    
    for ($i = 0; $i <= $#objids; $i++) {
        if ($objids[$i] eq $objid) {
            $location = $i;
        }
        
    }
    &debug("Set $location locations in getLocation!",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $location;
}


################################ SCRIPT SPECIFIC QUERY SETUP ROUTINES #########################################
##############################################################################
=pod

=head2 build_site_search_queries

build_site_search_queries - &build_site_search_queries($keyword, $indx, \@queries);

=over 4

=item DESCRIPTION

Build the information necessary to build a select statement for search.cgi.  Each keyword must be
searched in things and extraprop.  This subroutine will be unique for every
search function which uses the libraries!

=item INPUT

$keyword - The keyword to do the search on for and and or searches, and the entire string for
           exact searches.

$indx - The next location to put the next search query

$p_queries - pointer to an array, to populate with hashrefs describing the query to issue in each index.
    
=item OUTPUT

$p_queries - populated with a hashref describing the query to issue in each index.

$indx - The next location to put the next search query

=item NOTES

=back

=cut
##############################################################################
sub build_site_search_queries {
    my ($keyword) = shift;
    my ($indx) = shift;
    my ($p_queries) = shift;

    my ($select_hashref, $h, $x, $fld);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("(objid", "!%-A)", "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("(title", "%$keyword%", "or");
    $select_hashref->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("brief", "%$keyword%", "or");
    $select_hashref->{'TABLE'}->{'WHERE'}[3] = &build_where_hash("keywords", "%$keyword%)", "or");

    $$p_queries[$indx++] = $select_hashref;
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "fieldname";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("fieldname", "alttitle", "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("body", "%$keyword%", "and");
    $$p_queries[$indx++] = $select_hashref;

    return $indx;
}


################################ DEBUG PRINTING SUBROUTINES #########################################
##############################################################################
=pod

=head2 print_results

print_results - &print_results($site, $results, $numrows, "objid,alttitle");

=over 4

=item DESCRIPTION

Output the results of the search to the debug log if the debug level is set.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$results - The current set of results, up to this point. A hashref of information, initially 
           seperated by tables names, and then by objid, and then the contents of the record,
  	   of records that met the search criteria  

$numrows - The total number of rows found in the search

$fields_to_print - A list of fieldnames which are to be printout out

=item OUTPUT

Outputs the results to the debug log

=item NOTES

=back

=cut
##############################################################################
sub printResults {
    my ($site) = shift;
    my ($results) = shift;
    my ($numrows) = shift;
    my ($fields_to_print) = shift;
    
    my ($table, $obj, $h, $fld);
    
    &debug("\nFOUND $numrows ROWS:",3,$site->{'debug_level'}, $site->{'debug_log'});
    foreach $table (keys %{$results}) {
        &debug("Table is $table",3,$site->{'debug_level'}, $site->{'debug_log'});
        foreach $obj (keys %{$results->{$table}}) {
            foreach $fld (keys %{$results->{$table}->{$obj}}) {
                if ((!$fields_to_print) || ($fields_to_print =~ $fld)){
                    &debug("Found $fld: $results->{$table}->{$obj}->{$fld}",3,$site->{'debug_level'}, $site->{'debug_log'});
                }
            }
        }
    }
}

##############################################################################
=pod

=head2 printQueries

printQueries - &printQueries($site, @search_queries);

=over 4

=item DESCRIPTION

Output the hashref information that will be used to create the select queries to
the debug log if the debug level is set.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

@search_queries - Array of hashrefs consisting of information required to build a 
        select query.

=item OUTPUT

Outputs the query configuration to the debug log

=item NOTES

=back

=cut
##############################################################################
sub printQueries {
    my ($site) = shift;
    my (@search_queries) = @_;
    my ($x, $h, $fld, $where_hash, $while_indx);
    
    for ($x=0; $x <= $#search_queries; $x++) {
        $h = $search_queries[$x];
        &debug("Table: $h->{'TABLE'}->{'NAME'}",3,$site->{'debug_level'}, $site->{'debug_log'});
        
	if (defined $h->{'TABLE'}->{'WHERE'}) {
	    $while_indx = 0;
	    while ($where_hash = $h->{'TABLE'}->{'WHERE'}[$while_indx++]) {
		&debug("WHERE: $where_hash->{'field'}, $where_hash->{'value'}, $where_hash->{'and_or'}",3,$site->{'debug_level'}, $site->{'debug_log'});
	    }
	}

        foreach $fld (keys %{$h->{'TABLE'}->{'FIELDS'}}) {
            &debug("WHERE: $fld --> $h->{'TABLE'}->{'FIELDS'}->{$fld}",3,$site->{'debug_level'}, $site->{'debug_log'});
            
        }
    }
}

##############################################################################
=pod

=head2 printSearchArray

printSearchArray - &printSearchArray($site, @search_queries);

=over 4

=item DESCRIPTION

Output the results found from the search after they have been accummulated into
an array for search.cgi

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

@array - Array of hashrefs consisting of information required to build a 
        select query.

=item OUTPUT

Outputs the information from the search query (the information that was converted
into an array datastructure)

=item NOTES

=back

=cut
##############################################################################
sub printSearchArray {
    my ($site) = shift;
    my (@array) = @_;
    my ($i, $fld, $numResults);
    
    $numResults = $#array + 1;
    &debug("Found $numResults entries!",1,$site->{'debug_level'}, $site->{'debug_log'});
    
    for ($i = 0; $i<= $#array; $i++) {
        foreach $fld (keys %{$array[$i]}) {
            if ($array[$i]->{$fld}) {
                &debug("$fld - $array[$i]->{$fld}",1,$site->{'debug_level'}, $site->{'debug_log'});
            }
        }
    }
}

1;


















