#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  validate.pm

use strict;

##############################################################################
=pod

=head1 validate.pm

validate.pm - subroutines for validating information

=head1 DESCRIPTION

This library is used for all routines related to validation of data entered
by the user.

=head1 NOTES & BUGS

eapi.pm must be required by calling script

=cut

##############################################################################

##############################################################################
=pod

=head2 validate_creditcard

validate_creditcard - &validate_creditcard($h, $site);

=over 4

=item DESCRIPTION

Takes a hashref of payment information, and validates the following:
    * Credit Card Name
    * Credit Card Number (for MasterCard and VISA)
    * Invalid Credit Card
    * Phone Number
    * Expired Credit Card
If there are any errors in the above checks, they are written to the output hashref,
and if there are missing fields that are required, a request for the user to enter
the data is also written to the output hashref.

=item INPUT

$h - Hashref of order information (payment information)

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$errors -  All errors detected during the validation, $errors->{'CC_ok'} = 0 if 
           errors were found, and is 1 if no errors were found.

=item NOTES

=back

=cut
###############################################################################
sub validate_creditcard {
    my ($h) = shift;
    my ($site) = shift;

    my($errors,@temp,$k,$v,$err,$f,@dig,$st,$d,$x,$y );
    my ($mo, $yr, $y2);

    $err="";
    $f='CCID_inp,CCEXPMO_inp,CCEXPYR_inp,CCNAME_inp,CCPHONE_inp';
    
    $errors={};                 # put new keys here...
    @temp = keys %{$h};
    while ( @temp ) {
        $k=shift @temp;
        next if ( $f !~ /$k/ );
        $errors->{$k}=$h->{$k};
        if ( $k eq "CCNAME_inp" ) {            # perform checksum if name...
            $errors->{$k} =~ tr/a-z/A-Z/;   # cannonicalize to uppercase
            $errors->{$k} =~ tr/ //d;       # remove spaces
            if (!$errors->{$k}) { 
		# "Please select a credit card name."
		$err = &validate_message($site, 38);
		$errors->{'CCNAME_err'}= &validate_message($site, 38);
	    }
	} 
	elsif ( $k eq "CCID_inp" ) {         # minimalist error checks
            $errors->{$k} =~ tr/0-9//cd;    # strip all non-numerics
            ## construct checksum for all cards
            @dig = split /\s*/, "$errors->{$k}";
            $st=0;
            $d=1;
            while (@dig) {
                $x=pop @dig;
                $y= $x*$d ;
                if ( $y > 9 ) {   # need sum of digits, first digit is 1 ...
                    $y = ( $y - 10 ) + 1;
                } # done checking for 2-digit $y
                $st+= $y;
                $d++;
                $d=1 if ( $d>2 );
            } # done running through digits
            
            ## see if plausible checksum
            $y = $st - int($st/10)*10;
            if ( $y!=0 ) { # the card number is not plausible
		# "Invalid credit card number."
                $errors->{'CCID_err'} = &validate_message($site, 39);
		
		# "There were problems with your payment" 
                $err = &validate_message($site, 40);
	    }
	    else {       # check for reasonable number of digits
                if ( length($errors->{$k})==16 ) {       ## MC,Visa,Discover
                    if ( $h->{'CCNAME_inp'} eq "MC" ) {
                        if ( substr($errors->{$k},0,1) ne "5" ) {

                            # "Invalid MasterCard Number."
			    $errors->{'CCID_err'} = &validate_message($site, 42);

			    # "There were problems with your payment" 
			    $err = &validate_message($site, 40);
                        } # else plausible...
		    } 
		    elsif ( $h->{'CCNAME_inp'} eq "VISA" ) {
                        if ( substr($errors->{$k},0,1) ne "4" ) {
                            # "Invalid VISA Number."
			    $errors->{'CCID_err'} = &validate_message($site, 43);

			    # "There were problems with your payment" 
			    $err = &validate_message($site, 40);
                        } # else plausible...
                    } # what else has 16 digits?
                    } 
		elsif ( length($errors->{$k})==15 ) {
                    # Amex, no data..
                    } 
		elsif ( length($errors->{$k})==14 ) {
                    # Visa?
                    if ( substr($errors->{$k},0,1) ne "4" ) {
			# "Invalid VISA Number."
			$errors->{'CCID_err'} = &validate_message($site, 43);

			# "There were problems with your payment" 
			$err = &validate_message($site, 40);
                    } # else plausible...
                    } 
		elsif ( length($errors->{$k})==13 ) {
                    # old cards, no data...
                    # Visa?
                    if ( substr($errors->{$k},0,1) ne "4" ) {
			# "Invalid VISA Number."
			$errors->{'CCID_err'} = &validate_message($site, 43);

			# "There were problems with your payment" 
			$err = &validate_message($site, 40);
                    } # else plausible...
		} else {                            ## no known cards
		    
                    # $errors->{'CCID_err'}="Credit card number not valid.";
		    $errors->{'CCID_err'} = &validate_message($site, 44);

		    # "There were problems with your information"
		    $err = &validate_message($site, 41);
                } # done with length permutations...
            } # done checking cc number for plausibility
            } 
	elsif ( $k eq "CCPHONE_inp" ) {
            $errors->{$k} = $h->{$k};
            $errors->{$k} =~ tr/0-9//cd;
            if (( length($errors->{$k})==10 )
                or  ( length($errors->{$k})==11 ) ) {
                # this is OK unless a 900 number...
                if ( substr($errors->{$k},0,4) =~ /900/ ) {
                    # this should NOT be a 900 number
		    # "Customer Service 900 numbers are still very rare"
                    $errors->{'CCPHONE_err'} = &validate_message($site, 45);

		    # "There were problems with your information"
		    $err = &validate_message($site, 41);
                } # end of ruling out 900 numbers
	    }
	    else {
		# "This information is required as a security precaution"
                $errors->{'CCPHONE_err'} = &validate_message($site, 46);

		# "There were problems with your information"
		$err = &validate_message($site, 41);
            } # done examining the phone number...
	} 
	elsif ( $k eq "CCEXPMO_inp" ) {
            if (( $h->{$k} > 12 ) or ( $h->{$k} < 1 )) {

		# "Please select month the credit card expires.";
		$errors->{'CCEXPMO_err'} = &validate_message($site, 47);
	
		# "There were problems with your information"
		$err = &validate_message($site, 41);
            }
	} 
	elsif ( $k eq "CCEXPYR_inp" ) {
            if ( (( $h->{$k} < 97 ) and ( $h->{$k} > 06 ))
		 or ( length($h->{$k}) < 2 ) ) {
		# "Please select the year the credit card expires.";
                $errors->{'CCEXPYR_err'} = &validate_message($site, 48);

		# "There were problems with your information"
		$err = &validate_message($site, 41);
            }
        } # no "else", irrelevant field...
    } # done validating the payment fields
    
    ## check month and year OK
    #
    ($mo,$yr) = ( localtime(time) )[4..5];
    $yr+=1900;
    $mo++;
    if ( $h->{'CCEXPYR_inp'} > 95 ) {
        $y2 = 1900 + $h->{'CCEXPYR_inp'};
        } 
    elsif ( length($h->{'CCEXPYR_inp'}) > 0 ) {
        $y2 = 2000 + $h->{'CCEXPYR_inp'};
        } 
    else {
        $y2=0;
    } # done with variations on date...
    if ( $yr == $y2 ) {
        if ( $mo > $h->{'CCEXPMO_inp'} ) {
	    # "There were problems with your information"
	    $err = &validate_message($site, 41);

            # "This credit card has expired.";
	    $errors->{'CCEXPYR_err'} = &validate_message($site, 49);
        } # else no error
    } elsif ($yr > $y2 ) {
            # "There were problems with your information"
            $err = &validate_message($site, 41);

            # "This credit card has expired.";
            $errors->{'CCEXPYR_err'} = &validate_message($site, 49);
    } # done checking for month, year in the past
    
    ## error state and messages
    #
    if ( length($err)>2 ) {
        $errors->{'CC_err'} = $err;
        $errors->{'CC_ok'} = 0;
        } 
    else {
        $errors->{'CC_ok'} = 1;
    } # done with the error state and messages
    
    return $errors;
} #/validate_payment


##############################################################################
=pod

=head2 validate_coupon($form, $site);

&validate_coupon - validate_coupon($co_id, $site);

=over 4

=item DESCRIPTION

this fuction test the validation of a coupon
it takes the number of the coupon, then search the table extraprop 
to see if it's valid or not, 
if it's valid, return a hash refernce contains coupon information
if not, return a hash refernce contains error message

=item INPUT

$co_id - the coupon id

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub validate_coupon {

    my $co_id = shift;
    my $site = shift;
    my $dbh = $site->{'dbh'};
    my $co_info = ();

    # use like in where clause is to make difference between
    # the coupon object and coupon payment
    # the fieldname of coupon payment has "CO|id|amount" format
    # the fieldname of coupon object has 
    # "CO|id|amount|exmonth|exyear|stmonth|styear" format
    my $q = "SELECT body FROM extraprop ";
       $q .= "WHERE fieldname LIKE 'CO|id|name|amount|exmonth|exyear|%' ";
       $q .= "AND body LIKE 'CO|$co_id|%' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;

    my $body = $sth->fetchrow;

    # if no row is found, error message
    if (!$body) {
        # Error - The coupon number cannot be found, please enter again.
        $co_info->{'CO_err'} = &validate_message($site, 80);
    } else {
        my @info = split /\|/, $body;

        # now theck if coupon in valid time period
        my $expire_date = &date_compare($info[3], $info[4]);
        my $start_date = &date_compare($info[5], $info[6]);

        # if now date earlier than start date
        # or later than expire date, give error message, 
        # else return coupon infomation
        if ( $start_date eq "earlier" || $expire_date eq "later" ) {
            # Error - The coupon is not in valid period.
            $co_info->{'CO_err'} = &validate_message($site, 81);
        } else {
            $co_info->{'name'} = "coupon";
            $co_info->{'number'} = $info[1];
            $co_info->{'amount'} = $info[2];
            $co_info->{'expire'} = $info[3].'/'.$info[4];
            $co_info->{'start'} = $info[5].'/'.$info[6];
        } # end of if else

   } # end of if else

   return $co_info;

  #############################################################
  # this sub function compare now date and input date         #
  # if now date is earlier than input date, return "earlier", #
  # if now date is same as input date, return "same",         #
  # if now date is later than input date, return "later".     #
  #                                                           #
  # input: month and year                                     #
  # output: string "eralier" or "same" or "later"             #
  #############################################################
  sub date_compare {

    my $month = shift;
    my $year = shift;

    # build now date
    my ($now_mo,$now_yr) = ( localtime(time) )[4..5];
    $now_yr += 1900;
    $now_mo;

    # build year
    # if there are 2 digits for year, make it to 4 digits
    if ( length( $year ) <= 2 ) {
        if ( $year > 95 ) {
           $year = 1900 + $year;
        } elsif ( length( $year ) > 0 ) {
           $year = 2000 + $year;
        } else {
           $year = 0;
        }
    } # done with build up year...

    if ( $now_yr == $year ) {
       if ( $now_mo > $month ) {
           return "later";
       } elsif ( $now_mo = $month ) {
           return "same";
       } else {
           return "earlier";
       }
    } elsif ( $now_yr > $year ) {
        return "later";
    } else {
        return "earlier";
    }
  } # end of sub if_later_than_now

} # end of sub validate_coupon

##############################################################################
=pod

=head2 validatefield

validatefield - &validatefield("title", "Hi", $objdef_hashref, \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Take a fieldname, it's value, any additional input that is required, and validate
the data.  If there is an error, set the error in the $FORM location (if desired), 
and return the error message to the calling routine

=item INPUT

$field - Name of the field to validate

$value - Value of the field to validate

$h - Hash of input data that may be required through validation

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - An error message if the field was evaluated to be invalid, or "" if validation
         passed.

=item NOTES

It may be nice to get rid of $h or $FORM.

=back

=cut
###############################################################################
sub validatefield {
    my ($field) = shift;
    my ($value) = shift;
    my ($h) = shift;
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my($error,$testmode,$q,$sth, @selnames, $select_hashref, $cont, $i, $tmp );
    my (@rows, $row, $code, @text, @newv, $x, $num, $dec, $numlen, $declen); 
    my (@sql_results, $numRows, $numRes, $where_indx, $field_length, $variant_errors);
    my (@variants, $variant, @perms, $perm);

    $error="";

    if ($field eq "obj2") {
        # only validate if one was entered in.  Ignore blank entries for this
        if ($value) {
            $error=ValidateOBJID($value, $site, $dbh);
            if ($error eq "INVALID_OBJ") {
		# "Invalid object id!"
		$error = &validate_message($site, 3);
            }
            if ($error eq "NO_OBJ") {
                # "The object entered could not be found!";
		$error = &validate_message($site, 2);
            }
        }
    }
    # V 0.7.29 - start add
    elsif ($field eq "celldepth") {
        if ($value =~ /(\D+)/) {
	    #"Cell depth must be a positive number."
            $error.=&validate_message($site, 4);
        }
	if ($value > 2147483647) {
            # check for overflow
	    # "Error - Value is too large";
	    $error = &validate_message($site, 26);
	}

	if ($value < -2147483648) {  # check for overflow
	    #  "Error - Value is too small"
	    $error = &script_message($site, 27);
	}
    }
    
    elsif ( $field eq "alttitle" ) {
        # should be longer than 4 characters and less than 40
        #    or zero characters
        #
        &debug("Alttitle is $value", 3, $site->{'debug_level'}, $site->{'debug_log'});
        if ( (( length($value)>3 ) and (length($value)<41))
            or ( length($value)==0 )) {
            # get a warm fuzzy feeling, it is OK
        }
        else {
	    # Error - Alternate title should be 0 or 4 to 40 chars
            $error = &validate_message($site, 1);
        }
        
        
        # done with ALTTITLE
    }
    elsif ( $field eq "sku_series" ) {
        
        if ($value eq '') {
            # "Error - Must specify a brand"
	    $error = &validate_message($site, 5);
        }
        elsif (($$FORM{'sku_series_pre'} ne '') &&
            ($$FORM{'sku_series_inp'} ne $$FORM{'sku_series_pre'})) {
            $$FORM{'sku_series_inp'} = $$FORM{'sku_series_pre'};
            # Error - You cannot change the brand
	    $error = &validate_message($site, 11);
        }
    }
    
    elsif ( $field eq "sku_skel" ) {
        $value =~ s/(\s)+//g;
        if ($value =~ /[^a-zA-Z]+/) {
            # Error - Must be $site->{'site_sku_prelen'} letters
	    $error = &validate_message($site, 7) . " $site->{'site_sku_prelen'} " . &validate_message($site, 8);
        }
        elsif (length($value) != $site->{'site_sku_prelen'}) {
            # Error - Must be $site->{'site_sku_prelen'} letters
	    $error = &validate_message($site, 7) . " $site->{'site_sku_prelen'} " . &validate_message($site, 8);
        }
        # see if this is the first time to create a sku_skel, and validate it
        elsif ($h->{'sku_skel_pre'} eq "") {
            # this needs to be a unique letter code if the
            # sku is a brand or series header
            #
            
            # this is a brand or series header
	    $select_hashref = {};
	    $select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "fieldname";
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
	    $where_indx = 0;
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "sku_skel", "and");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("body", "$value%", "and");

	    $select_hashref->{'statement'}="SELECT";
	    $q = &build_sql($select_hashref, $site, $dbh);
	    $numRows = &query_getAll($q, $site, $dbh, \@sql_results);
	    &debug("$q\n\t-->$numRows affected",2, $site->{'debug_log'}, $site->{'debug_level'});
	    if ($numRows > 0) {
                # Error - $site->{'site_sku_prelen'} letter code not unique
		$error = &validate_message($site, 9) . "$site->{'site_sku_prelen'} " . &validate_message($site, 10);
                $$FORM{'sku_skel_inp'} = '';
            }
            
        }
        else {
            if ($h->{'sku_skel_inp'} ne $h->{'sku_skel_pre'} ) {
                # Error - You cannot change your SKU Code
		$error = &validate_message($site, 11);
            }
            
        }
        
        if ($error) {
            $h->{'sku_skel_inp'} = $h->{'sku_skel_pre'};
        }
    }
    elsif ($field eq "objtype") {
#	if (defined $$FORM{'new_object'}) {
	if ((defined $$FORM{'new_object'}) && ($$FORM{'new_object'} eq "yes")) {
	    # If it is a warehouse, make sure there is only one defined
	    if ($value eq $site->{'OBJTYPES'}->{'warehouse'}) {
		# this is a brand or series header
		$select_hashref = {};
		$select_hashref->{'TABLE'}->{'NAME'} = "things";
		$select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
		$where_indx = 0;
		$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objtype", $value, "and");
		$select_hashref->{'statement'}="SELECT";
		$q = &build_sql($select_hashref, $site, $dbh);
		$numRows = &query_getAll($q, $site, $dbh, \@sql_results);
		if ($numRows > 0) {
		    $$FORM{'page_error'} = &validate_message($site, 68 );
		    return $$FORM{'page_error'};
		}
	    }
	}
    }

    elsif ( $field eq "sel_name" ) {
        
        if (length($value) == 0) {
            # Error - You must specify a Select File Name
	    $error = &validate_message($site, 12);
        }
        else {
	    next if ($$FORM{'sel_name_inp'} eq $$FORM{'sel_name_pre'});   # hasn't changed, so let them modify the contents
                                                                      # of an interal record anyways.

            $value = string_todbi($value);
	    if (!defined ($$FORM{'override_selname_check'})) {
		if (($$FORM{'sel_name_inp'} eq "sel_behavior") ||
		    ($$FORM{'sel_name_inp'} eq "sel_status") ||
		    ($$FORM{'sel_name_inp'} eq "sel_ccname") ||
		    ($$FORM{'sel_name_inp'} eq "sel_ccexpyr") ||
		    ($$FORM{'sel_name_inp'} eq "sel_ccexpmo") ||
		    ($$FORM{'sel_name_inp'} eq "sel_cellstyle") ||
		    ($$FORM{'sel_name_inp'} eq "sel_pagestyle") ||
		    ($$FORM{'sel_name_inp'} eq "sel_state") ||
		    ($$FORM{'sel_name_inp'} eq "sel_visibility") ||
		    ($$FORM{'sel_name_inp'} eq "sel_objtype") ||
		    ($$FORM{'sel_name_inp'} eq "sel_orderstatus") ||
		    ($$FORM{'sel_name_inp'} eq "sel_warehouse")) {
		    # Error - This is an internal record required by AllCommerce! You cannot use the name '$FORM{'sel_name_inp'}'
		    $error = &validate_message($site, 13) . "'$$FORM{'sel_name_inp'}'";
		    
		}
	    }
            elsif ($$FORM{'sel_name_inp'} ne $$FORM{'sel_name_pre'}) {
                
                # see if this already exists!
		$select_hashref = {};
		$select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
		$select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";

		$where_indx = 0;
		$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "sel_name", "and");
		$select_hashref->{'statement'} = "SELECT";
		$q = &build_sql($select_hashref, $site, $dbh);
                $numRows = &query_getAll($q, $site, $dbh, \@selnames);
                if ($numRows > 0) {
                    $cont = 1;
                    $i = 0;
                    while (($cont) && ($i < $numRows)) {
                        if ($value eq $selnames[$i++]->{'body'}) {
                            $tmp = $value;
                            $tmp =~ s/\\//g;
			    # "Error - The name is not unique ($tmp)
			    $error = &validate_message($site, 72) . " ($tmp)";
                            $cont = 0;
                        }
                    }
                }
                if (($value !~ /^sel\\_/) && ($value !~ /^sel_/)) {
                    # Error - The name must start with sel_ 
		    $error = &validate_message($site, 14);
                }
                else {
                    if ($value =~ /[A-Z]/) {
                        # Error - The name must have all lowercase characters
			$error = &validate_message($site, 15);
                    }
                }
            }
        }
        
        if ($error) {
            $$FORM{'sel_name_inp'} = $$FORM{'sel_name_pre'};
        }
    }

    elsif ( $field eq "gui_name") {
        if (length($value) == 0) {
            # Error - You must specify a Select File Name
	    $error = &validate_message($site, 12);
        }
        else {
            $value = string_todbi($value);
            if ($$FORM{'gui_name_inp'} ne $$FORM{'gui_name_pre'}) {
                # see if this already exists!
		$select_hashref = {};
		$select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
		$select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
		
		$where_indx = 0;
		$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "gui_name", "and");
		$select_hashref->{'statement'} = "SELECT";
		$q = &build_sql($select_hashref, $site, $dbh);
                $numRows = &query_getAll($q, $site, $dbh, \@selnames);
                if ($numRows > 0) {
                    $cont = 1;
                    $i = 0;
                    while (($cont) && ($i < $numRows)) {
                        if ($value eq $selnames[$i++]->{'body'}) {
                            $tmp = $value;
                            $tmp =~ s/\\//g;
			    # "Error - The name is not unique ($tmp)
			    $error = &validate_message($site, 72) . " ($tmp)";
                            $error=
                            $cont = 0;
                        }
                    }
                }
                if ($value !~ /_gui$/) {
                    # Error - The name must end in _gui
		    $error = &validate_message($site, 6);
                }
                else {
                    if ($value =~ /[A-Z]/) {
                        # Error - The name must have all lowercase characters
			$error = &validate_message($site, 15);
                    }
		    
		}
	    }
	}
    }
    
    elsif ( $field eq "encoding" ) {
        
        $value = string_todbi($value);
        &debug("In encoding validation with $value, type $$FORM{'objtype_inp'}!", 3, $site->{'debug_level'}, $site->{'debug_log'});
       
        # Encoding checks
        if ($$FORM{'objtype_inp'} eq $site->{'OBJTYPES'}->{'encoding'}) {
            @rows = split /\n/, $value;
            foreach $row (@rows) {
                ($code, @text) = split /\|/, $row;
                if ($#text < 0) {
                    # "Error: Each line must have the syntax code|text"
		    $error = &validate_message($site, 17);
                }
                elsif (!($text[0] =~ /\w/)) {
                    # "Error: Each line must have the syntax code:text"
                    $error = &validate_message($site, 16);
                }
            }
        }

        # Variant checks
        elsif ($$FORM{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'variant'}) {
	    &debug("In variant checks", 3, $site->{'debug_level'}, $site->{'debug_log'});
            @rows = split /\n/, $value;
            
            foreach $row (@rows) {
                ($code, @text) = split /:/, $row;
                if ($#text < 0) {
                    # "Error: Each line must have the syntax code:text"
                    $error = &validate_message($site, 16);
                }
                
                elsif ($text[0] !~ /\w/) {
                    # "Error: Each line must have the syntax code:text"
                    $error = &validate_message($site, 16);
                }
            }
        }
        
        
        
    }
    elsif ( $field eq "axisnames" ) {
        
        @newv = split /:/, $value;
        if ($#newv > 3) {
	    # "Error - Maximum of 4 skus allowed"
	    $error = &validate_message($site, 18);
            $$FORM{'axisnames_inp'} = $$FORM{'axisnames_pre'};
        }
        
        foreach $x (@newv) {
            $error = &ValidateOBJID($x, $site, $dbh, 4) if (!$error);
            if ($error eq "INVALID_OBJ") {
		# Invalid object id ($x)!
		$error = &validate_message($site, 3) . " ($x)";
                $$FORM{'axisnames_inp'} = $$FORM{'axisnames_pre'};
            }
            if ($error eq "NO_OBJ") {
                # The object entered could not be found ($x)!
		$error = &validate_message($site, 2) . " ($x)";
                $$FORM{'axisnames_inp'} = $$FORM{'axisnames_pre'};
            }

            if ($error eq "NOT_VARIANT_SET") {
                # "The object entered is not a variant set ($x)!"
		$error = &validate_message($site, 37) . " ($x)";
                $$FORM{'axisnames_inp'} = $$FORM{'axisnames_pre'};
            }
        }
        
        if (($$FORM{'axisnames_pre'} ne '') &&
            ($$FORM{'axisnames_pre'} ne $$FORM{'axisnames_inp'})) {
            # "Error - You cannot modify the axisnames"
	    $error = &validate_message($site, 20);
            $$FORM{'axisnames_inp'} = $$FORM{'axisnames_pre'};
        } 
    }
    elsif ( $field eq "body" ) {
	# no restrictions?
    } 
    elsif ( $field eq "brand" ) {
	# should be a valid objid
	if ( $error=validatebrand($value, $site, $dbh) ) {
	    # "Error - This does not appear to be a good brand."
	    $error = &validate_message($site, 21);
	}
	
    } 
    elsif ( $field eq "brief" || $field eq "descript") {
	$value =~ s/\s*$//g;
	# should be longer than 5 characters and less than 1000
	if ( length($value) > 1000 ) {
	    # "Error - Brief should be less than 1000 characters"
	    $error= &validate_message($site, 22);
	} 
	if ( length($value) < 5 ) {
	    # "Error - Brief should be between 5 and 1000 characters"
	    $error= &validate_message($site, 23);
	}
    }
    elsif ( $field eq "manweight" ) {
	# should be a number 01-100, or blank
	if ($value =~ /(\D)+/) {
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	if ($value<0) {
	    # "Error - Must be a positive number"
	    $error = &validate_message($site, 25);
	} 
	elsif ($value > 100) {
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	# done with Manual Link Weighting
    }
    # V 0.7.29 - Added
    # V 0.7.30 - Added weight_man
    
    elsif (( $field eq "linkwt_man" )  || ($field eq "weight_man")) {
	# should be a number 01-100, or blank
	if ($value =~ /(\D)+/) {
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	if ($value<0) {
	    # "Error - Must be a positive number"
	    $error = &validate_message($site, 25);
	} 
	elsif (($value>100) || ($value eq '0')){
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	# done with Manual Link Weighting
    }
    
    
    elsif ( $field eq "warehouse" ) {
	if (length ($value) == 0) {
	    # "Error - You must specify a warehouse"
	    $error = &validate_message($site, 32);
	}
    }
    
    elsif ($field eq "weight") {
	if (length($value) == 0) {
	    # "Error - You must enter a weight"
	    $error = &validate_message($site, 31);
	}
	else {
	    $error = &validate_numeric($value, $site);
	}
    }
    
    elsif ( $field eq "adjust" ) {
	
	if (length($value) > 0) {
	    $error = &validate_numeric($value, $site, 1);

	}
	
    }
    elsif ( $field eq "unitct" ) {
	$error = &validate_numeric($value, $site);
    }

    elsif ($field eq "unitpr") {
	if (length($value) > 0) {
	    $error = &validate_floating_point($value, $site, 99999.99, 0);
	}
    }
    
    elsif ( $field eq "linkwt_use" ) {
	# should be a number 01-100, or blank
	if ($value =~ /(\D)+/) {
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	if ($value<0) {
	    # "Error - Must be a positive number"
	    $error = &validate_message($site, 25);
	} 
	elsif ($value>100) {
	    # "Error - Weight should be blank or from 1 through 100";
	    $error = &validate_message($site, 24);
	}
	# done with Manual Link Weighting
    } elsif ( $field eq "repl_by" ) {
	if (length($value)==8) {
	    $error=validatesku($value, $site, $dbh);
	}
    }  
    elsif ( $field eq "see_sku" ) {
	if (length($value)==$site->{'site_sku_totlen'}) {
	    $error=validatesku($value, $site, $dbh);
	} 
	else {
	    #"Error - Sku must be of length $site->{'site_sku_totlen'}"
	    $error = &validate_message($site, 35) . " $site->{'site_sku_totlen'}";
	}
	# done with SEE_SKU
    } 
    elsif ( $field eq "use_sku" ) {
	if (length($value)==$site->{'site_sku_totlen'}) {
	    $error=validatesku($value, $site, $dbh);
	} 
	else {
	    #"Error - Sku must be of length $site->{'site_sku_totlen'}"
	    $error = &validate_message($site, 35) . " $site->{'site_sku_totlen'}";
	}
    } 
    elsif ( $field eq "obj2" ) {
	$error=validateobjid($value);
    } 
    elsif ( $field eq "title" ) {
	# should be longer than 3 characters and less than 80
	if (( length($value)<3 ) or (length($value)>80)) {
	    $value=length($value);
	    # "Error - Should be between 3 and 80 characters"
	    $error = &validate_message($site, 36);
	}
        
    } 
    elsif ($field eq "uname") {
	# should be longer than 3 characters and less than 80
	if (( length($value)<3 ) or (length($value)>80)) {
	    $value=length($value);
	    # "Error - Should be between 3 and 80 characters"
	    $error = &validate_message($site, 36);
	}
    } 


    elsif ($field eq "passwd") {
	# should be longer than 3 characters and less than 80
	if (( length($value)<3 ) or (length($value)>80)) {
	    $value=length($value);
	    # "Error - Should be between 3 and 80 characters"
	    $error = &validate_message($site, 36);
	}
    } 

    elsif ($field eq "variant_set") {
	$field_length = length($value); 
	if ($field_length == 0) {
	    # "Error - You must specify at least one variant
	    $error = &validate_message($site, 69);
	}
	@variants = split /\:/, $value;
	foreach $variant (@variants) {
            $error = &ValidateOBJID($variant, $site, $dbh, 2) if (!$error);
	    next if (!$error);
            if ($error eq "INVALID_OBJ") {
		# Invalid object id ($variant)!
		$error = &validate_message($site, 3) . " ($variant)";
            }
            if ($error eq "NO_OBJ") {
                # The object entered could not be found ($variant)!
		$error = &validate_message($site, 2) . " ($variant)";
            }
            if ($error eq "NOT_VARIANT") {
                # "The object entered is not a variant ($variant)!"
		$error = &validate_message($site, 19) . " ($variant)";
            }
	}
    }

    elsif ($field eq "uperms") {
	@perms = split /\:/, $value;
	foreach $perm (@perms) {
            $error = &ValidateOBJID($perm, $site, $dbh, 3) if (!$error);
	    next if (!$error);
            if ($error eq "INVALID_OBJ") {
		# Invalid object id ($uperm)
		$error = &validate_message($site, 3) . " ($perm)";
            }
            if ($error eq "NO_OBJ") {
                # The object entered could not be found ($perm)!
		$error = &validate_message($site, 2) . " ($perm)";
            }
            if ($error eq "NOT_UPERM") {
                # "The object entered is not a permission set ($perm)!"
		$error = &validate_message($site, 73) . " ($perm)";
            }
	}
    }

    elsif ($field eq "ship_weight") {
	if (length($value) > 0) {
	    $error = &validate_floating_point($value, $site, "", "");
	}
	else {
	    # Only show error if not a master product
	    if (!defined $h->{'has_axisnames'}) {
		# You must enter a numeric value
		$error = &validate_message($site, 74);
		return $error;
	    }
	}
    }

    if ($error) {
	&debug("ERROR: $error!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return $error;
    
} #/validatefield

################ INDIVIDUAL FIELD VALIDATEION ROUTINES  ######################
##############################################################################
=pod

=head2 validate_numeric

validate_numeric - &validate_numeric($value, $site);

=over 4

=item DESCRIPTION

Takes a value and make sure that it is in numeric format.  It must
have only numbers in it.

If there is an error, return the error message.

=item INPUT

$value - The value to verify floating point syntax

$site - A hashref containing information required throughout about the current
        site and session

$allow_neg - OPTIONAL, if set then allow negative numbers, otherwise not.

=item OUTPUT

$error - Error string if an error was detected, or "" if validation passed

=item NOTES

=back

=cut
###############################################################################
sub validate_numeric {
    my ($value) = shift;
    my ($site) = shift;
    my ($allow_neg) = shift;

    my ($error, $tmp_val, $num_neg);

    if (length($value) <= 0) {
	return;
    }

    if ($allow_neg) {
	$tmp_val = $value;
	$num_neg = ($tmp_val =~ s/-//gms);
	
	&debug("num_neg is $num_neg", 3, $site->{'debug_level'}, $site->{'debug_log'});

	# - in wrong spot.
	if ($num_neg > 1) {
	    # You must enter a numeric value
	    $error = &validate_message($site, 74);
	    return $error;
	}

	# - in wrong spot.
	if (($num_neg == 1) && ($value !~ /^-/)) {
	    # You must enter a numeric value
	    $error = &validate_message($site, 74);
	    return $error;
	}
    }
    else {
	$tmp_val = $value;
    }

    
    if ($tmp_val =~ /(\D)+/) {
	if (!$allow_neg) {
	    # "Error - You must enter a positive numeric value"
	    $error = &validate_message($site, 29);
	}
	else {
	    # You must enter a numeric value
	    $error = &validate_message($site, 74);
	}
    }


    # check for overflow
    elsif ($value > 2147483647) {
	# "Error - Value is too large";
	$error = &validate_message($site, 26);
    }
    
    elsif ($value < -2147483648) {  # check for overflow
	#  "Error - Value is too small"
	$error = &validate_message($site, 27);
    }

    return $error;
}

##############################################################################
=pod

=head2 validate_floating_point

    validate_floating_point - &validate_floating_point($value, $site);

=over 4

=item DESCRIPTION

Takes a value and make sure that it is in floating point format.  It must
have only numbers, + or - and the '.'.  It must have them in appropriate
positions.

If there is an error, return the error message.

=item INPUT

$value - The value to verify floating point syntax

$site - A hashref containing information required throughout about the current
        site and session

$max_number - specify the maximum number that this can be and we will 
              check that it is not larger than it.  If "", don't check

$min_number - specify the minimum number that can be and we will check that
              it is not smaller than it.  If "", don't check

$allow_neg - OPTIONAL, if set then allow negative numbers, otherwise not.

=item OUTPUT

$error - Error string if an error was detected, or "" if validation passed

=item NOTES

=back

=cut
###############################################################################
sub validate_floating_point {
    my ($value) = shift;
    my ($site) = shift;
    my ($max_number) = shift;
    my ($min_number) = shift;
    my ($allow_neg) = shift;

    my ($error, $num, $dec, $tmp_val, $numlen, $declen, $num_neg);
    my ($num_dec, $numeric_msg_number);

    # If there is nothing to check then return
    if (length($value) == 0) {
	return;
    }

    if (!defined $allow_neg) {
	$numeric_msg_number = 77;
    }
    else {
	$numeric_msg_number = 28;
    }

    ($num, $dec) = split /\./, $value;
    $numlen = length($num);
    $declen = length($dec);

    # Make sure no - if not allowed.
    if ((!defined $allow_neg) && ($value =~ /-/)) {   
	# Error - You must enter a positive numeric value
	$error = &validate_message($site, 77);
	return $error;
    }

    # Check negative information
    if ((defined $allow_neg) && ($value =~ /-/)) {
	$tmp_val = $value;
	$num_neg = ($tmp_val =~ s/-//gms);    
	&debug("num_neg is $num_neg", 3, $site->{'debug_level'}, $site->{'debug_log'});

	# - in wrong spot.
	if ($num_neg > 1) {
	    if (($max_number ne "") && ($min_number ne "")) {
		# "Error - You must specify a number between $min_number and $max_number
		$error = &validate_message($site, 75);
		$error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	    }
	    else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	    }
	    return $error;
	}

	# - in wrong spot.
	if (($num_neg == 1) && ($value !~ /^-/)) {
	    if (($max_number ne "") && ($min_number ne "")) {
		# "Error - You must specify a number between $min_number and $max_number
		$error = &validate_message($site, 75);
		$error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	    }
	    else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	    }
	    return $error;
	}
    }
    else {
	$tmp_val = $value;
    }

    # Check decimal point stuff
    if ($tmp_val =~ /\./) {
	$num_dec = ($tmp_val =~ s/\.//gms);
	# - in wrong spot.
	if ($num_dec > 1) {
	    if (($max_number ne "") && ($min_number ne "")) {
		# "Error - You must specify a number between $min_number and $max_number
		$error = &validate_message($site, 75);
		$error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	    }
	    else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	    }
	    return $error;
	}
    }

    if ($declen > 2) {
	# "Error -  You can only have 2 numbers after the decimal point"
	$error = &validate_message($site, 34);
    }

    elsif ($tmp_val =~ /(\D)+/) {
	    if (($max_number ne "") && ($min_number ne "")) {
		# "Error - You must specify a number between $min_number and $max_number
		$error = &validate_message($site, 75);
		$error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	    }
	    else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	    }
    }

    elsif (($max_number ne "") && ($value > $max_number)) {
	if (($max_number ne "") && ($min_number ne "")) {
	    # "Error - You must specify a number between $min_number and $max_number
	    $error = &validate_message($site, 75);
	    $error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	}
	else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	}
    }

    elsif (($min_number ne "") && ($value < $min_number)) {
	if (($max_number ne "") && ($min_number ne "")) {
	    # "Error - You must specify a number between $min_number and $max_number
	    $error = &validate_message($site, 75);
	    $error .= "$min_number" . &validate_message($site, 76) . "$max_number";
	}
	else {
		# Error - You must enter a numeric value
		$error = &validate_message($site, $numeric_msg_number);
	}
    }

    return ($error);

}

##############################################################################
=pod

=head2 validatesku

validatesku - &validatesku($value, $site, $dbh);

=over 4

=item DESCRIPTION

Takes a sku, and validates the following:
    * It is not blank
    * It is 8 characters long
    * It is in the right format (3 letters, site code 'm', then 4 numbers)
    * That a brand already exists for the sku to be generated from.
If there is an error, return the error message.

=item INPUT

$value - Value of the sku

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error string if an error was detected, or "" if validation passed

=item NOTES

This will phase out as sku's do.

=back

=cut
###############################################################################
sub validatesku {
    my ($value) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my($sth,$q,$error,$num, $select_hashref, @sql_results );
    my ($where_indx);

    $error="";
    
    # this needs to be an 8 digit alphanumeric
    # with the 4th digit being the store code
    
    $value =~ tr/A-Za-z0-9//cd;   # strip everything but alphanumerics
    $value =~ tr/A-Z/a-z/ ;       # lowercase it
    if ( length($value)==0 ) {
	# "Error - SKU is blank";
        $error = &validate_message($site, 50);
        }
    elsif ( length($value)!=8 ) {
        # "Error - SKUs are 8 characters long!";
	$error = &validate_message($site, 51);
        }
    elsif ( $value !~ /[a-z]{3}($site->{'sitecode'})[0-9]{4}/ ) {
        # "Error - SKUs are 3 characters, a store code, then four digits" ;
	$error=&validate_message($site, 52);
        } 
    else {
        # check to see if it exists...
	$select_hashref = {};
	$select_hashref->{'TABLE'}->{'NAME'} = "things";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'sku'} = "sku";
	$where_indx = 0;
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("sku", $value, "and");

	$select_hashref->{'statement'} = "SELECT";
	$q = &build_sql($select_hashref,$site, $dbh);
	$num = &query_getAll($q, $site, $dbh, \@sql_results);
	&debug("$q\n\t-->$num affected",2, $site->{'debug_log'}, $site->{'debug_level'});

        if ( $num == 0 ) {
            # "Error - nothing found"
	    $error = &validate_message($site, 53);
            } 
	elsif ( $num > 1 ) {
            # "Error - your SKU should have one record, found $num records!"
	    $error = &validate_msg($site, 54) . " $num " . &validate_msg($site, 55);
            } 
    }
    
    return $error;
} #/validatesku

##############################################################################
=pod

=head2 validatebrand

validatebrand - &validatebrand("3-AA", $site, $dbh);

=over 4

=item DESCRIPTION

Takes an objid of a brand and validates it for:
    * It is in the format ###...###-AA 
       where ###...### are any number of numeric values
             AA is 2 capital alphabetical characters

    * If it already exists, then return "", otherwise return 1.

Return the error message.

=item INPUT

$objid - Objid of the brand

$site - Current site parameter

$dbh - Current active database handler

=item OUTPUT

$error - "" if the value already exists in the things table and it was a valid
         objid.
       - 1 if there was an error

=item NOTES

=back

=cut
###############################################################################
sub validatebrand {
    my ($objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($nums, $letters, $sts,$error, $q, $num, $select_hashref, @sql_results);
    my ($where_indx);

    $error="";
    
    if (length ($objid) == 0) {
        $error = 1;
        return $error;
    }
    
    ($nums,$letters) = split (/-/,$objid);
    
    if ((length $nums == 0) || (length $letters == 0)) {
        $error = 1;
        return $error;
    }
    
    if ($nums =~ /(\D)+/) {
        $error = 1;
        return $error;
    }
    
    if ($letters =~ /([^a-zA-Z])+/) {
        $error = 1;
        return $error;
    }
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh);
    $num = &query_getAll($q, $site, $dbh, \@sql_results);

    if ( $num==1 ) {
        $error='';
        return $error;
        
        } else {
        $error = 1;
        return $error;
    }
    
} #/validate brand

##############################################################################
=pod

=head2 ValidateOBJID

ValidateOBJID - &ValidateOBJID("3-AA", $dbh, 2);

=over 4

=item DESCRIPTION

Takes an objid of a brand and validates it for:
    * It is in the format ###...###-AA 
       where ###...### are any number of numeric values
             AA is 2 capital alphabetical characters

    * If it already exists, then return "", otherwise return 1.

Return the error message.

=item INPUT

$objid - Objid of the brand

$site - Current site parameter

$dbh - Current active database handler

$optional - Optional parameter, if set:
             1 - check that the object is a product (134-A or 136-A)
             2 - check that the object is a variant
             3 - check that the object is a permission set
             4 - check that it is a variant set

=item OUTPUT

$error - Flag to indicate the type of error found, or "" if no error found

=item NOTES

=back

=cut
###############################################################################
sub ValidateOBJID {
    my $objid = shift;
    my $site = shift;
    my $dbh = shift;
    my $optional = shift;     # optional parameter - if set, then check if the product is 134-A or 136-A
    
    my ($nums, $letters, $error, @res, $q, $numRows, $select_hashref);
    my ($where_indx);
    
    $error = "";
    
    if (length ($objid) == 0) {
        $error = "INVALID_OBJ";
        return $error;
    }
    
    if ($objid eq '') {
        $error = "INVALID_OBJ";
        return $error;
    }
    
    ($nums,$letters) = split (/-/,$objid);
    
    if ((length $nums == 0) || (length $letters == 0)) {
        $error = "INVALID_OBJ";
        return $error;
    }
    
    if ($nums =~ /(\D)+/) {
        $error = "INVALID_OBJ";
        return $error;
    }
    
    if ($letters =~ /([^A-Z])+/) {
        $error = "INVALID_OBJ";
        return $error;
    }

    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = "ref_from";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'msku'} = "msku";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'vcode'} = "vcode";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objclass'} = "objclass";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objtype'} = "objtype";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh);

    $numRows = &query_getAll($q, $site, $dbh, \@res);

    if ($numRows == 0) {
        $error = "NO_OBJ";
        return $error;
    }

    if ($optional == 2) {   # check that the object is a variant encoding
        if (($res[0]->{'objtype'} ne $site->{'OBJTYPES'}->{'variant'}) &&
	    ($res[0]->{'objclass'} ne $site->{'OBJCLASSES'}->{'variant'})) {
            $error = "NOT_VARIANT";
            return $error;
        }
    }

    if ($optional == 3) {   # check that the object is a permission set
        if ($res[0]->{'objclass'} ne $site->{'OBJCLASSES'}->{'permission_set'}) {
            $error = "NOT_UPERM";
            return $error;
        }
    }

    if ($optional == 4) {   # check that the object is a variant set
        if ($res[0]->{'objclass'} ne $site->{'OBJCLASSES'}->{'variant_set'}) {
            $error = "NOT_VARIANT_SET";
            return $error;
        }
    }
    
    # if we are supposed to check for valid product, check now
    if (($optional == 1) && (($res[0]->{'objclass'} ne "134-A") && ($res[0]->{'objclass'} ne "136-A"))) {
        $error = "INVALID_OBJ";
        return $error;
    }
}

##############################################################################
=pod

=head2 validatelink

validatelink - &validatelink($h, \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

A hashref is passed in which consists on information of a linkage record.

Take the input hashref, and validate all input fields.  Set any errors
found in the $h, and set flags to indicate that validation passed or failed.
Then check whether the data has changed since the last time, and if it has and
validation is ok, then set a flag to tell us to write the changes, if the values
have changed, and validation failed, then set a flag to tell us NOT to write the
changes.

Checks that:
    * The alttitle is not being set for a variant product
    * All other input has valid information 

=item INPUT

$h - Hashref of linkage information

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

* Writes all errors to appropriate location in the FORM.

$h -  The original hashref with additional flags set, $h->{'validationcheck'}="ok" 
    if validation passed, or "" if it fails,  $h->{'changestowrite'}="no" if we are 
    not to write the changes, or "yes" if we are to write the changes.

=item NOTES

=back

=cut
###############################################################################
sub validatelink {
    my ($h) =shift;
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my($i,@temp,$k,$value);
    
    ## adds two fields as well as error messages
    #    changestowrite - "yes|no" if there are changes to write
    #    validationcheck - "ok|-" if the record passes validation
    $h->{'validationcheck'}="ok";   # default
    $h->{'changestowrite'}="no";   # default
    
    ## run through keys and validate _inp
    #
    foreach ( keys %{$h} ) {
        next if ( ! /_inp$/ );
        $k = $_;
        $k =~ s/_inp$//;
        
        $value = validatefield( $k, $h->{$_}, $h, $FORM, $site, $dbh );
        if ( $value ) {
            ## it failed to pass validation
            #
            # V 0.7.29 - changed from OBJ2 to obj2
            if (( $k eq "obj2" )
                and ( $h->{'objid'} =~ /^newrow/ )) {  # no error msg
                $value="" if ( $value =~ /must specify/ );         # for blank skus on
            }                                      # new rows...
            $k.="_err";
            $h->{$k}=$value;
            $h->{'validationcheck'}="";
            } else {
            ## it passed, see if there is something to write
            #
            $k.="_pre";
            if ($k =~ /alttitle/) {
		&debug("In with alttitle, vcode is $h->{'vcode'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
                if (($h->{'alttitle_inp'} ne "") && (defined $h->{'vcode'})) {
                    $h->{alttitle_err}="Error: You cannot specify an alternate title for variant products\n";
		    &debug("Set alttitle error to $h->{'alttitle_err'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
                    $h->{'validationcheck'} = "";
                    $h->{'alttitle_inp'} = "";
                    $h->{'alttitle_pre'} = "";
                    $h->{'alttitle_ini'} = "";
		    &debug("Set validation check to null because $h->{'vcode'} is the vcode", 3, $site->{'debug_level'}, $site->{'debug_log'});
                }
                elsif ($h->{$_} ne $h->{$k}) {
                    $h->{'alttitle_changed'} = "yes";
                }
            }
            
            elsif ( $h->{$_} ne $h->{$k} ) {
                $h->{'changestowrite'}="yes";
		&debug("Writing because \$h->{$_}. $h->{$_} != $h->{$k}", 3, $site->{'debug_level'}, $site->{'debug_log'});
            }
        } # done examining data
    } # end of running through hash keys
    
    
    if ($h->{'validationcheck'} eq "") {
        $h->{'previous_error'} = "yes";
    }
    
    if (($h->{'previous_error'} eq "yes") && ($h->{'validationcheck'} eq "ok")) {
	&debug("Setting changes to write to yes!", 3, $site->{'debug_level'}, $site->{'debug_log'});
        $h->{'changestowrite'} = "yes";
        $h->{'alttitle_changed'} = "yes";
    }
    
    ## just in case...
    $h->{'changestowrite'}="no" if ( $h->{'validationcheck'} ne "ok" );
    
    
    ## cleanup
    #
    undef @temp;
    return $h;
    
} #/validatelink

##############################################################################
=pod

=head2 validatevariant

validatevariant - &validatevariant($h, $site, $dbh);

=over 4

=item DESCRIPTION

A hashref is passed in which consists on information of a variant product.

Take the input hashref, and validate all input fields.  Set any errors
found in the FORM, and set flags to indicate that validation passed or failed.
Then check whether the data has changed since the last time, and if it has and
validation is ok, then set a flag to tell us to write the changes, if the values
have changed, and validation failed, then set a flag to tell us NOT to write the
changes.

This checks that:
    * All variant pulldowns have been set when creating a new variant product.
    * That the variant product does not already exist.
    * All other input has valid input.

=item INPUT

$h - Hashref of input for the variant product.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

* Writes all errors to appropriate location in the FORM.

$h -  The original hashref with additional flags set, $h->{'validationcheck'}="ok" 
    if validation passed, or "" if it fails,  $h->{'changestowrite'}="no" if we are 
    not to write the changes, or "yes" if we are to write the changes.

=item NOTES

=back

=cut
###############################################################################
sub validatevariant {
    my ($h)= shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error_indx, @axis, $i, $numBlank, $vcode, $q, @res, $numRows, $k2, $k, $select_hashref, $where_indx);
    my ($num);
    my ($objid);
    ## adds two fields as well as error messages
    #    changestowrite - "yes|no" if there are changes to write
    #    validationcheck - "ok|-" if the record passes validation
    
    $h->{'validationcheck'}="ok";   # default
    $h->{'changestowrite'}="no";   # default
    
    undef @axis;
    #   undef $objid;
    foreach $k ( keys %{$h} ) {
        if ($k =~ /objid/) {
            $objid = $h->{$k};
	    &debug("Saving $k since it is the objid --> $objid!!!", 3, $site->{'debug_level'}, $site->{'debug_log'});
            next;
        }
        
        #      next if (($k =~ /objid/) && ($objid = $h->{$k}));  # save the objid for axis varification
        next if ( $k !~ /_inp$/ ); # we are only interested in the input fields
        
        # save the axis definitions
        if ($k =~ /axis/) {
            if ($k =~(/\w*(\d+)\w*/)) {
                $num = $1;
            }
            # If it is an axis, they must specify all of them, or don't write it....
            if ($h->{$k} eq "") {
                $h->{'validationcheck'} = "";    #invalid!
		&debug("Set \$h->{'validationcheck'} = $h->{'validationcheck'} since have a blank axis name for $num!", 3, $site->{'debug_level'}, $site->{'debug_log'});
            }
            $axis[$num] = $h->{$k};
            next;
        }
        
        # Check if the values changed
        $k2 = $k . "_ini";
        if ( $h->{$k} ne $h->{$k2} ) {
            $h->{'changestowrite'}="yes";
        }
    }
    
    # If we didn't pass the validation check completely check validity of the axis names
    if ($h->{'validationcheck'} ne "ok") {
        for ($i=1; $i <= $#axis; $i++) {
            if ($axis[$i] eq "") {
                $numBlank++;
            }
        }
        
        # If all not all of the axis pulldowns were blank, then they must get an error
        if ($numBlank != $#axis) {
            #	   $error_indx = "row.$objid.axis_err";
	    # "Error - You must configure all variants to add a product";
            $h->{'axis_err'} = &validate_message($site, 56);
        }
        else {
        }
    }
    
    else {   # validation went good so far, set the vcode if necessary, and check if it already exists
        
        if (defined @axis) {   # there were axis names, and it was valid, add vcode
	    &debug("Setting the VCODE in validatevariant!", 3, $site->{'debug_level'}, $site->{'debug_log'});
            $vcode = $axis[1] if (defined $axis[1]);
            $vcode .= ":" . "$axis[2]" if (defined $axis[2]);
            $vcode .= ":" . "$axis[3]" if (defined $axis[3]);
            $vcode .= ":" . "$axis[4]" if (defined $axis[4]);
            $h->{'vcode'} = $vcode;
            
            if (defined $vcode) {
		&debug("VCODE IS in Validate Field!  $vcode", 3, $site->{'debug_level'}, $site->{'debug_log'});
            }
            
	    $select_hashref = {};
	    $select_hashref->{'TABLE'}->{'NAME'} = "things";
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
	    $where_indx = 0;
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("vcode", $vcode, "and");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("ref_from", $h->{'master_objid'}, "and");
	    $select_hashref->{'statement'} = "SELECT";
	    $q = &build_sql($select_hashref, $site, $dbh);
            $numRows = &query_getAll($q, $site, $dbh, \@res);
	    &debug("$q\n\t-->$numRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});            
            if ($numRows > 0) {
                $h->{'validationcheck'} = "";
                # "Error - The variant product already exists!"
		$h->{'axis_err'}= &validate_message($site, 57);
            }
            
        }
        
    }
    
    $h->{'changestowrite'}="no" if ( $h->{'validationcheck'} ne "ok" );
    return $h;
    
} #/validatevariant

##############################################################################
=pod

=head2 validate_object_def

validate_object_def - &validate_object_def(\%FORM, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

Take the FORM, and validate all input fields.  Set any errors
found in the FORM, and set flags to indicate that validation passed or failed.
Then check whether the data has changed since the last time, and if it has and
validation is ok, then set a flag to tell us to write the changes, if the values
have changed, and validation failed, then set a flag to tell us NOT to write the
changes.

=item INPUT

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$row_info - If this is a row, it needs to have a row.#. tag before the input field, to
            display the error in the right location.  This is what is stored in here if it is
            a row.  If not, it is just "" or undefined

=item OUTPUT

* Writes all errors to appropriate location in the FORM.

$flags - Hashref with flags, $flags->{'validationcheck'}="ok" if validation passed, or "" if it
         fails,  $flags->{'changestowrite'}="no" if we aren't supposed to write the changes,
         or "yes" if we are to write the changes.

=item NOTES

=back

=cut
###############################################################################
sub validate_object_def {
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($row_info) = shift;


   my ($error_indx, @axis, $i, $numBlank, $vcode, $q, @res, $numRows, $k2, $k, $flags);
    my ($kt, $error);

   ## adds two fields as well as error messages
   #    changestowrite - "yes|no" if there are changes to write
   #    validationcheck - "ok|-" if the record passes validation 


   $flags->{'validationcheck'}="ok";   # default
   $flags->{'changestowrite'}="no";   # default

   foreach $k ( keys %{$FORM} ) {
      next if ( $k !~ /_inp$/ ); # we are only interested in the input fields
      next if ($k =~ /regenerate/);  # don't worry about regenerate page flag
      $kt = $k;
      $kt =~ s/_inp$// ;

      &debug ("Going into validatefield with $kt, $$FORM{$k}", 3, $site->{'debug_level'}, $site->{'debug_log'});
      $error = validatefield( $kt, $$FORM{$k}, $FORM, $FORM, $site, $dbh);
      if ( $error ) {
	  ## it failed to pass validation
	  $kt.="_err";

	  if (defined $row_info) {
	      $kt = $row_info . "." . $kt;
	  }
	  &debug ("Set $kt to $error", 3, $site->{'debug_level'}, $site->{'debug_log'});
	  $$FORM{$kt}=$error;

	  &debug ("\tmake sure: $$FORM{$kt}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	  $flags->{'validationcheck'}="";
      }
      else {
#	  # Check if the values changed
#	  if (defined $row_info) {
#	      $k = $row_info . "." . $k;
#	  }
	  $k2 = $k;
	  $k2 =~ s/_inp$// ;
	  $k2 = $k2 . "_pre";
	  &debug ("Seeing if we need to write: $k vs $k2, $$FORM{$k} vs $$FORM{$k2}", 3, $site->{'debug_level'}, $site->{'debug_log'});

	  if ( $$FORM{$k} ne $$FORM{$k2} ) {
	      &debug ("\tYes!  $$FORM{$k} vs $$FORM{$k2}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	      $flags->{'changestowrite'}="yes";
	  }
      }
  }


   if ($flags->{'validationcheck'} eq "") {
       $$FORM{'previous_error'} = "yes";
       $$NVP{'previous_error'} = "yes";
   }

   if (($$FORM{'previous_error'} eq "yes") && ($flags->{'validationcheck'} eq "ok")) {
       $flags->{'changestowrite'} = "yes";
   }

    if (defined $$FORM{'is_a_new_row'} && $flags->{'changestowrite'} eq "yes") {
	$flags->{'new_row_has_input'} = "yes";
    }
   $flags->{'changestowrite'}="no" if ( $flags->{'validationcheck'} ne "ok" );
   return $flags;

} #/validate_object_def

##############################################################################
=pod

=head2 validate_row

validate_row = &validate_row($h, \%FORM, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

Take a row (subset of the FORM data) of user input, and validate each field in 
the row.  Set any errors found in the FORM, and set flags to indicate that 
validation passed or failed.  Then check whether the data has changed since 
the last time, and if it has and validation is ok, then set a flag to tell us 
to write the changes, if the values have changed, and validation failed, then 
set a flag to tell us NOT to write the changes.  

=item INPUT

$h - Hashref which is one row of input data (either price or inventory use this)

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

* Writes all errors to appropriate location in the FORM.

$check - Hashref with flags, {'validationcheck'}="ok" if validation passed, or "" if it
         fails,  $check->{'changestowrite'}="no" if we aren't supposed to write the changes,
         or "yes" if we are to write the changes.

=item NOTES

=back

=cut
###############################################################################
sub validate_row {
    my ($h) = shift;
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($error_indx, @axis, $i, $numBlank, $vcode, $q, @res, $numRows, $k2, $k,$key);
    my ($check, $kt, $error, $objid, $junk, @price_res, $price, $select_hashref, $where_indx);
    
    ## adds two fields as well as error messages
    #    changestowrite - "yes|no" if there are changes to write
    #    validationcheck - "ok|-" if the record passes validation
    
    $check->{'validationcheck'}="ok";   # default
    $check->{'changestowrite'}="no";   # default
    
    foreach $k ( keys %{$h} ) {
        next if ( $k !~ /_inp$/ ); # we are only interested in the input fields
        
        $kt = $k;
        $kt =~ s/_inp$// ;
        $error = validatefield( $kt, $h->{$k}, $h, $FORM, $site, $dbh);
        if ( $error ) {
	    &debug("Failed validation: $error, $kt, $h->{$k}", 3, $site->{'debug_level'}, $site->{'debug_log'});              
            ## it failed to pass validation
            $kt.="_err";
            if ($h->{key}) {
                $kt = "row.$h->{key}.$kt";
		&debug("Set $kt for form!", 3, $site->{'debug_level'}, $site->{'debug_log'});              
            }
	    &debug("Setting error: $error FOR $kt!", 3, $site->{'debug_level'}, $site->{'debug_log'});    
            $$FORM{$kt}=$error;
            $$NVP{$kt} = $error;
            $h->{$kt} = $error;
            $check->{'validationcheck'}="";
        }
        else {
            # Check if the values changed
            $k2 = $k;
            $k2 =~ s/_inp$// ;
            $k2 = $k2 . "_pre";
            
            
            if ( $h->{$k} ne $h->{$k2} ) {
		&debug("Changes to write because \$h->{$k} ne \$h->{$k2} --> $h->{$k} ne $h->{$k2}", 3, $site->{'debug_level'}, $site->{'debug_log'});              
                $check->{'changestowrite'}="yes";
            }
        }
    }
    
    # special check for prices, do not allow for same price & unitct more than once
    if ( ($check->{'changestowrite'} eq "yes") && ((defined $h->{'unitpr_inp'}) || (defined $h->{'currency_inp'}))) {
        ($objid, $junk) = split /_/,$h->{'objid'};
            
	$select_hashref = {};
	$select_hashref->{'TABLE'}->{'NAME'} = "prices";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'unitpr'} = "unitpr";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'unitct'} = "unitct";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "status";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'groupid'} = "groupid";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'currency'} = "currency";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'autogen'} = "autogen";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'pkflag'} = "pkflag";
	$where_indx = 0;
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

	$select_hashref->{'statement'} = "SELECT";
	$q = &build_sql($select_hashref, $site, $dbh);
        $numRows = &query_getAll($q, $site, $dbh, \@price_res);
	&debug("$q\n\t-->$numRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});            
        
        for ($i = 0; $i < $numRows; $i++) {
            $price = $price_res[$i];
            if (($h->{'unitpr_inp'} != $h->{'unitpr_pre'}) || 
		((defined $h->{'currency_inp'}) && ($h->{'currency_inp'} ne $h->{'currency_pre'}))) {
                if ( ($price->{'unitpr'} == $h->{'unitpr_inp'}) &&
                    ($price->{'unitct'} eq $h->{'unitct_ini'}) &&
                    ($price->{'groupid'} eq $h->{'groupid_inp'}) &&
                    ($price->{'currency'} eq $h->{'currency_inp'}) &&
                    ($price->{'autogen'} eq 'N') &&
                    ($price->{'pkflag'} eq 'N')) {
                    
                    $kt ="unitpr_err";
                    if ($h->{key}) {
                        $kt = "row.$h->{key}.$kt";
                    }
                    # "Error - Unit count of $h->{'unitct_ini'}, price of $h->{'unitpr_inp'} $h->{'currency_inp'} already exists"
		    $error = &validate_message($site, 58) . " $h->{'unitct_ini'}, " . &validate_message($site, 59) . " $h->{'unitpr_inp'} $h->{'currency_inp'} " . &validate_message($site, 60);
                    $$FORM{$kt}=$error;
                    $$NVP{$kt} = $error;
                    $h->{$kt} = $error;
                    $check->{'validationcheck'}="";
                }
            }
            
        }
    }
    $check->{'changestowrite'}="no" if ( $check->{'validationcheck'} ne "ok" );
    return $check;
    
} #/validate_row

##############################################################################
=pod

=head2 validateemail

validateemail - &validateemail("xyz@domain.com");

=over 4

=item DESCRIPTION

Take a email address which is entered and do reasonability checking on it.
If it fails the validation, return an error code.

=item INPUT

$email - Email address to validate

=item OUTPUT

$error - String representing the error detected, or "" if no error

=item NOTES

=back

=cut
###############################################################################
sub validateemail {
    my ($email) = shift;

    my($error, @e);

    $error="";
    if ( ! $email ) {
        # Error - no email address entered!
        $error="a";
        return $error;
    };
    
    if ($email !~ /\@/) {
        # Error - Wrong email address format, need \@
        $error ="b";
        return $error;
    };
    
    @e = split "\@", $email;
    
    if ($e[0] !~ /(\w+)/ || $e[1] !~ /\w+\.\w+/ || length($e[1])< 4 ) {
        # Error - Invalid Email address!
        $error ="c";
        return $error;
    }
    
    return $error;
} #/validateemail

##############################################################################
=pod

=head2 validateaddr

validateaddr - &validateaddr(%information, $site);

=over 4

=item DESCRIPTION

Validate a user address is in valid format, if not, then return error messages

=item INPUT

%information - Hash of user information, name and address

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

%addr - Hash of address information, $addr{errors} is set to the error message
        if errors were detected. 

=item NOTES

=back

=cut
###############################################################################
sub validateaddr {
    my( $f,%addr,$k,$k2,$err,$e, $kr, %information);

    %information = ();
    %information = %{$_[0]} if ($_[0]);
    my ($site) = $_[1];

    # fieldlist
    $f="addrid,status,firstname,lastname,address1,address2,city,county,state,postcode,";
    $f.="country,dayphone";
    
    $err="";
    # build address hashref
    foreach $k ( keys %information ) {
        $kr=$k;
        $kr=~s/(_inp|_pre|_ini|_err)$// ;
        next if ( $f !~ /$kr/ );
        $addr{$k}=$information{$k};
        next if ( $k !~ /_inp$/ );
        
        # run the validation
        $k2 = $kr."_err";
        $e="";
        ## quick decision tree
        #     based on field name
        if ( $k =~ /^firstname/ ) {
            # "Error -  Please enter your first name here, must be at least two letters"
	    $e = &validate_message($site, 61) if (length($information{$k})<2 );
            } 
	elsif ( $k =~ /^address1/ ) {
            # "Error - Please enter street number and name here" 
            $e = &validate_message($site, 62) if (length($information{$k})<2);
	} 
	elsif ( $k =~ /^lastname/ ) {
            # "Error - Please enter your last name here, must be at least two letters"
	    $e = &validate_message($site, 63) if (length($information{$k})<2);
	}
	elsif ( $k =~ /^city/ ) {
            # "Error - Please enter the name of the city"
	    $e = &validate_message($site, 64) if (length($information{$k})<2);
	} 
	elsif ( $k =~ /^state/ ) {
            # "Error - Please select your state from this list" 
	    $e = &validate_message($site, 65) if (length($information{$k})<2);
	} 
	elsif ( $k =~ /^postcode/ ) {
            # "Error - Please enter at least your 5 digit zip code "
            $e = &validate_message($site, 66) if (length($information{$k})<5);
            } 
	elsif ( $k =~ /^dayphone/ ) {
            $addr{$k}=~tr/-\(\)[]{}.A-Za-z //d;
	    # "Error - Please enter your telephone number including your Area Code"
	    $e = &validate_message($site, 67) if ((length($addr{$k})!=10) && (length($addr{$k})!=11));
	} # done with fields we care about
            
	$addr{$k2}=$e if ( $e );
	$err.=$e if ( $e );
    } # done constructing address href
    $addr{'errors'}=$err if ( $err );
    
    return \%addr;
} #/validateaddr

############### Start of Admin Object routines ###############################
##############################################################################
=pod

=head2 validate_attributes

validate_attributes - &validate_attributes(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Take the FORM, and validate all input fields.  Set any errors
found in the FORM, and set flags to indicate that validation passed or failed.
Then check whether the data has changed since the last time, and if it has and
validation is ok, then set a flag to tell us to write the changes, if the values
have changed, and validation failed, then set a flag to tell us NOT to write the
changes.

=item INPUT

$FORM - Hashref to the current page that is displayed

$extras - Hashref for extra information, used for validation

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

* Writes all errors to appropriate location in the FORM.

$flags - Hashref with flags, $flags->{'validationcheck'}="ok" if validation passed, or "" if it
         fails,  $flags->{'changestowrite'}="no" if we aren't supposed to write the changes,
         or "yes" if we are to write the changes.

=item NOTES

validate_object_def should be renamed to this, and then this can go away.  This is
more meaningful in the newer versions of AllCommerce.

=back

=cut
###############################################################################
sub validate_attributes {
    my ($FORM) = shift;
    my ($extras) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($flags);

    &debug("validate_attributes: Going to validate, it is row? $$FORM{'row_info'}", 3, $site->{'debug_level'},$site->{'debug_log'});
    $flags = &validate_object_def($FORM, $extras, $site, $dbh, $$FORM{'row_info'});
    &debug_hashref($FORM, 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $flags;
}

##############################################################################
=pod

=head2 validate_object 

validate_object - &validate_object();

=over 4

=item DESCRIPTION

Sometimes all of the input fields can be valid, but a combination of them is
invalid, or additional information must be found out after we know that everything
is individually valid, in order to determine whether an entire object definition
is valid. 

This routine does specific checks for certain objects.  It verifies that the object
on a whole is valid, or invalid.

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

TARA NEED TO OVER-RIDE certain indivdual attribute checks for the object, ie brief
is not 1000 for attributes, it is at most 20.

=back

=cut
###############################################################################
sub validate_object {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    &debug("In validate_object", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($input_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($$input_hashref{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'attribute'}) {
	&debug("validate_object: Need to validate attribute definition", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return (&validate_attribute_object($input_hashref, $site, $dbh));
    }

    return;
}

##############################################################################
=pod

=head2 validate_variant_product

validate_variant_product - &validate_variant_product(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Validate that the variant product is not already defined, and that all of the
axis pulldowns are selected.

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_variant_product {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@results, $q, $numRows, $results, $axisnames, @axis, $numAxis);
    my ($error, $err, $select_hashref, $i, $axis_check);
    my (@vcode);

    &debug("validate_variant_product: in here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # Set up the error tag
    if ($$input_hashref{'row_info'}) {
	$err = $$input_hashref{'row_info'} . ".axis0_err";
    }
    else {
	$err = "axis0_err";
    }

    # Make sure they specified all variant pulldowns
    $select_hashref = {};
    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'axisnames'} = "axisnames";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $$input_hashref{'master_objid'}, "and");
    $q =  &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@results);
    # The variant product already exists
    if ($numRows > 0) {
	@axis = split /\:/, $results[0]->{'axisnames'};
    }
    @vcode = split /\:/, $$input_hashref{'vcode_inp'};

    &debug("validate_variant_product: There should be $#axis pulldowns initialized, and there are $#vcode vcodes set", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($#axis != $#vcode) {
	$$input_hashref{$err} = $site->{'script_msg'}{'set_variant_pulldowns'};
	&debug("validate_variant_product: Returning $$input_hashref{$err} in $err", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{'7'};
    }

    # See if the product already exists
    $select_hashref = {};
    $select_hashref->{'statement'} = "SELECT";
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("ref_from", $$input_hashref{'master_objid'}, "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("vcode", $$input_hashref{'vcode_inp'}, "and");
    $q =  &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@results);

    # The variant product already exists
    if ($numRows > 0) {
	$$input_hashref{$err} = $site->{'script_msg'}{'variant_already_exists'};
	&debug("validate_variant_product: Returning $$input_hashref{$err} in $err", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{'7'};
    }


    return;
}

##############################################################################
=pod

=head2 validate_attribute_object

validate_attribute_object - &validate_attribute_object(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Validate that all of the information for the attribute definition work 
together.  Checks the following:
    - fieldname is unique
    - if the html is a select file, verifies that the fieldname is the same as
        the sel filename after the sel_
    - That the fieldname and Description are no more than 20 characters long

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_attribute_object {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($changed_items, $field, $check_html_def, $select_file_name, $junk, $fieldname);
    my ($error, $has_error);
    &debug("validate_attribute_object: in here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $check_html_def = "no";
    $select_file_name = "";
    $changed_items = &get_input_that_changed($input_hashref);
    &debug_hashref($changed_items, 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $field (keys %{$changed_items}) {
	# check that the fieldname doesn't already exist
	if ($field eq "title") {
	    $error = &validate_attribute_fieldname($input_hashref, $site, $dbh);
	    if ($error) {
		$has_error = $error;
	    }
	    $check_html_def = "yes";
	    $fieldname = $changed_items->{$field}->{'inp'};
	}		

	# if the html field is select, then make sure that the sel file is sel_'title'
	if ($field eq "html") {
	    $error = &validate_html_definition($input_hashref, $site, $dbh);
	    if ($error) {
		$has_error = $error;
	    }

	    if ($changed_items->{$field}->{'inp'} =~ /select:/) {
		($junk, $select_file_name) = split /\:/, $changed_items->{$field}->{'inp'};
	    }
	    $check_html_def = "yes";
	}

	if ($field eq "brief") {
	    $error = &validate_attribute_brief($input_hashref, $site, $dbh);
	    if ($error) {
		$has_error = $error;
	    }
	}
    }    

    if ($check_html_def eq "yes") {  
	$error = &validate_html_select_definition($input_hashref, $fieldname, $select_file_name, $site, $dbh);
	if ($error) {
		$has_error = $error;
		if (defined $changed_items->{'title'}) {
		    $$input_hashref{'title_inp'} =   $$input_hashref{'title_pre'};
		    $$input_hashref{'title_err'} =   $error;
		}

		if (defined $changed_items->{'html'}) {
		    $$input_hashref{'html_inp'} =   $$input_hashref{'html_pre'};
		    $$input_hashref{'html_err'} =   $error;
		}
	    }
	}

    if ($has_error) {
	return $site->{'script_msg'}{7};
    }
    return;
}


##############################################################################
=pod

=head2 get_input_that_changed

get_input_that_changed - &get_input_that_changed(\%FORM);

=over 4

=item DESCRIPTION

Parse through the $input_hashref, and find all _inp fields that are different
from their _pre fields.  Copy these into a new hashref keyed by fieldname and
then by 'inp' and 'pre'.

=item INPUT

$input_hashref - Pointer to the input hash, usually FORM.

=item OUTPUT

$changed - Hashref that lists all fields that have changed as the main key,
           with keys 'inp' which was the new input, and 'pre' which was the
           previous input for these field listed under the field.

=item NOTES

=back

=cut
###############################################################################
sub get_input_that_changed {
    my ($input_hashref) = shift;

    my ($changed, $field, $pre, $key);

    foreach $key (keys %{$input_hashref}) {
	next if ($key !~ /_inp$/);  # only get input fields
	$field = $key;
	$field =~ s/_inp//gms;
	$pre = $field . "_pre";
	if ($$input_hashref{$key} ne $$input_hashref{$pre}) {
	    $changed->{$field}->{'pre'} = $$input_hashref{$pre};
	    $changed->{$field}->{'inp'} = $$input_hashref{$key};

	}
    }

    return $changed;
}

##############################################################################
=pod

=head2 validate_attribute_brief

validate_attribute_brief - &validate_attribute_brief(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Validates the brief field when defining an attribute object.  Makes sure that the
brief is between 5 & 20 characters.  We don't want it too long here, as we want it
to show up on displays and in the attach/detach screen nicely

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_attribute_brief {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, $brief_len);

    $brief_len = length ($$input_hashref{'brief_inp'});

    if (($brief_len > 20) || ($brief_len < 5)) {
	$error = $site->{'script_msg'}{'between_5_and_20'};
    }
    if ($error) {
	$$input_hashref{'brief_err'} = $error;
	$$input_hashref{'brief_inp'} = $$input_hashref{'brief_pre'};
	return $site->{'script_msg'}{7};
    }
    return;
}

##############################################################################
=pod

=head2 validate_attribute_fieldname

validate_attribute_fieldname - &validate_attribute_fieldname(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Validates the fieldame for a new/modified attribute.  Make sure that it is not already
used for another attribute, that it doesn't have any white space, and that it is no
longer than 20 characters.

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_attribute_fieldname {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error);

    # no spaces
    if ($$input_hashref{'title_inp'} =~ / /) {
	$error = $site->{'script_msg'}{'no_spaces_allowed'};
    }
    elsif (length($$input_hashref{'title_inp'}) > 20) {
	$error = $site->{'script_msg'}{'only_20'};
    }
    else {
	$error = &check_attribute_fieldname($$input_hashref{'title_inp'}, $site, $dbh);
    }
    

    if ($error) {
	$$input_hashref{'title_err'} = $error;
	$$input_hashref{'title_inp'} = $$input_hashref{'title_pre'};
	return $site->{'script_msg'}{7};
    }

    return;
}


##############################################################################
=pod

=head2 check_attribute_fieldname

check_attribute_fieldname - &check_attribute_fieldname();

=over 4

=item DESCRIPTION

This verifies that the fieldname defined in an attribute object definition:
    - is not already taken

=item INPUT

$field - Name of the field input in the attribute object definition

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub check_attribute_fieldname {
    my ($field) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $sql_hashref, $num_rows, @results);

    $sql_hashref->{'statement'} = "SELECT";
    $sql_hashref->{'TABLE'}->{'NAME'} = "things";
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $sql_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("title", $field, "and");
    $sql_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'attribute'}, "and");
    $q = &build_sql($sql_hashref, $site, $dbh, 1);
    $num_rows = &query_getAll($q, $site, $dbh, \@results);
    if ($num_rows > 0) {
	# the field is already taken, cannot use this fieldname
	&debug("check_attribute_fieldname: Already chosen: $site->{'script_msg'}{'not_unique_fieldname'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return ($site->{'script_msg'}{'not_unique_fieldname'});
    }
}

##############################################################################
=pod

=head2 validate_html_definition

validate_html_definition - &validate_html_definition(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Validates the html definition for attribute definitions.  It checks:
    - that an html definition is specified

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_html_definition {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    if (length($$input_hashref{'html_inp'}) == 0) {
	# This must be specified
	return $site->{'script_msg'}{'must_specifiy'};
    }

    return;

}

##############################################################################
=pod

=head2 validate_html_select_definition

validate_html_select_definition - &validate_html_select_definition(\%FORM, $fieldname, $selectname, $site, $dbh);

=over 4

=item DESCRIPTION

Verifies that when an html definition string for a select file is given, that
the attribute fieldname is the same as the select file name after the sel_.
Therefore, if the select file name is sel_???? then the fieldname must be ????.

=item INPUT

$input_hashref - Pointer to FORM hash

$fieldname - The name of the fieldname of the attribute

$select_file - Name of the select file (sel_????).

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if the object is invalid, undef otherwise.

=item NOTES

=back

=cut
###############################################################################
sub validate_html_select_definition {
    my ($input_hashref) = shift;
    my ($fieldname) = shift;
    my ($select_file) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($junk);

    if ($select_file eq "") {  # we need to see if the html is a select, it didn't change.
	&debug("validate_html_select_definition: See if $$input_hashref{'html_pre'} matched $fieldname", 3, $site->{'debug_level'}, $site->{'debug_log'});
	if ($$input_hashref{'html_pre'} =~ /select:/) {
	    ($junk, $select_file) = split /\:/, $$input_hashref{'html_pre'};
	}
	&debug("validate_html_select_definition: filename is $select_file",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    # if the html previously defined is a select, make sure that they still match
    if ($select_file ne "") {
	if ($fieldname eq "") {
	    # The field name didn't change, so use previous value
	    $fieldname = $$input_hashref{'title_pre'};
	}
	$select_file =~ s/sel_//gms;
	if ($select_file ne $fieldname) {
	    return ($site->{'script_msg'}{'must_match_select'});
	}

    }
    return;
}

############### End of Admin Object routines #################################
##############################################################################
=pod

=head2 validate_message

validate_message - $ret = &validate_message($site, 1);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This returns the error 
message based the number passed in.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$msg_number - The number of the message to be returned

=item OUTPUT

The text of the message in the appropriate language.

=item NOTES

=back

=cut
###############################################################################
sub validate_message {
    my ($site) = shift;
    my ($msg_number) = shift;
    
    my (@temp);
    my (%script_msg);

    # check the language that is being used and open appropriate
    # message language file or get messages below
    if ($site->{'om_render_lang'} eq "eng" ) {
        # validatefield
        $script_msg{1} = "Error - Alternate title should be 0 or 4 to 40 chars";
        $script_msg{2} = "Error - The object entered could not be found";
	$script_msg{3} = "Error - Invalid Object ID";
	$script_msg{4} = "Error - Must be a positive number.";
	$script_msg{5} = "Error - Must specify a brand";
	$script_msg{6} = "Error - The name must end in _gui";
	$script_msg{7} = "Must be";
	$script_msg{8} = "letters";
	$script_msg{9} = "Error - ";
	$script_msg{10} = "letter code not unique";
	$script_msg{11} = "Error - You cannot change this value";
	$script_msg{12} = "Error - You must specify a name";
	$script_msg{13} = "Error - This is an internal record required by AllCommerce! You cannot use the name ";
	$script_msg{14} = "Error - The name must start with sel_"; 
	$script_msg{15} = "Error - The name must have all lowercase characters";
	$script_msg{16} = "Error - Each line must have the syntax code:text";
	$script_msg{17} = "Error - Each line must have the syntax code|text";
	$script_msg{18} = "Error - Maximum of 4 objects allowed";
	$script_msg{19} = "Error - The object entered is not a variant";
	$script_msg{20} = "Error - You cannot modify the axisnames";            
	$script_msg{21} = "Error - This does not appear to be a good brand.";
	$script_msg{22} = "Error - Should be less than 1000 characters";
	$script_msg{23} = "Error - Should be between 5 and 1000 characters";
	$script_msg{24} = "Error - Weight should be blank or an integer from 1 through 100";
	$script_msg{25} = "Error - Must be a positive number";
	$script_msg{26} = "Error - Value is too large";
	$script_msg{27} = "Error - Value is too small";
	$script_msg{28} = "Error - You must enter a numeric value";
	$script_msg{29} = "Error - Must be a positive number";
	$script_msg{30} = "Error - You must enter an integer value";
	$script_msg{31} = "Error - You must enter a weight";
	$script_msg{32} = "Error - You must specify a warehouse";
	$script_msg{33} = "Error - You must specify a number between 0 and 99999.99";
	$script_msg{34} = "Error - You can only have 2 numbers after the decimal point";
	$script_msg{35} = "Error - Must be of length ";
	$script_msg{36} = "Error - Should be between 3 and 80 characters";
	$script_msg{37} = "Error - The object entered is not a variant set";

        # validate_payment
	$script_msg{38} = "Please select a credit card name.";
	$script_msg{39} = "Invalid credit card number.";
	$script_msg{40} = "There were problems with your payment";
	$script_msg{41} = "There were problems with your information";
	$script_msg{42} = "Invalid MasterCard number";
	$script_msg{43} = "Invalid VISA number";
        $script_msg{44} = "Credit card number not valid.";
        $script_msg{45} = "Customer Service 900 numbers are still very rare";
	$script_msg{46} = "This information is required as a security precaution";
	$script_msg{47} = "Please select month the credit card expires.";
	$script_msg{48} = "Please select the year the credit card expires.";
	$script_msg{49} = "This credit card has expired.";
	
        # validatesku
	$script_msg{50} = "Error - SKU is blank!";
	$script_msg{51} = "Error - SKUs are 8 characters long!";
	$script_msg{52} = "Error - SKUs are 3 characters, a store code, then four digits";
	$script_msg{53} = "Error - nothing found";
	$script_msg{54} = "Error - your SKU should have one record, found";
	$script_msg{55} = "records";
        # validatevariant
	$script_msg{56} = "Error - You must configure all variants to add a product";
	$script_msg{57} = "Error - The variant product already exists!";
	    
        # validate_row
	$script_msg{58} = "Error - Unit count of ";
	$script_msg{59} = "price of";
	$script_msg{60} =  "already exists";
	
        # validateaddr
	$script_msg{61} = "Error -  Please enter your first name here, must be at least two letters";
	$script_msg{62} = "Error - Please enter street number and name here" ;
	$script_msg{63} = "Error - Please enter your last name here, must be at least two letters";
	$script_msg{64} = "Error - Please enter the name of the city";
	$script_msg{65} = "Error - Please select your state from this list";
	$script_msg{66} = "Error - Please enter at least your 5 digit zip code ";
	$script_msg{67} = "Error - Please enter your telephone number including your Area Code";
	
	# Additional Validate Field
	$script_msg{68} = "AllCommerce only supports one warehouse in this version";
	$script_msg{69} = "Error - You must specify at least one variant";
	$script_msg{70} = "Error - You must specify a variant.  (";
        $script_msg{71} = " are not variants)";
	$script_msg{72} = "Error - The name is not unique.";
	$script_msg{73} = "Error - The object entered is not a permission set";
	$script_msg{74} = "Error - You must enter a numeric value";
	$script_msg{75} = "Error - You must specify a number between ";
	$script_msg{76} = "and ";
	$script_msg{77} = "Error - You must enter a positive numeric value";


        # new added 
        $script_msg{80} = "Error - The coupon number cannot be found, please enter again.";
        $script_msg{81} = "Error - The coupon is not in valid period.";
        $script_msg{82} = "Error - The coupon amount you put is more than the amount you need to pay, please remove your credit card payment and try to input coupon before credit card payment.";
        $script_msg{83} = "Error - The payment amount you put is more than the amount you need to pay, please enter again.";
        $script_msg{84} = "Error - You must input a positive payment amount.";
        $script_msg{85} = "Please input your payment information.";

    }
    else {
        open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'script'}.messages");
        while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
        }
        close(EM);
		  $script_msg{$msg_number} = $site->{'script_msg'}{$msg_number} if $site->{'script_msg'}{$msg_number};
    }
    return $script_msg{$msg_number};
}
1;




