use strict;
###############################################################################
=pod

=head1 boxes.pm

boxes.pm -- Library that implements a WMS "box" object.

=head1 DESCRIPTION

This library wraps wms_object.pm specifically for boxes. It defines the 
format of a box, and manages all the details of creating, saving, etc.

=head1 NOTES & BUGS

None yet

=cut
###############################################################################
###############################################################################
=pod

=head2 get_boxes

get_boxes - &get_boxes($site, $boxids);

=over 4

=item DESCRIPTION

Fetches all box objects from a list of boxids.

=item INPUT

$site   -- Standard site hashref

$boxids -- Array ref of box id's

=item OUTPUT

$boxes -- Array ref of complete box objects matching input list.

=item NOTES

Tailor-made to pass off the list of boxids found by the search routines to, and 
easily get multiple boxes.

=back

=cut
###############################################################################
sub get_boxes {
	my ($site, $boxids) = @_;
	
	my @ret;
	foreach my $id (@{$boxids}) {
		my $box = &get_one_box($site, $id);		
		push @ret, $box;
	}
	
	return \@ret;
}
		
###############################################################################
=pod

=head2 get_one_box

get_one_box - &get_one_box($site, $boxid);

=over 4

=item DESCRIPTION

The box-specific wrapper for &get_one_object(). Should always be used to fetch 
a single box. For multiple boxes, use the above &get_boxes(), which calls this 
iteratively.

=item INPUT

$site  -- Standard site hashref

$boxid -- object id of the box to fetch

=item OUTPUT

$box -- The box object found, or undef if there's an error.

=item NOTES

This method makes sure that 'nosave' is filled in properly, and that $box->{'location'} and 
$box->{'edit'} are both valid arrayrefs.

=back

=cut
###############################################################################
sub get_one_box {
	my ($site, $id) = @_;
	
	my $aprops = [
		{fieldname => 'locationid', propname => 'location'},
		{fieldname => 'editid', propname => 'edit'}
	];
	
	my $box = &get_one_object($site, $id, 'wmsbox', $aprops);
	$box->{'nosave'} = 	{
		'objid' => 1,
		'msku' => 1,
		'objclass' => 1,
	};
	
	$box->{'location'} = [] unless (defined $box->{'location'} && (ref($box->{'location'}) eq 'ARRAY'));
	$box->{'edit'} = [] unless (defined $box->{'edit'} && (ref($box->{'edit'}) eq 'ARRAY'));

	return $box;
}

###############################################################################
=pod

=head2 store_box

store_box - &store_box($site, $box);

=over 4

=item DESCRIPTION

Box-specific wrapper for &store_object(). This is almost a pass-through funtion, 
all it does is check that there's an objid for the box, then it calls &store_object() 
on it. Should always be used for box storage though.

=item INPUT

$site -- Standard site hashref

$box  -- The box hashref to store.

=item OUTPUT

$box -- The box, after being stored and re-fetched

$err -- Any errors from the storing process.

=item NOTES

See &store_object() docs in wms_object.pm for more info.

=back

=cut
###############################################################################
sub store_box {
	my ($site, $box) = @_;
	return undef unless $box->{'objid'};
	
	my ($box, $err) = &store_object($site, $box);
	return ($box, $err);
}


###############################################################################
=pod

=head2 get_boxes_by_po

get_boxes_by_po - &get_boxes_by_po($site, $ponum);

=over 4

=item DESCRIPTION

Find boxes that were part of a specific purchase order

=item INPUT

$site  -- Standard site hashref

$ponum -- The PO number to look for

=item OUTPUT

$boxes -- Array ref of complete boxes that have a 'ponum' field matching $ponum. 

=item NOTES

Return arrayref is made up of box *objects* not just object ids.

=back

=cut
###############################################################################
sub get_boxes_by_po {
	my ($site, $ponum) = @_;
	
	my $boxids = &get_objids_by_extraprop($site, 'ponum', $ponum);
	my $boxes = &get_boxes($site, $boxids);
	
	return $boxes;
}

###############################################################################
=pod

=head2 get_boxes_by_date

get_boxes_by_date - &get_boxes_by_date($site, $date);

=over 4

=item DESCRIPTION

A function to easily find boxes that were created on a certain day. 

=item INPUT

$site -- Standard site hashref.

$date -- The date to find in MMDDYYYY format.

=item OUTPUT

$boxes	-- Array reference of box hashes. These are not just ID's, but complete 
box entities, which have a creation date during the day specified.

=item NOTES

If $date is invalid, or not supplied, this function will use the current system 
date as a default. So, it will basically never fail, but may not return what 
you want if the input format is wrong. It should probably default to "today" with 
no date input, but invalid input ought to fail instead.

=back

=cut
###############################################################################
sub get_boxes_by_date {
	my $site = shift;
	my $day = shift;
	use Time::Local;
	
	my $time = time();
	my $rv = &check_date($day);
	
	if ($day && $rv) {
		$day =~ /(\d\d)(\d\d)(\d\d\d\d)/;
		my $m = $1;
		my $d = $2;
		my $y = $3;
		warn "Date is $m $d $y\n";
		$m = $m - 1;
		$y = $y - 1900;
		$time = &timelocal(0, 0, 12, $d, $m, $y);
	}
	
	my @date = localtime($time);
	my $start = &timelocal(0, 0, 0, $date[3], $date[4], $date[5]);
	my $end = &timelocal(59, 59, 23, $date[3], $date[4], $date[5]);
	my $boxids = &get_objids_by_property($site, 
		{1 => {'field' => 'cdate', 'value' => ">$start", 'join' => 'and'}, 
		 2 => {'field' => 'cdate', 'value' => "<$end", 'join' => 'and'},
		 3 => {'field' => 'objclass', 'value' => 'wmsbox', 'join' => 'and'}}, 
		'cdate ASC'); 

	my $boxes = &get_boxes($site, $boxids);

	return $boxes;
}	

	
###############################################################################
=pod

=head2 move_box

move_box - &move_box($site, $boxid, $uid, $location, $newstatus, $oldstatus);

=over 4

=item DESCRIPTION

A function to move a box from one location to another, optionally updating the 
box's status. This is all wrapped together because a box move usually 
involves changing location, status, and keeping records of who did what.

=item INPUT

$site      -- Standard site hashref

$boxid     -- The ID of the box to move (not the full box hashref, just the objid)

$uid       -- The user ID entering the move record

$location  -- The final location, after moving

$newstatus -- The status to change the box to (optional)

$oldstatus -- The status the box should be to start with (optional)

=item OUTPUT

$msg       -- Returns a text message indicating success or failure, and reason.

=item NOTES

Status changes are optional. You may include a newstatus, to change the status 
no matter what it already is, or both a new and old, to first check that the 
status is what you thought it should be, and then change it, if so. Or you 
may include neither, which will not change a box's status at all.

=back

=cut
###############################################################################
sub move_box {
	my ($site, $boxid, $uid, $location, $newstatus, $oldstatus) = @_;
	
	my $rv = &does_obj_exist($site, 'things', {'objid' => $boxid, 'objclass' => 'wmsbox'});
	return "Box $boxid does not exist!\n" unless ($rv);

	my $loc_id = &make_id($site);
	
	my $new_location = {
		'locationid' => $loc_id,
		'uid' => $uid,
		'loc' => $location
	};
	
	my $box = &get_one_box($site, $boxid);
	return "Cannot move box: Box status is not currently '$oldstatus'\n" if ($oldstatus && ($box->{'status'} ne $oldstatus));
	
	
	unshift @{$box->{'location'}}, $new_location;
	
	my ($box, $err) = &store_box($site, $box);
	my $box_txt = &dump_obj($box);
	warn "After move, box is now:\n$box_txt\n";
	
	if ($newstatus) {
		my $form = {
			'status' => $newstatus,
			'boxid' => $box->{'objid'},
			'uid'	=> $uid
		};
		&change_box_status($site, $form);
	}
	
	
	return "Error moving $boxid to $location" if ($err);
	return "Moved box $boxid to $location.";
}


###############################################################################
=pod

=head2 display_box

display_box - &display_box($site, $boxid);

=over 4

=item DESCRIPTION

Format a box for display on a page.

=item INPUT

$site   -- Standard site hashref

$boxid  -- The objid of the box you wish to format

=item OUTPUT

$box_disp -- (String) The box, formatted for display

=item NOTES

Basically, feed this a boxid, and it gives you back a cell suitable for use 
whenever you need to show the full details of a box. Uses cell_box_display, 
cell_box_location_line, and cell_box_edit_line to output the complete box record.

=back

=cut
###############################################################################
sub display_box {
	my ($site, $boxid) = @_;
	
	my $box = &get_one_box($site, $boxid);
	
	return "Cannot find box $boxid\n" unless ($box->{'objid'});
	
	my $head = &localize_template('wm_cell_box_display', $site->{'current_interface'},0,$site);
	my $loc = &localize_template('wm_cell_box_location_line', $site->{'current_interface'},0,$site);
	my $ed = &localize_template('wm_cell_box_edit_line', $site->{'current_interface'},0,$site);

	foreach my $location (@{$box->{'location'}}) {
		$location->{'hdate'} = &cdate_to_hdate($location->{'cdate'});
		$box->{'location_lines'} .= &permutebit($loc, $location);
	}

	foreach my $edit (@{$box->{'edit'}}) {
		$edit->{'hdate'} = &cdate_to_hdate($edit->{'cdate'});
		$box->{'edit_lines'} .= &permutebit($ed, $edit);
	}
	
	my $ret = &permutebit($head, $box);
	return $ret;
}

	
1;
