use strict;
###############################################################################
=pod

=head1 wms_object.pm

wms_object.pm -- This library implements the "WMS Object". Most data objects in 
the WMS system should be "WMS Objects", and should use this module for all fetching, 
storing, and manipulation.

=head1 DESCRIPTION

A WMS Object is pretty simple. It is, at heart, a hash reference. If it is assembled 
and serialized using the following routines, though, it will have some convenient 
properties for use in the WMS. SKUs and Boxes, for example, are WMS objects.

=head1 EXAMPLE

The following gives a basic description of how a WMS object works. For more complete 
info, see the docs for boxes.pm and sku.pm, which are both WMS Objects.

=head2 Creating a new object

Say I want to create a new object. To make this general, we'll say that the object 
I'm creating is a Foo. First, I'd create a hashref to describe my Foo which might 
look like the following:

	my $new_foo = {
	    'objid'    => $id,
	    'objclass' => 'wmsfoo',
	    'name'     => 'Bob',
	    'size'     => '100'
	};

Ok, pretty simple. It's a hashref with some fields. Now, to store this Foo, I would 
call &store_object(), like the following:

	my $error;
	($new_foo, $error) = &store_object($site, $new_foo);

This will create a row in 'things' that stores the objid and the objclass. This 
acts as a pointer to all future object data, which actually lives in extraprop. So, 
along with the 'things' record, &store_object() will also save all the other data 
fields in 'extraprop', each keyed to the main objid. Now, there are some exceptions 
to this, which I'll get into later.

Note that &store_object() returns the object itself. It's important to discard 
your hand-assembled object and continue on using the returned object instead, 
because it has some special properties that you'd rather not worry about managing manually. 

=head2 Getting an existing object

To fetch an object that already exists, you use &get_one_object(). This needs to 
know the objid, the objclass, and also what properties of this object should be 
considered "arrayprops". Basically, to implement an object, you should wrap &get_one_object() 
with a more specific method for your particular object that implements these things.

=head2 What is an array prop?

Some properties of an object are one-dimensional. That is, they have a name and a 
value, and that's it. For example, "name" => "Bob" is a one-dimensional property. 
These are simply stored in extraprops as-is.

But some properties are really like little objects themselves. For example, a Box 
needs to store a history of it's locations-- i.e. "where have I been?". This is an 
arrayprop, because each location record need to know several pieces of information. 
So, a box with a location history ends up looking like:

	$sample_box = {
		objid => 146-AA30276,
		objclass => wmsbox,
		msku => test-01,
		rec_by => rusty,
		status => new,
		quant => 10,
		ponum => 3,
		location => [
			{
				locationid => 161-AA99278
				cdate => 977476706
				loc => res-03
				uid => fred
			},
			{
				locationid => 161-BB16546
				cdate => 977476757
				loc => act-01
				uid => rusty
			}
		]
	};

Ok, it looks a little hairy, I know. But follow it through. All the properties 
that look like "quant => 10" are scalar props. But location is an array prop. 
The field "location" points to an anonymous array reference, which holds (currently) 
two anonymous hash references that describe "location events". That is, every time 
that box got moved, a new location property was added. 

So, when you implement a WMS object, your wrapper code needs to identify and 
manage these array props. As long as it does that, the programmer working with 
objects doesn't need to worry about them. Say I want to add a new location record 
to the above example object. Here's what I'd do:

	# Create the new location record
	my $new_location = {
	    locationid => $loc_id,
	    loc        => 'act-03',
	    uid        => 'user'
	}
	
	# Push it onto the front of the existing list
	unshift @{$sample_box->{'location'}}, $new_location;

	# Save the object
	($sample_box, $error) = &store_box($site, $sample_box);

So, what all this object stuff does is abstract away all the details of where that 
data actually goes. The programmer need only know what an object's structure is like, 
and this takes care of the rest.

Read the docs for these functions, and then look at boxes.pm, which is probably the best 
sample implementation of a WMS object, and it ought to be clear how to use this.

=head1 NOTES & BUGS

None yet

=cut
###############################################################################


###############################################################################
# Main interface fetch and store methods
###############################################################################

###############################################################################
=pod

=head1 Main Interface Methods

The following are the primary useful methods in this module.

=cut
###############################################################################
###############################################################################
=pod

=head2 get_one_object

get_one_object - &get_one_object($site, $objid, $objclass, $arrayprops);

=over 4

=item DESCRIPTION

The basic method to fetch a single object. You should wrap this in a custom 
fetch method for your specific object, which can provide the arrayprop info.

=item INPUT

$site       -- Standard site heshref

$objid      -- The object id to fetch. You must know this before you "fetch". See below for 
some object-finding routines

$objclass   -- The object class to find. Again, your wrapper should provide this.

$arrayprops -- An array reference of hash references, describing what this object's 
array properties are. These hashrefs must include "fieldname" and "propname". For the 
example above ($sample_box), $arrayprops looks like:

	my $arrayprops = [
	    {fieldname => 'locationid', propname => 'location'},
	    {fieldname => 'editid',     propname => 'edit'}
	];

'fieldname' is the name of the extraprop record that is the "container" for this 
property. 'propname' is the field in the object that will hold the array of these 
elements.

=item OUTPUT

$obj -- The fetched WMS Object. Returns undef if the object was not found.

=item NOTES

This is the only "get" method that an object implementor needs to worry about. It should be 
wrapped with an object-specific version, but this should be the only way objects are accessed.

=back

=cut
###############################################################################
sub get_one_object {
	my ($site, $objid, $objclass, $arrayprops) = @_;
	
	my @where;
	$where[0] = &build_where_hash('objid', $objid, 'and');
	$where[1] = &build_where_hash('objclass', $objclass, 'and');	

	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'things',
			 'WHERE'  => \@where,
			 'FIELDS' => 
		 		{
				 'objid' => 'objid',
				 'objclass' => 'objclass',
				 'msku' => 'msku',
				 'cdate' => 'cdate',
				 'mdate' => 'mdate'
				}
			}
	};
	
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "get_one_object: Found $rows for $objid\n";
	
	return undef unless ($results->[0]->{'objid'});
	
	my $obj = $results->[0];
	
	$obj->{'hdate'} = &cdate_to_hdate($obj->{'cdate'});
	$obj->{'mhdate'} = &cdate_to_hdate($obj->{'mdate'});

	$obj = &get_obj_extraprops($site, $obj, $arrayprops);
	
	foreach my $aprop (@{$arrayprops}) {
		$obj = &get_obj_array_props($site, $obj, $aprop->{'fieldname'}, $aprop->{'propname'});
	}	
	
	my $obj_dump = &dump_obj($obj);
	warn "Object is:\n$obj_dump\n";
	
	return $obj;
}

###############################################################################
=pod

=head2 store_object

store_object - &store_object($site, $obj);

=over 4

=item DESCRIPTION

Takes an object hashref and serializes it to the database. This will check to see 
if an object exists already, so the code need not distinguish between saving an 
existing object and creating a new one. This is very general-- given any valid 
WMS object, it will store it properly. Nevertheless, it is still a good idea to wrap 
it in an object-specific method in case future object-specific checking or filtering 
becomes necessary.

=item INPUT

$site -- Standard site hashref

$obj  -- Valid WMS object hash (MUST include a field called 'objid', which must have a vlue)

=item OUTPUT

$obj -- The stored object. The process is store(), fetch() return, so your code 
should use the returned object after storing.

$err -- any error messages generated by the object storage process.

=item NOTES

A note on arrayprops-- this method automatically detects array props and saves 
them accordingly. Any element of the object hash where ref($element) eq 'ARRAY' will 
be stored as an arrayprop. 

Also, if there are any scalar elements of the hash you do not want stored as extraprops, 
they should be included in a hashref element called 'nosave'. nosave itself, cdate, mdate, 
hdate, and mhdate are automatically not saved as extraprops (dates are auto-generated, 
and 'hdates' are translations of system dates that are created at runtime). If your 
object includes any other data which is generated at fetch-time, you should add it to 
'nosave'. The format of this should be like:

	$obj->{'nosave'} =
		{
		'objid' => 1,
		'objclass' => 1,
		'msku' => 1,
		};

=back

=cut
###############################################################################
sub store_object {
	my ($site, $obj) = @_;
	
	my $id = $obj->{'objid'};
	return undef unless ($id);

	# Find out what our array props are
	my $aprops = [];
	foreach my $prop (keys %{$obj}) {
		if (ref($obj->{$prop}) eq 'ARRAY') {
			push @{$aprops}, {'propname' => $prop, 'fieldname' => $prop.'id'};
		}
	}
	
	# Find out if the box exists yet
	my $exists = &does_obj_exist($site, 'things', {'objid' => $obj->{'objid'}, 'objclass' => $obj->{'objclass'}});	

	undef my $err;
	if ($exists) {
		$err = &update_obj($site, $obj);
	} else {
		$err = &new_obj($site, $obj);
	}
	
	my $class = $obj->{'objclass'};
	
	undef $obj;
	
	$obj = &get_one_object($site, $id, $class, $aprops);
	
	return ($obj, $err);
}

###############################################################################
=pod

=head2 get_objids_by_property

get_objids_by_property - &get_objids_by_property($site, $properties, $order);

=over 4

=item DESCRIPTION

Finds a list of object id's based on "real" properties of that object (i.e. 
properties that are not stored in extraprops). In practice, this basically means 
it finds objid's by msku or date, since those are the only properties stored in 
'things' itself.

=item INPUT

$site       -- Standard site hashref

$properties -- Hashref expressing the conditions to search for. Looks like:

	my $find = {
	    1 => {'field' => 'cdate', 'value' => ">$start", 'join' => 'and'}, 
	    2 => {'field' => 'cdate', 'value' => "<$end", 'join' => 'and'},
	    3 => {'field' => 'objclass', 'value' => 'wmsbox', 'join' => 'and'}
	};

That is, it's a hashref of hashrefs. The keys don't matter much, Ordinal numbers 
work fine. The individual items must include 'field', 'value', and 'join', which 
get interpreted to build up the WHERE clause. 'value' accepts any of the special 
value characters that db_layer accepts. So, the above example will translate to: 

	SELECT objid FROM things WHERE 
	    cdate > '$start' AND
	    cdate < '$end'   AND
	    objclass = 'wmsbox'

$order -- If you need to order results in a particular way, this will but put in 
the ORDER clause of the query. To expand the example above, if I passed in an $order 
variable, the query would become:

	SELECT objid FROM things 
	WHERE 
	    cdate > '$start' AND
	    cdate < '$end'   AND
	    objclass = 'wmsbox'
	ORDER BY
	    $order

=item OUTPUT

$ids -- array reference of found object id's, ordered as directed, or undef.

=item NOTES

Once you have this list of id's you can then iterate through and fetch all the objects 
you need of the results.

=back

=cut
###############################################################################
sub get_objids_by_property {
	my ($site, $properties, $order) = @_;
	
	my @where;
	my $i = 0;
	foreach my $key (keys %{$properties}) {
		$where[$i] = &build_where_hash($properties->{$key}->{'field'}, $properties->{$key}->{'value'}, $properties->{$key}->{'join'});
		$i++;
	}
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'things',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'objid' => 'objid'
				}
			}
	};

	if ($order) {
		$select_q->{'TABLE'}->{'ORDER'} = $order;
	}

	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "get_objids_by_property: Query is $q\n";

	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "get_objids_by_property: Rows returned: $rows\n";
	return undef unless ($rows);
	
	my @return;
	foreach my $row (@{$results}) {
		push @return, $row->{'objid'};
	}
	return \@return;
}	

###############################################################################
=pod

=head2 get_objids_by_extraprop

get_objids_by_extraprop - &get_objids_by_extraprop($site, $fieldname, $value, $order);

=over 4

=item DESCRIPTION

Find object id's by extraprops. Gets a list of objects that match a set of 
extraprop criteria. This one only allows you to search by a single property. 
The best use is to determine what property should produce the smallest set of 
valid results, get those id's then fetch those objects, optionally throwing 
out any that don't match further criteria.

=item INPUT

$site      -- Standard site hashref

$fieldname -- The property's name

$value     -- The value required (uses db_layer notation)

$order     -- Optional ORDER BY criteria

=item OUTPUT

$ids -- arrayref of matching object id's.

=item NOTES

For most object searching you'll want to find objects based on one or more 
of their "custom" properties. This makes that easy by finding objects "backward" 
from property to main object. This does *NOT* work with arrayprops.

=back

=cut
###############################################################################
sub get_objids_by_extraprop {
	my ($site, $fieldname, $value, $order) = @_;
	
	my @where;
	$where[0] = &build_where_hash('fieldname', $fieldname, 'and');
	$where[1] = &build_where_hash('body', $value, 'and');
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'extraprop',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'objid' => 'objid'
				}
			}
	};
	
	if ($order) {
		$select_q->{'TABLE'}->{'ORDER'} = $order;
	}
	
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "get_objids_by_extraprop: Query is $q\n";

	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "get_objids_by_extraprop: Rows returned: $rows\n";
	return undef unless ($rows);
	
	my @return;
	foreach my $row (@{$results}) {
		push @return, $row->{'objid'};
	}
	return \@return;
}	
			
###############################################################################
=pod

=head2 dump_obj

dump_obj - &dump_obj($obj);

=over 4

=item DESCRIPTION

Stringifies an object, suitable for debugging purposes. If you need to see exactly 
what the guts on a WMS object look like, do something like:

	my $obj_dump = &dump_obj($obj);
	warn "Object $obj->{'objid'} looks like:\n$obj_dump\n";

This will put a dump of the object in your apache error log, suitable for 
inspection at your leisure.


=item INPUT

$object -- The full object to dump

=item OUTPUT

$obj_dump -- the textified object

=item NOTES



=back

=cut
###############################################################################
sub dump_obj {
	my $obj = shift;
	
	my $txt = qq|ID => $obj->{'objid'}\n|;
	
	foreach my $field (keys %{$obj}) {
		next if ($field eq 'objid');
		if (ref($obj->{$field}) eq 'ARRAY') {
			$txt .= &dump_array_prop($field, $obj->{$field});
		} else {
			$txt .= qq|\t$field => $obj->{$field}\n|;
		}
	}
	
	return $txt;
}

sub dump_array_prop {
	my ($name, $array) = @_;
	my $txt;
	
	my $num = $#{$array} + 1;
	$txt .= qq|\t$name => ($num)\n|;
		
	foreach my $element (@{$array}) {
		my $idtag = $name.'id';
		$txt .= qq|\t\t$idtag => $element->{$idtag}\n|;
		foreach my $field (sort keys %{$element}) {
			next if ($field eq $idtag || !$field);
			$txt .= qq|\t\t\t$field => $element->{$field}\n|;
		}
	}
	
	return $txt;
}


###############################################################################
=pod

=head1 Object Fetching subroutines

The following should not be used by outside code, but are documented here in 
case you need to edit this module. These are routines used by &get_one_object()

=cut
###############################################################################


###############################################################################
=pod

=head2 get_obj_extraprops

get_obj_extraprops - &get_obj_extraprops($site, $obj, $aprops);

=over 4

=item DESCRIPTION

Given an object hashrerf, fetch and populate it's scalar extraprops.

=item INPUT

$site   -- Standard site hashref

$obj    -- The object to populate

$aprops -- an array ref of fields that are array properties, and should I<not> 
be treated as properties in this sweep.

=item OUTPUT

$object -- The object you passed in, updated with it's scalar props.

=item NOTES

$aprops should hold database fieldnames, not array propnames. 
See &get_obj_array_props() below for more on the difference.

=back

=cut
###############################################################################
sub get_obj_extraprops {
	my ($site, $obj, $aprops) = @_;
	
	my @results = &fetch_extraprops($obj->{'objid'}, $site);
	
	my $not_these = {};
	foreach my $aprop (@{$aprops}) {
		$not_these->{$aprop->{'fieldname'}} = 1;
	}
	
	foreach my $row (@results) {
		next unless ($row->{'fieldname'});
		next if ($not_these->{$row->{'fieldname'}});
		
		$obj->{$row->{'fieldname'}} = $row->{'body'};
	}
	
	return $obj;
}

###############################################################################
=pod

=head2 get_obj_array_props

get_obj_array_props - &get_obj_array_props($site, $obj, $fieldname, $propname);

=over 4

=item DESCRIPTION

This method populates an object's array properties. Called repeatedly (once for 
each property) after &get_obj_extraprops() to finish collecting an object 
together.

=item INPUT

$site      -- Standard site hashref

$obj       -- The object to add an array prop to

$fieldname -- The fieldname of the array prop (name in DB)

$propname  -- The propname of the array prop (name in object)

=item OUTPUT

$obj -- The object, populated with the relevant arrayprop

=item NOTES

=over 4 

=item * Propname vs. Fieldname

Array props will have a "propname" which is what they are accessed by in the 
object. For example, a box object has an array property called "location", 
which stores all the locations the box has occupied (oldest to newest). In the 
database, each location is stored in an extraprop named "locationid". Thus, 
"locationid" is the fieldname. Make sure you have this distinction right in 
your code, or things go poorly.

=item * Empty array props

Some objects, new ones for example, may not have any data for some array 
properties. This is ok, and calling this method for each array property will 
populate the appropriate propname with a pointer to an empty array. So, if 
you create your objects correctly, array props will always be valid array 
references, even if they are blank. This is important.

=back

=back

=cut
###############################################################################
sub get_obj_array_props {
	my ($site, $obj, $fieldname, $propname) = @_;
	$obj->{$propname} = [];
	
	my @where;
	$where[0] = &build_where_hash('objid', $obj->{'objid'});
	$where[1] = &build_where_hash('fieldname', $fieldname);
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'extraprop',
			 'WHERE'  => \@where,
			 'ORDER'  => 'cdate DESC',
			 'FIELDS' => 
		 		{
				 'fieldname' => 'fieldname',
				 'body' => 'body',
				 'cdate' => 'cdate'
				}
			}
	};
	
	my $results = [];
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "get_obj_array_props: Query is $q\n";
	
	my $rows = &query($q, $site->{dbh}, $results);
	warn "get_obj_array_props: Found $rows $propname for $obj->{'objid'}\n";
	
	return $obj unless ($rows);
	
	my $i = 0;
	
	foreach my $rec (@{$results}) {
		next unless ($rec->{'fieldname'} && $rec->{'body'} && $rec->{'cdate'});
		my $hash = &get_obj_array_prop_data($site, $rec->{'body'});
		$hash->{'cdate'} = $rec->{'cdate'};
		$hash->{$fieldname} = $rec->{'body'};
		push @{$obj->{$propname}}, $hash;
	}
		
	return $obj;
}


###############################################################################
=pod

=head2 get_obj_array_prop_data

get_obj_array_prop_data - &get_obj_array_prop_data($site, $id, $fieldname);

=over 4

=item DESCRIPTION

Called from &get_obj_array_props() to populate an individual array prop element.

=item INPUT

$site      -- Standard site hashref

$id        -- ID of arrayprop object

$fieldname -- The fieldname of this arrayprop type

=item OUTPUT

$prop      -- The hashref containing the array prop item. Will be unshifted 
onto the list for this arrayprop.

=item NOTES

$fieldname should be the DB field name for this type of arrayprop (not the 
propname). See &get_obj_array_props() below for more on the difference.

=back

=cut
###############################################################################
sub get_obj_array_prop_data {
	my ($site, $id, $fieldname)	= @_;

	my @where;
	$where[0] = &build_where_hash('objid', $id);

	my $select_q = {
		'statement' => 'select',
		'TABLE' =>
			{
			 'NAME'   => 'extraprop',
			 'WHERE'  => \@where,
			 'FIELDS' =>
				{
				 'fieldname' => 'fieldname',
				 'body' => 'body'
				}
			}
	};

	my $results = [];
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "get_box_array_prop_data: Query is $q\n";

	my $rows = &query($q, $site->{dbh}, $results);
	warn "get_box_array_prop_data: Found $rows data items for $id\n";

	my $hash = {};
	$hash->{$fieldname} = $id;

	foreach my $line (@{$results}) {
		next unless ($line->{'fieldname'});
		$hash->{$line->{'fieldname'}} = $line->{'body'};
	}
	
	return $hash;
}


###############################################################################
=pod

=head1 Object Saving subroutines

The following should not be used by outside code, but are documented here in 
case you need to edit this module. These are routines used by &store_object()

=cut
###############################################################################


###############################################################################
=pod

=head2 new_obj

new_obj - &new_obj($site, $obj);

=over 4

=item DESCRIPTION

This method creates a new object in the database. 

=item INPUT

$site -- Standard site hashref

$obj  -- Object to create

=item OUTPUT

Returns 1 if there was an error. Otherwise, returns undef.

=item NOTES

Use &store_object() for external code, whether creating a new object or updating an 
existing one. That will determine which it is for you.

=back

=cut
###############################################################################
sub new_obj {
	my ($site, $obj) = @_;
	
	warn "Creating obj $obj->{'objid'}\n";

	my $insert_q = {
		'statement' => 'insert',
		'TABLE' => 
			{
			 'NAME' => 'things',
			 'FIELDS' =>
			 	{
				 'objid' => $obj->{'objid'},
				 'msku' => $obj->{'msku'},
				 'objclass' => $obj->{'objclass'}
				}
			}
	};
	
	my $q = &build_sql($insert_q, $site, $site->{dbh}, 0);
	warn "Query is: $q\n";
	
	my $rv = &query($q, $site->{dbh});
	
	unless ($rv) {
		return 1;
	}

	&store_obj_props($site, $obj);

	return;
}


###############################################################################
=pod

=head2 update_obj

update_obj - &update_obj($site, $obj);

=over 4

=item DESCRIPTION

Updates an existing object in the database.

=item INPUT

$site -- Standard site hashref

$obj  -- Object to store

=item OUTPUT

None.

=item NOTES

Use &store_object() for external code, whether creating a new object or updating 
an existing one. That will determine which it is for you.

=back

=cut
###############################################################################
sub update_obj {
	my ($site, $obj) = @_;
	
	warn "Updating obj $obj->{'objid'}\n";
	my @where;
	$where[0] = &build_where_hash('objid', $obj->{'objid'}, 'and');
	$where[1] = &build_where_hash('objclass', $obj->{'objclass'}, 'and');
	
	my $update_q = {
		'statement' => 'update',
		'TABLE' =>
			{
			 'NAME' => 'things',
			 'WHERE' => \@where,
			 'FIELDS' => 
			 	{
				 'msku' => $obj->{'msku'},
				}
			}
	};
	
	my $q = &build_sql($update_q, $site, $site->{dbh}, 0);
	warn "Updating object. Query is $q\n";
	
	my $rv = &query($q, $site->{dbh});

	&store_obj_props($site, $obj);

	return;
}

		
###############################################################################
=pod

=head2 store_obj_props

store_obj_props - &store_obj_props($site, $obj);

=over 4

=item DESCRIPTION

Stores properties for an object. Most properties are not "things record" parts 
of the database, so this does the majority of the work in storing an object's 
data.

=item INPUT

$site -- Standard site hashref

$obj  -- Object to store

=item OUTPUT

None.

=item NOTES

=over 4

=item * Nosave

This is the bit that respects the "nosave" field of an object, which tells it 
what not to consider an "extraprop". System-wide nosaves are:

	* mdate (stored automatically)
	* cdate (stored automatically)
	* hdate (created at load-time)
	* mhdate (created at load-time)
	* nosave (itself, obviously)

=item * New vs. Update

This routine automatically checks whether a property exists and needs updating, 
or is new and needs creating.

=item * Arrayprops

This routine also checks if a property is an arrayref, and stores it as an arrayprop 
if so.

=back 

=back

=cut
###############################################################################
sub store_obj_props {
	my ($site, $obj) = @_;
	
	my $nosave = $obj->{'nosave'};
	
	# System permanent nosaves
	$nosave->{'mdate'} = 1;
	$nosave->{'cdate'} = 1;
	$nosave->{'hdate'} = 1;
	$nosave->{'mhdate'} = 1;
	$nosave->{'nosave'} = 1;

	foreach my $prop (keys %{$obj}) {
		next if ($nosave->{$prop});
		
		if (ref($obj->{$prop}) eq 'ARRAY') {
			&update_array_prop($site, $obj, $prop, $prop.'id');
		} else {
			if (&does_obj_exist($site, 'extraprop', {'objid' => $obj->{'objid'}, 'fieldname' => $prop})) {
				&update_extraprop($obj->{'objid'}, $prop, $obj->{$prop}, $site);
			} else {
				&create_extraprop($obj->{'objid'}, $prop, $obj->{$prop}, $site);
			}
		}
	}	
	
	return;
}


###############################################################################
=pod

=head2 update_array_prop

update_array_prop - &update_array_prop($site, $obj, $propname, $fieldname);

=over 4

=item DESCRIPTION

Takes an array property and adds any items which have not yet been saved. By 
their nature, array props don't get updated, only added to.

=item INPUT

$site      -- standard site hashref

$obj       -- The object we're working on

$propname  -- The propname of the arrayprop to save

$fieldname -- The DB fieldname of this arrayprop

=item OUTPUT

None.

=item NOTES

Really ought to return something on failure.

=back

=cut
###############################################################################
sub update_array_prop {
	my ($site, $obj, $propname, $propidname) = @_;
	
	my $i = 0;
	my $found = &does_obj_exist($site, 'extraprop', 
		{'objid' => $obj->{'objid'}, 
		 'fieldname' => $propidname, 
		 'body' => $obj->{$propname}->[$i]->{$propidname}});

	while (!$found) {
		my $newest_elem = $obj->{$propname}->[$i];
		warn "ID of newest $propname element: $newest_elem->{$propidname}\n";

		return unless ($newest_elem->{$propidname});

		warn "Element $newest_elem->{$propidname} not yet saved\n";
		return if &does_obj_exist($site, 'extraprop', 
			{'objid' => $obj->{'objid'}, 'fieldname' => $propidname, 'body' => $newest_elem->{$propidname}});

		foreach my $line (keys %{$newest_elem}) {
			my $id = $newest_elem->{$propidname};
			if ($line eq $propidname) {
				$id = $obj->{'objid'};
			} 

			&create_extraprop($id, $line, $newest_elem->{$line}, $site);	
		}
		warn "Saved element $newest_elem->{$propidname}\n";
		$i++;
		$found = &does_obj_exist($site, 'extraprop', 
 				{'objid' => $obj->{'objid'}, 
				 'fieldname' => $propidname, 
				 'body' => $obj->{$propname}->[$i]->{$propidname}});
	}	

	return;
}










###############################################################################
# Utility search functions
###############################################################################


1;
