use strict;
###############################################################################
=pod

=head1 sku.pm

sku.pm -- This library implements SKUs for the WMS. 

=head1 DESCRIPTION

Contains methods for getting and storing SKU objects.

=head1 NOTES & BUGS

None yet

=cut
###############################################################################

###############################################################################
=pod

=head2 get_sku

get_sku - &get_sku($sku, $site);

=over 4

=item DESCRIPTION

Get a SKU object when all you know is the SKU you're looking for.

=item INPUT

$sku   -- The SKU top find

$site  -- Standard site hashref

=item OUTPUT

$sku_obj -- The complete SKU object, or undef if not found.

=item NOTES

None

=back

=cut
###############################################################################
sub get_sku {
	my ($sku, $site) = @_;
	
	my $props = {
		1 => {
			   'field' => 'msku',
			   'value' => $sku,
			   'join'  => 'and'
			  },
		2 => {
			   'field' => 'objclass',
			   'value' => 'wmssku',
			   'join'  => 'and'
			  }
	};

	my $skuids = &get_objids_by_property($site, $props, 0);
	return undef unless (ref($skuids) eq 'ARRAY');
	
	my $id = $skuids->[0];
	
	my $sku_obj = &get_one_sku($site, $id);
	
	return $sku_obj;
}

###############################################################################
=pod

=head2 get_one_sku

get_one_sku - &get_one_sku($site, $id);

=over 4

=item DESCRIPTION

Wrapper for &get_one_object() for SKU. Finds a SKU object by object id.

=item INPUT

$site -- Standard site hashref

$id   -- The objid of the desired SKU

=item OUTPUT

$sku_obj -- The SKU object if found, or undef otherwise.

=item NOTES

This only finds SKUs if you know the Object ID, which is not the same as the 
SKU itself. Use &get_sku() for that.

=back

=cut
###############################################################################
sub get_one_sku	{
	my ($site, $id) = @_;
	
	my $sku = &get_one_object($site, $id, 'wmssku');
	
	$sku->{'exp_date'} = &indate_to_outdate($sku->{'edate'});
	$sku->{'nosave'} = 	
			{
			'exp_date' => 1,
			'objid' => 1,
			'msku' => 1,
			'objclass' => 1,
			} if ($sku);
	
	return $sku;
}
	
###############################################################################
=pod

=head2 store_sku

store_sku - &store_sku($site, $sku);

=over 4

=item DESCRIPTION

Save a SKU object to the DB. Wrapper for &store_object()

=item INPUT

$site -- Standard site hashref

$sku  -- SKU object to save

=item OUTPUT

$sku -- The SKU object, as stored. 

$err -- Any errors that came from the operation

=item NOTES

You should *always* use the $sku object returned from the save, not the one 
you had before. See &store_object() for more info on that.

=back

=cut
###############################################################################
sub store_sku {
	my ($site, $sku) = @_;
	return undef unless ($sku->{'objid'} && ($sku->{'objclass'} eq 'wmssku'));
	
	my ($newsku, $err) = &store_object($site, $sku);
	return ($newsku, $err);
}
		

sub display_sku {
	my ($site, $skuid) = @_;
	
	my $sku = &get_one_sku($site, $skuid);
	
	return "Cannot find sku $skuid\n" unless ($sku->{'objid'});
	my $show = &localize_template('wm_cell_sku_display', $site->{'current_interface'},0,$site);
	return &permutebit($show, $sku);
}	
	
sub check_sku_input {

	my $form = shift;
	my $site = shift;
	my $rv = 1;
	my $failed = {};
	my $err;
	
	# First just check for presence of required fields
	my $req = {
		"sku"           => "SKU"          ,
		"vendor_id"     => "Vendor ID"    ,
		"manufacturer"  => "Manufacturer" ,
		"barcode"       => "Barcode"      ,
		"description"   => "Description"  ,
		"vstyle"        => "Vendor Style" ,
		"expiration"    => "Expiration"   };
	
	($rv, $failed) = &check_required_fields($form, $req);
	
	unless ($rv) {
		$err .= "Please fill in all required fields!\n";
	}
	
	# Check that the sku doesn't exist already
	my $sku_err;
	$rv = &check_sku($site, $form->{'sku'}) if ($form->{'sku'});
	$err .= "SKU already exists.\n" unless ($rv);
	
	$rv = &check_date($form->{'expiration'}, 1);
	$err .= "Expiration date is invalid\n" unless ($rv);
	
	return $err;
}

sub check_sku {
	my $site = shift;
	my $sku = shift;
	return unless ($sku);

	my $sku = &get_sku($sku, $site);
	
	return 0 if ($sku);
	return 1;
}

sub create_new_sku {
	my ($form, $site) = @_;
	
	# Make a new objid for this sku
	my $id = &make_id($site);
	return "Cannot make object ID" unless ($id);
	warn "New SKU Objid is $id\n";
	
	my $newsku = {
		'objid' => $id,
		'objclass' => 'wmssku',
		'msku' => $form->{'sku'},
		'mfg_style' => $form->{'vstyle'},
		'upc' => $form->{'barcode'},
		'edate' => $form->{'expiration'},
		'vendid' => $form->{'vendor_id'},
		'manufacturer' => $form->{'manufacturer'},
		'wmsdesc' => $form->{'description'}, 
		'master_carton' => $form->{'master_carton'},
		'nosave' => 	
			{
			'objid' => 1,
			'msku' => 1,
			'objclass' => 1,
			}
	};
	
	my ($sku, $err) = &store_sku($site, $newsku); 		
	
	if ($err) {
		$form->{'inp_err'} = $err;
		return $form;
	}
		
	my @fields = (
		"manufacturer",
		"vstyle",      
		"barcode",     
		"expiration",  
		"description", 
		"sku",         
		"vendor_id",
		"master_carton"
	);
	$form = &wipe_fields($form, @fields);
	
	$form->{'inp_err'} = "Created SKU $sku->{'msku'}\n";
	$form->{'sku_show'} = &display_sku($site, $sku->{'objid'});
	return $form;
	
}
		


1;
