use strict;

###############################################################################
=pod

=head1 wms_funcs.pm

wms_funcs.pm -- Library of standard routines for use in the Zelerate WMS. 
Functions that do things that may be useful for many different WMS scripts.

=head1 DESCRIPTION

This is a library of general purpose WMS-related functions. Included are methods
for determining if a user is logged in as a qualified admin, initializing 
scripts, and so forth. Any methods that can be useful across different parts of 
the system (and should be done in a standard way!) ought to be in here.

=head1 NOTES & BUGS

None yet

=cut
###############################################################################

###############################################################################
=pod

=head2 init_script

init_script - &init_script($scriptname, $relpath);

=over 4

=item DESCRIPTION

Once your script has determined it's name and path, you should call &init_script 
with those two parameters to do the rest of the standard init stuff. Sets $site, 
$form, $dbh, and finds the language and interface to use.

=item INPUT

$scripname - (Scalar) The name of the caller script.

$relpath   - (Scalar) Relative path to store root

=item OUTPUT

$site      - Hashref from &setsite()

$form      - Hashref of form input

$dbh       - Connected database handle

=item NOTES

$site also gets the following params:

$site->{scriptname} - The scriptname you passed in

$site->{relpath} - The relpath you passed in

$site->{om_render_lang} - Current render language

$site->{current_interface} - current output interface

=back

=cut
###############################################################################
sub init_script {
	my ($scriptname, $relpath) = @_;
	
	## Get the site hash
	#
	my $site = &setsite();

	## get the FORM bits
	#
	my $form = &ReadParse();

	## establish connection to database  
	#
	my $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}) 
		|| die "Can't connect to database! ".DBI->errstr();

	## Add dbh to $site, for convenience
	$site->{dbh} = $dbh;

	## Get the language and interface, if possible
	#
	my $h = {};
	if ($form->{'sid'}) {
	   $site = &get_lang_interface($form->{'sid'}, $site->{dbh}, $site);
	} else {
		$site->{'om_render_lang'} = $site->{'om_locale_lang_def'} || 'eng';
		$site->{'current_interface'} = "admin";
	}	

	# Add scriptname and relpath to site for convenience
	$site->{scriptname} = $scriptname;
	$site->{relpath} = $relpath;
	
	## Get the script error messages hash
	#
	#my %script_msg = &message($site->{'om_render_lang'}, $scriptname, $relpath);          # user interaction messages

	# Print a logout cookie if there's a logout param
	if ($form->{'logout'}) {
		#Clear sid ENV
		my $td;
		$td = $ENV{'SERVER_NAME'};
	    $td = $ENV{'SERVER_ADDR'} unless ( $td );
		print "Set-Cookie: sid=; path=/; domain=$td\n";
		$site->{sid} = '';
		$form->{sid} = '';
		$ENV{'HTTP_COOKIE'} = '';
	}
	
	return ($site, $form, $dbh);
}

###############################################################################
=pod

=head2 get_lang_interface

get_lang_interface - &get_lang_interface($sid, $dbh, $site);

=over 4

=item DESCRIPTION

Gets the current language and interface desired by the user (or site 
defaults if there's no user).

=item INPUT

$sid  -- user's session id (decrypted)

$dbh  -- A database handle

$site -- site hashref from setsite()

=item OUTPUT

$site -- site hashref, with added items: $site->{om_render_lang} 
and $site->{current_interface}

=item NOTES

This is called by &initscript() anyway, so you don't need to worry 
about it much. It populates the fields:

$site->{om_render_lang}    -- language code

$site->{current_interface} -- interface to use

=back

=cut
###############################################################################

sub get_lang_interface {
	my $sid  = shift;
	my $dbh  = shift;
	my $site = shift;
	
	my @where;
	$where[0] = &build_where_hash("sid", $sid, "and");
	my $q_hash = {
		'statement' => 'select',
		'TABLE' => 
		 {
		     'NAME'   => 'sessions',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			  {
			 	'language'  => 'language',
				'interface' => 'interface'
			  }
		 }
	};
	
	my $q = &build_sql($q_hash, $site, $dbh, 1);
	my @results;
	&query($q, $dbh, \@results);
	
	my $h = $results[0] || {};
	
	$site->{'om_render_lang'} = $h->{'language'} || $site->{'om_locale_lang_def'} || 'eng';
	$site->{'current_interface'} = $h->{'interface'} || "admin";
	return $site;
}

###############################################################################
=pod

=head2 load_standard_libs

load_standard_libs - &load_standard_libs($relpath);

=over 4

=item DESCRIPTION

Calls 'require' on libraries that all WMS scripts will likely need to load. This 
is all done in one place because sometimes libraries change names, or functions 
change files, and it's much easier to update this one routine than every CGI. 

Best called during script init, right after 'use DBI' and before loading any 
other libs that that script might need.

=item INPUT

$relpath -- The path where libraries live.

=item OUTPUT

1 -- simply returns true after loading a set of standard libs. See below.

=item NOTES

Currently, this method loads the following libraries:

	lib/cgi-lib.pm
	lib/formtools.pm
	lib/filetools.pm
	conf/site.pm
	lib/state.pm
	lib/localization.pm
	lib/conversions.pm
	lib/blowfisher.pm
	lib/permission_layer.pm
	lib/db_layer.pm
	lib/wm_object.pm
	lib/wm_boxes.pm
	lib/wm_sku.pm
	lib/wm_po.pm
	lib/wm_receiving.pm
	lib/wm_bin.pm

=back

=cut
###############################################################################

sub load_standard_libs {
	my $relpath = shift;
	
	# Load the libs that all scripts are likely to need
	require "$relpath/lib/cgi-lib.pm";
	require "$relpath/lib/formtools.pm";
	require "$relpath/lib/filetools.pm";
	require "$relpath/conf/site.pm";
	require "$relpath/lib/state.pm";
	require "$relpath/lib/localization.pm";
	require "$relpath/lib/conversions.pm";
	require "$relpath/lib/blowfisher.pm";
	require "$relpath/lib/permission_layer.pm";
	require "$relpath/lib/db_layer.pm";
	require "$relpath/lib/wm_object.pm";
	require "$relpath/lib/wm_boxes.pm";
	require "$relpath/lib/wm_sku.pm";
	require "$relpath/lib/wm_po.pm";
	require "$relpath/lib/wm_receiving.pm";
	require "$relpath/lib/wm_bin.pm";

	return 1;
}
	
###############################################################################
=pod

=head2 set_nav

set_nav -- &set_nav($form, $title, $home_url);

=over 4

=item DESCRIPTION

Sets a couple $form elements related to the top-of-page nav links. Recommended 
for clarity, and also because it's easy to change if the design changes.

=item INPUT

$form      - Standard $form hashref

$title     - (Scalar) Page subtitle (for browser title bar)

$home_url  - (Scalar) URL to link "Home" to in nav bar

=item OUTPUT

$form      - $form hash, with added items

=item NOTES

Items added to $form:

$form->{'home_url'} - The home_url you passed in.

$form->{'title'}    - The page subtitle you passed in.

=back

=cut
###############################################################################
sub set_nav {
	my $form = shift;
	my $site = shift;
	my $title = shift;
	my $home_url = shift;
	
	$form->{'title'} = $title;
	$form->{'home_url'} = $home_url;
	
	# See if we're logged in
	my $id = &check_for_login($form, $site);
	
	# If so, make sure the logout part is set.
	if ($id) {
		my $logout = &localize_template("wm_cell_wms_logout",$site->{'current_interface'},0,$site);
		$form->{'logout_button'} = &permutebit($logout, {'name' => $site->{'scriptname'}});
	}
	
	#Set nav cell color
	$form->{'bgcolor_po'} 			= '#455d83';
	$form->{'bgcolor_receiving'} 	= '#455d83';
	$form->{'bgcolor_pf_pick'} 		= '#455d83';
	$form->{'bgcolor_packship'} 	= '#455d83';
	$form->{'bgcolor_probleminv'} 	= '#455d83';
	$form->{'bgcolor_stats'} 		= '#455d83';
	$form->{'bgcolor_custserv'} 	= '#455d83';
	$form->{'bgcolor_orders'} 		= '#455d83';
	$form->{'bgcolor_pullback'} 	= '#455d83';
	$form->{'bgcolor_asn'} 			= '#455d83';
	
	my $me = $site->{'scriptname'};
	$me =~ /(.*?).cgi/;
	$me = $1;
	
	$form->{"bgcolor_".$me} = '#e86b10';
	
	return $form;
}


###############################################################################
=pod

=head2 check_required_fields

check_required_fields -- &check_required_fields($form, $required);

=over 4

=item DESCRIPTION

Method to check form input for required fields. Takes in the form data hash, 
and a hashref of things that must be non-null in the input. Returns a t/f boolean, 
and a hashref consisting of the fields that were not found (out of the "required" 
hashref you passed in to begin with)

=item INPUT

$form      - Standard $form hashref

$required  - (Hashref) Required fields (keys), and free-form descriptors (vals)

=item OUTPUT

$rv        - (Bool) True if all fields were found, False if not

$fail      - (hashref) The portion of your $required hashref that were not found. 
Useful for error checking and such.

=item NOTES

All this does is compare keys found in $form to keys you require. It returns a 
true/false, to say if all keys were found or not, and a hashref of the ones that 
were not found.

=back

=cut
###############################################################################

sub check_required_fields {
	my $form = shift;
	my $reqd = shift;
	my $fail = {};
	my $pass = 1;
	
	foreach my $field (keys %{$reqd}) {
		warn "Checking field: $field\n";
		unless ((defined $form->{$field}) && ($form->{$field} ne '')) {
			warn "Not found!\n";
			$fail->{$field} = $reqd->{$field};
			$pass = 0;
		}
	}

	return ($pass, $fail);
}

###############################################################################
=pod

=head2 check_date

check_date - &check_date($date, $future);

=over 4

=item DESCRIPTION

Checks a date string to make sure it is in valid MMDDYYYY format. Optionally, 
checks to be sure that the date is in the future.

=item INPUT

$date   -- Scalar: string to be checked

$future -- Bool: Should this date be in the future? (tru/false)

=item OUTPUT

1/0 -- Boolean to indicate if the date is valid or not

=item NOTES

Dates in WMS input must be of the form MMDDYYYY. This method should always be 
called on date input fields before accepting them as input.

Right now, this doesn't return an informative message (i.e. "Date must 
be MMDDYYYY" or "Date must be in the future"). This should be fixed.

=back

=cut
###############################################################################

sub check_date {
	my $date = shift;
	my $future = shift;
	
	return 0 unless ($date =~ /^\d{8}$/);
	
	$date =~ /^(\d\d)(\d\d)(\d\d\d\d)$/;
	my $month = $1;
	my $day = $2;
	my $year = $3;
	
	my @now = localtime(time());
	$now[5] += 1900;
	$now[4] += 1;
	
	return 0 unless (
		($month > 0 && $month <= 12) &&
		($day > 0 && $day <= 31) &&
		($year > 0)
	);
	
	# Passed unless it's required to be in the future
	return 1 unless ($future);
	
	# Passed if year > current year
	return 1 if ($year > $now[5]);
	# Passed if year eq current year, and month > current month
	return 1 if ($year == $now[5] && $month > $now[4]);
	# Passed if year eq this year, and month eq this month, and day > today
	return 1 if ($year == $now[5] && $month == $now[4] && $day > $now[3]);
	
	# Otherwise, this is not a future date.
	return 0;
}

###############################################################################
=pod

=head2 wipe_fields

wipe_fields - &wipe_fields($form, @fields);

=over 4

=item DESCRIPTION

Handy method to clear data out of the $form hashref. Takes that, and an array 
of fieldnames to clear, and returns $form with those fields cleared.

=item INPUT

$form   -- Standard form data hashref

@fields -- Array of fieldnames to clear

=item OUTPUT

$form -- The form hashref, minus any of the "clear me" fields found.

=item NOTES

None

=back

=cut
###############################################################################

sub wipe_fields {
	my $form = shift;
	my @fields = @_;
	foreach my $field (@fields) {
		$form->{$field} = '';
	}
	return $form;
}

###############################################################################
=pod

=head2 check_for_login

check_for_login - &check_for_login($form, $site);

=over 4

=item DESCRIPTION

Checks a request to see if the user is trying to log in. If so, then 
logs in the session, otherwise tries to find out if the user is already 
logged in, and if so, gets the custid. Always returns custid if there is one, 
or undef if not.

=item INPUT

$form -- Form data input hashref

$site -- Standard site hashref

=item OUTPUT

$id   -- The custid of the user, if the session is logged-in.

=item NOTES

This is the interface for several of the following methods. Script authors 
should simply use this whenever they need to check if a user is logged in. It will 
take care of the details of actually logging in for you, or find the existing 
customer ID (or admin ID). Less stress, less hassle. Simply use it like:

	my $id = &check_for_login($form, $site);

	if ($id) {
		...do logged-in user stuff...
	} else {
		...don't...
	}

=back

=cut
###############################################################################

sub check_for_login {
	my ($form, $site) = @_;
	
	# First make sure we're not supposed to be logging out!
	if ($form->{'logout'}) {
		return '';
	}
	
	# See if we have a login
	my $admin_id = &check_wms_login($form, $site);

	# Tag session with id if we had a valid login
	if ($admin_id) {
		&login_session($site, $admin_id);
	} else {
		# Else, see if we already have a logged in session
		$admin_id = &is_session_logged_in($site);
	}
	
	return $admin_id;
}


###############################################################################
=pod

=head2 check_wms_login

check_wms_login - &check_wms_login($form, $site);

=over 4

=item DESCRIPTION

The method that actually tries to determine if a login is valid. 

=item INPUT

$form -- standard form hashref
$site -- standard site hashref

=item OUTPUT

0/1 -- True or false, depending on if the login succeeded. Note that it 
returns false for any unsuccessful call, including if there was no attempt 
to login.

=item NOTES

This should probably not be used in scripts, but is called as part of 
&check_for_login() instead.

=back

=cut
###############################################################################
sub check_wms_login {
	my $form = shift;
	my $site = shift;
	
	# Just bail if no account data found
	return 0 unless ($form->{'account'} && $form->{'pass'});

	# Else, check the contacts table
	my @where;
	$where[0] = &build_where_hash('uname', $form->{'account'}, "and");
	$where[1] = &build_where_hash('passwd', $form->{'pass'}, "and");
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{ 
			 'NAME'   => 'contacts',
			 'FIELDS' => {
			              'objid'   => 'objid',
						  'objtype' => 'objtype'
						 },
			 'WHERE'  => \@where
			}
	};
	
	my $q = &build_sql(	$select_q, $site, $site->{dbh}, 1);	 
	my $results = [];
	
	my $rows = &query($q, $site->{dbh}, $results);
	
	# If no rows, or more than one row, came back, fail
	return 0 unless ($rows == 1);
	
	# If the objtype is wrong, fail
	return 0 unless ($results->[0]->{'objtype'} eq '39004-A');
	
	# Otherwise, we have a valid login!
	return $results->[0]->{'objid'};
}

###############################################################################
=pod

=head2 login_session

login_session - &login_session();

=over 4

=item DESCRIPTION

Updates the session table with a user's objid if they are logging in. This is the 
bit that actually lets us know if a session is logged in or not.

=item INPUT

$site   -- standard site hashref

$objid  -- scalar: objid of user to log in

=item OUTPUT

nothing much

=item NOTES

This is also hidden behind the &check_for_login() interface, and shouldn't be used.

=back

=cut
###############################################################################
sub login_session {
	my $site = shift;
	my $objid = shift;
	my $sid = $site->{'sid'};
	
	my $now = time();

	my @where;
	$where[0] = &build_where_hash('objid', $objid, "and");
	
	my $update_q = {
		'statement' => 'UPDATE',
		'TABLE' =>
			{
			 'NAME'   => 'contacts',
			 'FIELDS' => { 'mdate' => $now },
			 'WHERE'  => \@where
			}
	};
	my $q = &build_sql(	$update_q, $site, $site->{dbh}, 1);	 
	my $rv = &query($q, $site->{dbh});

	$where[0] = &build_where_hash('sid', $sid, "and");
	$update_q = {
		'statement' => 'UPDATE',
		'TABLE' =>
			{
			 'NAME'   => 'sessions',
			 'FIELDS' => { 'custid' => $objid },
			 'WHERE'  => \@where
			}
	};
	$q = &build_sql($update_q, $site, $site->{dbh}, 1);	 
	$rv = &query($q, $site->{dbh});
	
	return;
}


sub is_session_logged_in {
	my $site = shift;
	my $admin_objtype = shift || '39004-A';
	my $sid  = $site->{'sid'};
	
	# First see if we have a custid in this session.
	# If not, we have not been logged in
	my $custid = &has_custid($site);
	return 0 unless ($custid);

	# Ok, we have a custid. Now see if it belongs to
	# an administrative object type
	my $admin = &has_admin_custid($site, $custid, $admin_objtype);

	# The value of admin will be either the admin objid, or null
	return $admin;
}

sub has_custid {
	my $site = shift;
	
	my @where;
	$where[0] = &build_where_hash('sid', $site->{'sid'}, "and");
	my $select_q = {
		'statement' => 'SELECT',
		'TABLE' =>
			{
			 'NAME'   => 'sessions',
			 'FIELDS' => { 'custid' => 'custid' },
			 'WHERE'  => \@where
			}
	};
	
	my $results = [];
	my $q = &build_sql($select_q, $site, $site->{dbh}, 1);	 
	
	my $rows = &query($q, $site->{dbh}, $results);
	
	# If no rows, or more than one row, came back, fail
	return 0 unless ($rows == 1 && $results->[0]->{custid});
	return $results->[0]->{custid};
}

sub has_admin_custid {
	my ($site, $id, $admin_objtype) = @_;
	
	my @where;
	$where[0] = &build_where_hash('objid', $id, "and");
	
	my $select_q = {
		'statement' => 'SELECT',
		'TABLE' =>
			{
			 'NAME'   => 'contacts',
			 'FIELDS' => { 'objtype' => 'objtype' },
			 'WHERE'  => \@where
			}
	};

	my $results = [];
	my $q = &build_sql($select_q, $site, $site->{dbh}, 1);	 
	
	my $rows = &query($q, $site->{dbh}, $results);

	return 0 unless ($rows == 1 && ($results->[0]->{'objtype'} eq $admin_objtype));
	return $id;
}
	
###############################################################################
=pod

=head2 page_out

page_out - &page_out($form, $site, $current_page, $nav);

=over 4

=item DESCRIPTION

The last function that any script should call. This takes the page data and creates 
the output page. Then it prints it out to the client, disconnects the DB handle, 
and exits. 

=item INPUT

$form          -- Standard Form hashref

$site          -- Standard site hashref

$current_page  -- The name of the main output template to use

$nav           -- The depth navigation string, scalar, build as needed.  

=item OUTPUT

Prints out response page to client and exits.

=item NOTES

Do not call this until you are completely finished with script operations. This 
function ends the request by calling 'exit', so nothing after this function will 
work.

=back

=cut
###############################################################################

sub page_out {
	my ($form, $site, $current_page, $nav) = @_;
	
	# Initialize titles and stuff
	$form = &set_nav($form, $site, $site->{'subtitle'}, $site->{'home'}, $site->{'title'});
	$form->{'nav_links'} = $nav;

	$form->{'wm_cell_wms_nav'} = &localize_template("wm_cell_wms_nav",$site->{'current_interface'},0,$site);
	$form->{'wm_cell_wms_nav'} = &permutebit($form->{'wm_cell_wms_nav'}, $form );
	
	# Now permute pages
	my $thispage = &localize_template($current_page,$site->{'current_interface'},0,$site);
	$thispage 	 = &permutebit( $thispage, $form );
	$thispage 	 = &permutebit( $thispage, $site );
	$thispage 	 = &permutebit( $thispage, \%ENV );
	$thispage 	 = &permuteclean ( $thispage  );
	$thispage 	 = &permute_include( $thispage, "html", 0, $site );
	
	# Print output
	print "content-type: text/html\n\n";
	print $thispage;
	
	# Disconnect dbh
	$site->{dbh}->disconnect if ( $site->{dbh} );
	
	# And leave
	exit;
}

###############################################################################
=pod

=head2 create_extraprop

create_extraprop - &create_extraprop($objid, $fieldname, $value, $site);

=over 4

=item DESCRIPTION

Utility function to create an extraprop.

=item INPUT

$objid     -- The object id to insert

$fieldname -- The name of the extraprop to create

$value     -- The value to put in "body"

$site      -- Standard site hashref

=item OUTPUT

$rv -- Return value of the "INSERT" statement. Check return value to see if the create worked.

=item NOTES

This does not check to see if an extraprop matching the calling parameters already 
exists. Because of the design of the table, duplicate extraprops can be created. 
Your code should know beforehand if it should be creating or updating.

=back

=cut
###############################################################################

sub create_extraprop {
	my ($objid, $fieldname, $value, $site) = @_;
	return 0 unless ($objid && $fieldname);
	
	warn "Creating extraprop. Field: $fieldname, Value: $value, Object: $objid\n";

	my $insert_q = {
		'statement' => 'insert',
		'TABLE' =>
			{
			 'NAME' => 'extraprop',
			 'FIELDS' => 
			 	{
				 'objid' => $objid,
				 'fieldname' => $fieldname,
				 'body' => $value
				}
			}
	};
	my $q = &build_sql($insert_q, $site, $site->{dbh}, 0);
	my $rv = &query($q, $site->{dbh});
	return $rv;
}


###############################################################################
=pod

=head2 update_extraprop

update_extraprop - &update_extraprop($objid, $fieldname, $value, $site);

=over 4

=item DESCRIPTION

Utility function to update an existing extraprop.

=item INPUT

$objid     -- The object id to update

$fieldname -- The name of the extraprop to update

$value     -- The value to put in "body"

$site      -- Standard site hashref


=item OUTPUT

$rv -- Return value of the "UPDATE" statement. Check return value to see if the update worked.

=item NOTES

Will fail if the extraprop doesn't already exist. Check return value to ensure success.

=back

=cut
###############################################################################

sub update_extraprop {
	my ($objid, $fieldname, $value, $site) = @_;
	return 0 unless ($objid && $fieldname);

	warn "Updating extraprop. Field: $fieldname, Value: $value, Object: $objid\n";
	
	my @where;
	$where[0] = &build_where_hash('objid', $objid, 'and');
	$where[1] = &build_where_hash('fieldname', $fieldname, 'and');
	
	my $update_q = 	{
		'statement' => 'update',
		'TABLE' =>
			{
			 'NAME' => 'extraprop',
			 'WHERE' => \@where,
			 'FIELDS' => 
			 	{
				 'body' => $value
				}
			}
	};

	my $q = &build_sql($update_q, $site, $site->{dbh}, 0);
	my $rv = &query($q, $site->{dbh});
	return $rv;
}


###############################################################################
=pod

=head2 does_obj_exist

does_obj_exist - &does_obj_exist($site, $table, $where);

=over 4

=item DESCRIPTION

Utility function to check for the existence of anything that has an objid and can be described by a WHERE clause

=item INPUT

$site  -- Standard site hashref

$table -- Scalar holding table name to look in (e.g. "things", "extraprop", etc)

$where -- Hashref describing WHERE clause. Format is as follows:

	
	$where = {
	    'fieldname1' => 'value',
	    'fieldname2' => 'value'
	};

The values above follow all the rules defined by the DB layer. See db_layer.pm for that.

=item OUTPUT

$objid -- If the object exists, returns objid. Otherwise, returns undef.

=item NOTES

To make this more clear, here's an example of how to use. Say I wanted to find a "thing" 
with objid "100-AA". Here's what I'd do:

	my $where = { 'objid' => '100-AA' };
	if (&does_obj_exist($site, 'things', $where)) {
	    ...do stuff...
	}

=back

=cut
###############################################################################

sub does_obj_exist {
	my ($site, $table, $where) = @_;
	
	my @where;
	my $i = 0;
	foreach my $field (keys %{$where}) {
		$where[$i] = &build_where_hash($field, $where->{$field}, 'and');
		$i++;
	}
		
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => $table,
			 'WHERE'  => \@where,
			 'FIELDS' => 
		 		{
				 'objid' => 'objid'
				}
			}
	};

	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "does_obj_exist: Found $rows for $results->[0]->{'objid'}\n";
	
	return undef unless ($results->[0]->{'objid'});
	return $results->[0]->{'objid'};
}


sub fetch_extraprops {
	my ($id, $site) = @_;
	
	my @where;
	$where[0] = &build_where_hash('objid', $id, 'and');
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'extraprop',
			 'WHERE'  => \@where,
			 'FIELDS' => 
		 		{
				 'fieldname' => 'fieldname',
				 'body' => 'body'
				}
			}
	};
	
	my $results = [];
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "fetch_extraprops: Query is $q\n";
	
	my $rows = &query($q, $site->{dbh}, $results);
	warn "fetch_extraprops: Found $rows extraprops for $id\n";
	
	my @return;
	foreach my $row (@{$results}) {
		next unless (defined $row->{'fieldname'} && $row->{'fieldname'} ne '');
		warn "Adding extraprop $row->{'fieldname'} => $row->{'body'}\n";
		push @return, $row;
	}
	
	return @return;
}


###############################################################################
=pod

=head2 check_price

check_price - &check_price($price);

=over 4

=item DESCRIPTION

Check to see if a price field is in x.xx format.

=item INPUT

$price -- string containing the price to check

=item OUTPUT

t/f -- Returns true is the price is valid, false if not.

=item NOTES

This only checks that a string =~ /^\d+\.\d\d$/ 

=back

=cut
###############################################################################

sub check_price {
	my $price = shift;
	
	return 0 unless $price =~ /^\d+\.\d\d$/;
	return 1;
}

###############################################################################
=pod

=head2 cdate_to_hdate

cdate_to_hdate - &cdate_to_hdate($date);

=over 4

=item DESCRIPTION

Takes a "cdate"-type value and turns it into a human readable formatted date

=item INPUT

$cdate -- a date string from fields in the DB like cdate or mdate

=item OUTPUT

$hdate -- the same date, formatted like: "MM-DD-YYYY HH:MM:SS"

=item NOTES



=back

=cut
###############################################################################

sub cdate_to_hdate {
	my $date = shift;
	
	my @date = localtime($date);
	my $day = $date[3];
	my $mon = $date[4] + 1;
	my $yr  = $date[5] + 1900;
	my $hr  = sprintf("%02d", $date[2]);
	my $min = sprintf("%02d", $date[1]);
	my $sec = sprintf("%02d", $date[0]);
	return "$mon-$day-$yr $hr:$min:$sec";
}

###############################################################################
=pod

=head2 indate_to_outdate

indate_to_outdate - &indate_to_outdate($date);

=over 4

=item DESCRIPTION

Takes a date in MMDDYYYY format and converts it to a printable MM-DD-YYYY format

=item INPUT

$date -- should be 8 digits long

=item OUTPUT

$date -- same 8 digits, with dashes like: XX-XX-XXXX

=item NOTES

Doesn't check input yet to ensure validity

=back

=cut
###############################################################################

sub indate_to_outdate {
	my $date = shift;
	
	$date =~ s/(..)(..)(....)/$1-$2-$3/;
	return $date;
}

###############################################################################
=pod

=head2 make_id

make_id - &make_id($site);

=over 4

=item DESCRIPTION

Creates a new WMS object ID. Use this instead of the db_layer &get_new_objid()

=item INPUT

$site -- standard site hashref

=item OUTPUT

$id -- new objectid

=item NOTES

Because object ID's are created with reference to time, if youi attempt to create 
a lot of them quickly, they will frequently return the same value. This function 
gets around that by concatenating 5 random digits to the end of the objectid. So, 
where &get_new_objid() returns '100-AA', &make_id() would return something like 
'100-AA25346'. This ensures that objectids will remain unique.

=back

=cut
###############################################################################

sub make_id {
	my ($site) = @_;
	my $id = &get_new_objid($site, $site->{dbh}, 'wms objid');
	my $rand = rand;
	$rand =~ s/.*(.{5})$/$1/;
	$id .= $rand;
	warn "Making new wms object id: $id\n";
	return $id;
}


			
1;
