#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# xml.pm

use strict;

##############################################################################
=pod

=head1 xml.pm

xml.pm - subroutines for XML data exports

=head1 DESCRIPTION

This library is used for all subroutines that are used to develop an XML exporter.
An order xml exporter is provided with the product installation.

=head1 NOTES & BUGS

The calling routines must require 
    xml.pm
    db_layer.pm
    permission_layer.pm
    variants.pm
    localization.pm
    conversions.pm
    

=cut

##############################################################################
#
# Version 2 TG 20000921
#    Changed Error: to Missing template
# Version 1 TG 20000919
#    Wrote for version 1.1 standards
#

##############################################################################
=pod

=head2 getXMLData

getXMLData - &getXMLData($order_objid, $dbh, $site);

=over 4

=item DESCRIPTION

Given an order object ID, find all information for it, including the information
from the relational tables.  Return all data in a hashref.

=item INPUT

$order_objid - Object ID of the order to extract the data for

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$xml_hashref - Hashref of information containing the following information, each of which
    are also hashrefs:
    ->{'ORDER'} = all information from the order table where the objid = $order_objid
    ->{'SHIPTO'} = all information from the addresses table where the objid = ->{'ORDER'}->{'shipto'}
    ->{'BILLTO'} = all information from the addresses table where the objid = ->{'ORDER'}->{'billto'}
    ->{'OCART'} = all line items in the ocart where orderid = $order_objid
    ->{'CONTACTS'} = User name and objid from contacts where objid = ->{'ORDER'}->{'custid'}

=item NOTES

=back

=cut
###############################################################################
sub getXMLData {
    my $order_objid = shift;
    my $dbh = shift;
    my $site = shift;

    my ($xml_hashref, $num_items_ordered, @ocart_results, $item_info, $i);
    my ($object_hashref, $key);

    $xml_hashref->{$order_objid}->{'ORDER'} = &getOrderData($order_objid, $dbh, $site);
    $xml_hashref->{$order_objid}->{'SHIPTO'} = &getAddressData($xml_hashref->{$order_objid}->{'ORDER'}->{'shipto'}, $dbh, $site);
    $xml_hashref->{$order_objid}->{'BILLTO'} = &getAddressData($xml_hashref->{$order_objid}->{'ORDER'}->{'billto'}, $dbh, $site);
    $num_items_ordered = &getOcartData($order_objid, $dbh, $site, \@ocart_results);
    for ($i = 0; $i < $num_items_ordered; $i++) {
	$item_info = {};
	$item_info = $ocart_results[$i];
	# Can replace with just a select of title when variants are stored!
	$object_hashref =  &get_fields_and_title(undef, $item_info->{'lineobjid'}, $dbh, undef, $site);
	$xml_hashref->{$order_objid}->{'OCART'}->{$item_info->{'objid'}}->{'title'} = $object_hashref->{'title'};
	foreach $key (keys %{$item_info}) {
	    $xml_hashref->{$order_objid}->{'OCART'}->{$item_info->{'objid'}}->{$key} = $item_info->{$key};
	}
    }
    $xml_hashref->{$order_objid}->{'CONTACTS'} = &getContactsData($xml_hashref->{$order_objid}->{'ORDER'}->{'custid'}, $dbh, $site);
    return $xml_hashref;
}


##############################################################################
=pod

=head2 getOrderData

getOrderData - &getOrderData($orderid, $dbh, $site);

=over 4

=item DESCRIPTION

To retrieve all information from the orders table where the objid is the same
as the $order object ID passed in.  Return the results in a hashref or undef if
nothing found.

=item INPUT

$order - order object id

$dbh - current active database handle

$site - site hashref

=item OUTPUT

$sql_results[0] - Hashref containing the fields from the order select, or
    undef if nothing returned

=item NOTES

=back

=cut
###############################################################################
sub getOrderData {
    my $order = shift;
    my $dbh = shift;
    my $site = shift;

    my ($select_hashref, @sql_results, $q, $num);

    $select_hashref->{'TABLE'}->{'NAME'} = "orders";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $order, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $num = &query_getAll($q, $site, $dbh, \@sql_results);
    if ( $num == 1 ) {      # one order found...
	return ($sql_results[0]);
    }

    return undef;
}

##############################################################################
=pod

=head2 getContactsData

getContactsData - &getContactsData($contactsid, $dbh, $site);

=over 4

=item DESCRIPTION

To retrieve all information from the contacts table where the objid is the same
as the $custid object ID passed in.  Return the results in a hashref or undef if
nothing found.

=item INPUT

$custid - object id for the contacts

$dbh - current active database handle

$site - site hashref

=item OUTPUT

$sql_results[0] - Hashref containing the fields from the contacts select, or
    undef if nothing returned

=item NOTES

=back

=cut
###############################################################################
sub getContactsData {
    my $custid = shift;
    my $dbh = shift;
    my $site = shift;

    my ($select_hashref, @sql_results, $q, $num);

    $select_hashref->{'TABLE'}->{'NAME'} = "contacts";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'uname'} = "uname";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $custid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $num = &query_getAll($q, $site, $dbh, \@sql_results);
    if ( $num == 1 ) {      # one order found...
	return ($sql_results[0]);
    }
    return undef;
}

##############################################################################
=pod

=head2 getAddressData

getAddressData - &getAddressData($objid, $dbh, $site);

=over 4

=item DESCRIPTION

To retrieve all information from the addresses table where the objid is the same
as the $custid object ID passed in.  Return the results in a hashref or undef if
nothing found.

=item INPUT

$custid - object id for the addresses

$dbh - current active database handle

$site - site hashref

=item OUTPUT

$sql_results[0] - Hashref containing the fields from the addresses select, or
    undef if nothing returned

=item NOTES

=back

=cut
###############################################################################
sub getAddressData {
    my $objid = shift;
    my $dbh = shift;
    my $site = shift;

    my ($select_hashref, @sql_results, $q, $num);

    $select_hashref->{'TABLE'}->{'NAME'} = "addresses";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $objid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $num = &query_getAll($q, $site, $dbh, \@sql_results);
    if ( $num == 1 ) {      # one order found...
	return ($sql_results[0]);
    }
    return undef;
}

##############################################################################
=pod

=head2 getOcartData

getOcartData - &getOcartData($objid, $dbh, $site, \@lineitems);

=over 4

=item DESCRIPTION

To retrieve all of the items that were purchased in the order.  Return the results 
in an array of hashref or undef if nothing found.

=item INPUT

$order - object id for the addresses

$dbh - current active database handle

$site - site hashref

$sql_results - Pointer to an array of hashrefs, which is used for the output.
    Each index will be a hashref describing one item in the order

=item OUTPUT

$num - Number of items in the order

$sql_results - Pointer to an array of hashrefs, each index a hashref 
               describing one item in the order

=item NOTES

=back

=cut
###############################################################################
sub getOcartData {
    my $order = shift;
    my $dbh = shift;
    my $site = shift;
    my $sql_results = shift;

    my ($select_hashref, $q, $num);

    $select_hashref->{'TABLE'}->{'NAME'} = "ocart";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("orderid", $order, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $num = &query_getAll($q, $site, $dbh, $sql_results);
    return $num;
}

##############################################################################
=pod

=head2 printXMLData

printXMLData - &printXMLData($xml_hashref, $site);

=over 4

=item DESCRIPTION

Taking all of the information required to create the XML file, output the 
results to a directory.  

=item INPUT

$xml_hashref - Hashref of information describing the order, in the format returned
               from getXMLData.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

The .xml files are generated on an order by order basis

=item NOTES

=back

=cut
###############################################################################
sub printXMLdata {
    my $xml_hashref = shift;
    my $site = shift;

    my ($objid, $key, $i, $line);
    
    my ($orderpage, $contactspage, $shiptopage, $billtopage, $ocartpage);
    my ($template_data, $template, $xmltemplate);
    my ($xmlpage, $output_string);
    $orderpage = &localize_template("xml_page_order", "xml", "0", $site);
    if ($orderpage =~ /^Missing/) {
	$orderpage = "";
    }
    $contactspage = &localize_template("xml_page_contacts", "xml", 0, $site);
    if ($contactspage =~ /^Missing/) {
	$contactspage = "";
    }
    $shiptopage = &localize_template("xml_page_shipto", "xml", 0, $site);
    if ($shiptopage =~ /^Missing/) {
	$shiptopage = "";
    }
    $billtopage = &localize_template("xml_page_billto", "xml", 0, $site);
    if ($billtopage =~ /^Missing/) {
	$billtopage = "";
    }
    $ocartpage = &localize_template("xml_page_ocart", "xml", 0, $site);
    if ($ocartpage =~ /^Missing/) {
	$ocartpage = "";
    }

    $xmlpage = &localize_template("xml_page_order_format", "xml", "0", $site);
    if ($xmlpage =~ /^Missing/) {
	$xmlpage = "";
    }
    foreach $objid (keys %{$xml_hashref}) {
	$template_data->{'ORDER'} = &permuteOrderData("ORDER", $xml_hashref->{$objid}->{'ORDER'}, $orderpage, $site);
	$template_data->{'CONTACTS'} = &permuteOrderData("CONTACTS", $xml_hashref->{$objid}->{'CONTACTS'}, $contactspage, $site);
	$template_data->{'SHIPTO'} = &permuteOrderData("SHIPTO", $xml_hashref->{$objid}->{'SHIPTO'}, $shiptopage, $site);
	$template_data->{'BILLTO'} = &permuteOrderData("BILLTO", $xml_hashref->{$objid}->{'BILLTO'}, $billtopage, $site);
	$template_data->{'OCART'} = &permuteOrderData("OCART", $xml_hashref->{$objid}->{'OCART'}, $ocartpage, $site);
	open FH, ">$site->{'targetpath'}/$objid.xml";
	if (length ($xmlpage) > 0) {
	    $output_string = permutebit($xmlpage, $template_data);
	    print FH $output_string;
	}
	else {
	    foreach $template (keys %{$template_data}) {
		print FH "$template_data->{$template}";
	    }
	}
	close FH;
    }
}

##############################################################################
=pod

=head2 permuteOrderData

permuteOrderData - &permuteOrderData();

=over 4

=item DESCRIPTION

Using the information in the $data_hashref, format the output.  If the 
$template passed in is not blank, then use it.  If it is blank, then
use the default of printing out everything in a very basic format.

The following templates will need to exist if you want to format the data:

/os_allcommerce/bits/eng/xml/pages/xml_page_order_format.txt
    Outlines the format of the final .xml file, can include the following tags:
         __OCART__
         __ORDER__
         __SHIPTO__
         __BILLTO__
         __CONTACTS__
    The values of the tags in the $xml_hashref hashref are used to permute the
    page.  

    If this file does not exist, it just outputs each key and all subkeys in any
    order to the output file.

/os_allcommerce/bits/eng/xml/pages/xml_page_contacts.txt
    Uses the data from $xml_hashref->{'CONTACTS'} to permute the output.
    If this file does not exist, it just outputs all fields in this hashref.

/os_allcommerce/bits/eng/xml/pages/xml_page_order.txt
    Uses the data from $xml_hashref->{'ORDER'} to permute the output.
    If this file does not exist, it just outputs all fields in this hashref.

/os_allcommerce/bits/eng/xml/pages/xml_page_shipto.txt
    Uses the data from $xml_hashref->{'SHIPTO'} to permute the output.
    If this file does not exist, it just outputs all fields in this hashref.

/os_allcommerce/bits/eng/xml/pages/xml_page_billto.txt
    Uses the data from $xml_hashref->{'BILLTO'} to permute the output.
    If this file does not exist, it just outputs all fields in this hashref.

/os_allcommerce/bits/eng/xml/pages/xml_page_ocart.txt
    Uses the data from $xml_hashref->{'OCART'} to permute the output.
    If this file does not exist, it just outputs all fields in this hashref, 
    separating each seperate item with <LINE ITEM> and </LINE ITEM>.

=item INPUT

$type - String representing what group this data belongs to:
         OCART, SHIPTO, BILLTO, CONTACTS, ORDER

$data_hashref - Information used to permute this group of data.

$template - Loaded template if one existed, otherwise ""

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$permute_string - String which is permuted if a template was passed in, or
                  the string which was defined if "" was passed in.

=item NOTES

=back

=cut
###############################################################################
sub permuteOrderData {
    my $type = shift;
    my $data_hashref = shift;
    my $template = shift;
    my $site = shift;

    my ($permute_string, $line, $key);
    
    $permute_string = "";
    if (length ($template) > 0) {  # there is a template so permute it and return the results
	if ($type eq "OCART") {
	    # Do it for each line item!
	    foreach $line (keys %{$data_hashref}) {
		$permute_string .= &permutebit($template, $data_hashref->{$line});
		$permute_string .= "\n\n";
	    }
	}
	else {
	    $permute_string = &permutebit($template, $data_hashref);
	}   
    }
    else {
	$permute_string = "<$type>\n";
	if ($type eq "OCART") {
	    foreach $line (keys %{$data_hashref}) {	
		$permute_string .= "\t<LINE ITEM>\n";
		foreach $key (keys %{$data_hashref->{$line}}) {	
		    if (length ($data_hashref->{$line}->{$key}) > 0) {
			$permute_string .= "\t\t<$key> $data_hashref->{$line}->{$key} </$key>\n";
		    }
		}
		$permute_string .= "\t<\/LINE ITEM>\n";
	    }	    
	}
	else {  # for all other types
	    foreach $key (keys %{$data_hashref}) {
		if (length ($data_hashref->{$key}) > 0) {
		    $permute_string .= "\t<$key> $data_hashref->{$key} </$key>\n";
		}
	    }
	}
	$permute_string .= "<\/$type>\n";
    }

    return $permute_string;
}
1;



